@@ -44,6 +44,7 @@
 #include "stack.h"
 #include "globals.h"
 #include "lkowner.h"
+#include "syscall.h"
 
 #ifndef AI_ADDRCONFIG
 #define AI_ADDRCONFIG 0
@@ -58,6 +59,89 @@ struct dnscache6 {
 };
 
 
+/* works similar to mkdir(1) -p.
+ */
+int
+mkdir_p (char *path, mode_t mode, gf_boolean_t allow_symlinks)
+{
+        int             i               = 0;
+        int             ret             = -1;
+        char            dir[PATH_MAX]   = {0,};
+        struct stat     stbuf           = {0,};
+
+        strcpy (dir, path);
+        i = (dir[0] == '/')? 1: 0;
+        do {
+                if (path[i] != '/' && path[i] != '\0')
+                        continue;
+
+                dir[i] = '\0';
+                ret = mkdir (dir, mode);
+                if (ret && errno != EEXIST) {
+                        gf_log ("", GF_LOG_ERROR, "Failed due to reason %s",
+                                strerror (errno));
+                        goto out;
+                }
+
+                if (ret && errno == EEXIST && !allow_symlinks) {
+                        ret = lstat (dir, &stbuf);
+                        if (ret)
+                                goto out;
+
+                        if (S_ISLNK (stbuf.st_mode)) {
+                                ret = -1;
+                                gf_log ("", GF_LOG_ERROR, "%s is a symlink",
+                                        dir);
+                                goto out;
+                        }
+                }
+                dir[i] = '/';
+
+        } while (path[i++] != '\0');
+
+        ret = stat (dir, &stbuf);
+        if (ret || !S_ISDIR (stbuf.st_mode)) {
+                ret = -1;
+                gf_log ("", GF_LOG_ERROR, "Failed to create directory, "
+                        "possibly some of the components were not directories");
+                goto out;
+        }
+
+        ret = 0;
+out:
+
+        return ret;
+}
+
+int
+gf_lstat_dir (const char *path, struct stat *stbuf_in)
+{
+        int ret           = -1;
+        struct stat stbuf = {0,};
+
+        if (path == NULL) {
+                errno = EINVAL;
+                goto out;
+        }
+
+        ret = sys_lstat (path, &stbuf);
+        if (ret)
+                goto out;
+
+        if (!S_ISDIR (stbuf.st_mode)) {
+                errno = ENOTDIR;
+                ret = -1;
+                goto out;
+        }
+        ret = 0;
+
+out:
+        if (!ret && stbuf_in)
+                *stbuf_in = stbuf;
+
+        return ret;
+}
+
 int
 log_base2 (unsigned long x)
 {
@@ -113,7 +197,9 @@ gf_resolve_ip6 (const char *hostname,
                 memset(&hints, 0, sizeof(hints));
                 hints.ai_family   = family;
                 hints.ai_socktype = SOCK_STREAM;
+#ifndef __NetBSD__
                 hints.ai_flags    = AI_ADDRCONFIG;
+#endif
 
                 ret = gf_asprintf (&port_str, "%d", port);
                 if (-1 == ret) {
@@ -185,24 +271,26 @@ err:
 void
 gf_log_volume_file (FILE *specfp)
 {
-        extern FILE *gf_log_logfile;
         int          lcount = 0;
         char         data[GF_UNIT_KB];
+        glusterfs_ctx_t *ctx;
+
+        ctx = THIS->ctx;
 
         fseek (specfp, 0L, SEEK_SET);
 
-        fprintf (gf_log_logfile, "Given volfile:\n");
-        fprintf (gf_log_logfile,
+        fprintf (ctx->log.gf_log_logfile, "Given volfile:\n");
+        fprintf (ctx->log.gf_log_logfile,
                  "+---------------------------------------"
                  "---------------------------------------+\n");
         while (fgets (data, GF_UNIT_KB, specfp) != NULL){
                 lcount++;
-                fprintf (gf_log_logfile, "%3d: %s", lcount, data);
+                fprintf (ctx->log.gf_log_logfile, "%3d: %s", lcount, data);
         }
-        fprintf (gf_log_logfile,
+        fprintf (ctx->log.gf_log_logfile,
                  "\n+---------------------------------------"
                  "---------------------------------------+\n");
-        fflush (gf_log_logfile);
+        fflush (ctx->log.gf_log_logfile);
         fseek (specfp, 0L, SEEK_SET);
 }
 
@@ -338,26 +426,31 @@ out:
 /* Obtain a backtrace and print it to stdout. */
 /* TODO: It looks like backtrace_symbols allocates memory,
    it may be problem because mostly memory allocation/free causes 'sigsegv' */
+
 void
-gf_print_trace (int32_t signum)
+gf_print_trace (int32_t signum, glusterfs_ctx_t *ctx)
 {
-        extern FILE *gf_log_logfile;
-        struct tm   *tm = NULL;
         char         msg[1024] = {0,};
-        char         timestr[256] = {0,};
-        time_t       utime = 0;
+        char         timestr[64] = {0,};
         int          ret = 0;
         int          fd = 0;
 
-        fd = fileno (gf_log_logfile);
+        fd = fileno (ctx->log.gf_log_logfile);
 
+        /* Now every gf_log call will just write to a buffer and when the
+         * buffer becomes full, its written to the log-file. Suppose the process
+         * crashes and prints the backtrace in the log-file, then the previous
+         * log information will still be in the buffer itself. So flush the
+         * contents of the buffer to the log file before printing the backtrace
+         * which helps in debugging.
+         */
+        fflush (ctx->log.gf_log_logfile);
         /* Pending frames, (if any), list them in order */
         ret = write (fd, "pending frames:\n", 16);
         if (ret < 0)
                 goto out;
 
         {
-                glusterfs_ctx_t *ctx = glusterfs_ctx_get ();
                 struct list_head *trav = ((call_pool_t *)ctx->pool)->all_frames.next;
                 while (trav != (&((call_pool_t *)ctx->pool)->all_frames)) {
                         call_frame_t *tmp = (call_frame_t *)(&((call_stack_t *)trav)->frames);
@@ -365,10 +458,10 @@ gf_print_trace (int32_t signum)
                                 sprintf (msg,"frame : type(%d) op(%s)\n",
                                          tmp->root->type,
                                          gf_fop_list[tmp->root->op]);
-                        if (tmp->root->type == GF_OP_TYPE_MGMT)
-                                sprintf (msg,"frame : type(%d) op(%s)\n",
+                        else
+                                sprintf (msg,"frame : type(%d) op(%d)\n",
                                          tmp->root->type,
-                                         gf_mgmt_list[tmp->root->op]);
+                                         tmp->root->op);
 
                         ret = write (fd, msg, strlen (msg));
                         if (ret < 0)
@@ -394,9 +487,7 @@ gf_print_trace (int32_t signum)
         {
                 /* Dump the timestamp of the crash too, so the previous logs
                    can be related */
-                utime = time (NULL);
-                tm    = localtime (&utime);
-                strftime (timestr, 256, "%Y-%m-%d %H:%M:%S\n", tm);
+                gf_time_fmt (timestr, sizeof timestr, time (NULL), gf_timefmt_FT);
                 ret = write (fd, "time of crash: ", 15);
                 if (ret < 0)
                         goto out;
@@ -552,8 +643,7 @@ gf_strstr (const char *str, const char *delim, const char *match)
         }
 
 out:
-        if (tmp_str)
-                free (tmp_str);
+        free (tmp_str);
 
         return ret;
 
@@ -606,6 +696,8 @@ gf_string2time (const char *str, uint32_t *n)
         old_errno = errno;
         errno = 0;
         value = strtol (str, &tail, 0);
+        if (str == tail)
+                errno = EINVAL;
 
         if (errno == ERANGE || errno == EINVAL)
                 return -1;
@@ -624,11 +716,10 @@ gf_string2time (const char *str, uint32_t *n)
         return 0;
 }
 
-
 int
-gf_string2percent (const char *str, uint32_t *n)
+gf_string2percent (const char *str, double *n)
 {
-        unsigned long value = 0;
+        double value = 0;
         char *tail = NULL;
         int old_errno = 0;
         const char *s = NULL;
@@ -649,7 +740,9 @@ gf_string2percent (const char *str, uint32_t *n)
 
         old_errno = errno;
         errno = 0;
-        value = strtol (str, &tail, 0);
+        value = strtod (str, &tail);
+        if (str == tail)
+                errno = EINVAL;
 
         if (errno == ERANGE || errno == EINVAL)
                 return -1;
@@ -683,6 +776,8 @@ _gf_string2long (const char *str, long *n, int base)
         old_errno = errno;
         errno = 0;
         value = strtol (str, &tail, base);
+        if (str == tail)
+                errno = EINVAL;
 
         if (errno == ERANGE || errno == EINVAL)
                 return -1;
@@ -723,6 +818,8 @@ _gf_string2ulong (const char *str, unsigned long *n, int base)
         old_errno = errno;
         errno = 0;
         value = strtoul (str, &tail, base);
+        if (str == tail)
+                errno = EINVAL;
 
         if (errno == ERANGE || errno == EINVAL)
                 return -1;
@@ -763,6 +860,8 @@ _gf_string2uint (const char *str, unsigned int *n, int base)
         old_errno = errno;
         errno = 0;
         value = strtoul (str, &tail, base);
+        if (str == tail)
+                errno = EINVAL;
 
         if (errno == ERANGE || errno == EINVAL)
                 return -1;
@@ -794,6 +893,8 @@ _gf_string2double (const char *str, double *n)
         old_errno = errno;
         errno = 0;
         value = strtod (str, &tail);
+        if (str == tail)
+                errno = EINVAL;
 
         if (errno == ERANGE || errno == EINVAL)
                 return -1;
@@ -825,6 +926,8 @@ _gf_string2longlong (const char *str, long long *n, int base)
         old_errno = errno;
         errno = 0;
         value = strtoll (str, &tail, base);
+        if (str == tail)
+                errno = EINVAL;
 
         if (errno == ERANGE || errno == EINVAL)
                 return -1;
@@ -865,6 +968,8 @@ _gf_string2ulonglong (const char *str, unsigned long long *n, int base)
         old_errno = errno;
         errno = 0;
         value = strtoull (str, &tail, base);
+        if (str == tail)
+                errno = EINVAL;
 
         if (errno == ERANGE || errno == EINVAL)
                 return -1;
@@ -1207,7 +1312,7 @@ err:
 int
 gf_string2bytesize (const char *str, uint64_t *n)
 {
-        uint64_t value = 0ULL;
+        double value = 0.0;
         char *tail = NULL;
         int old_errno = 0;
         const char *s = NULL;
@@ -1228,7 +1333,9 @@ gf_string2bytesize (const char *str, uint64_t *n)
 
         old_errno = errno;
         errno = 0;
-        value = strtoull (str, &tail, 10);
+        value = strtod (str, &tail);
+        if (str == tail)
+                errno = EINVAL;
 
         if (errno == ERANGE || errno == EINVAL)
                 return -1;
@@ -1252,7 +1359,7 @@ gf_string2bytesize (const char *str, uint64_t *n)
                         return -1;
         }
 
-        *n = value;
+        *n = (uint64_t) value;
 
         return 0;
 }
@@ -1285,6 +1392,8 @@ gf_string2percent_or_bytesize (const char *str,
         old_errno = errno;
         errno = 0;
         value = strtoull (str, &tail, 10);
+        if (str == tail)
+                errno = EINVAL;
 
         if (errno == ERANGE || errno == EINVAL)
                 return -1;
@@ -1575,11 +1684,9 @@ get_nth_word (const char *str, int n)
         return word;
 }
 
-/* RFC 1123 & 952 */
-/* Syntax formed combining RFC 1123 & 952                                     *
-   <hname> ::= <first-name>*["."<gen-name>]                                   *
-   <first-name> ::= <let-or-digit> <[*[<let-or-digit-or-hyphen>]<let-or-digit>]
-   <gen-name> ::= <let>[*[<let-or-digit-or-hyphen>]<let-or-digit>]            */
+/* Syntax formed according to RFC 1912 (RFC 1123 & 952 are more restrictive)  *
+   <hname> ::= <gen-name>*["."<gen-name>]                                     *
+   <gen-name> ::= <let-or-digit> <[*[<let-or-digit-or-hyphen>]<let-or-digit>] */
 char
 valid_host_name (char *address, int length)
 {
@@ -1600,27 +1707,13 @@ valid_host_name (char *address, int length)
                 ret = 0;
                 goto out;
         }
-        temp_str = strtok_r (dup_addr,".", &save_ptr);
-
-        /* first-name */
-        if (!temp_str ||
-            !isalnum(temp_str[0]) ||
-            !isalnum (temp_str[strlen(temp_str)-1])) {
-                ret = 0;
-                goto out;
-        }
-        for (i = 1; i < (strlen (temp_str) - 1); i++) {
-                if (!isalnum (temp_str[i]) && (temp_str[i] != '-')) {
-                        ret = 0;
-                        goto out;
-                }
-        }
 
         /* gen-name */
-        while ((temp_str = strtok_r (NULL, ".", &save_ptr))) {
+        temp_str = strtok_r (dup_addr, ".", &save_ptr);
+        do {
                 str_len = strlen (temp_str);
 
-                if (!isalpha (temp_str[0]) ||
+                if (!isalnum (temp_str[0]) ||
                     !isalnum (temp_str[str_len-1])) {
                         ret = 0;
                         goto out;
@@ -1631,11 +1724,10 @@ valid_host_name (char *address, int length)
                                 goto out;
                         }
                 }
-        }
+        } while ((temp_str = strtok_r (NULL, ".", &save_ptr)));
 
 out:
-        if (dup_addr)
-                GF_FREE (dup_addr);
+        GF_FREE (dup_addr);
         return ret;
 }
 
@@ -1653,7 +1745,8 @@ valid_ipv4_address (char *address, int length, gf_boolean_t wildcard_acc)
         tmp = gf_strdup (address);
 
         /* To prevent cases where last character is '.' */
-        if (!isdigit (tmp[length - 1]) && (tmp[length - 1] != '*')) {
+        if (length <= 0 ||
+            (!isdigit (tmp[length - 1]) && (tmp[length - 1] != '*'))) {
                 ret = 0;
                 goto out;
         }
@@ -1699,7 +1792,7 @@ valid_ipv6_address (char *address, int length, gf_boolean_t wildcard_acc)
         tmp = gf_strdup (address);
 
         /* Check for compressed form */
-        if (tmp[length - 1] == ':') {
+        if (length <= 0 || tmp[length - 1] == ':') {
                 ret = 0;
                 goto out;
         }
@@ -1769,7 +1862,7 @@ out:
 char *
 uuid_utoa (uuid_t uuid)
 {
-        char *uuid_buffer = glusterfs_uuid_buf_get();
+        char *uuid_buffer = glusterfs_uuid_buf_get(THIS->ctx);
         uuid_unparse (uuid, uuid_buffer);
         return uuid_buffer;
 }
@@ -1788,7 +1881,7 @@ uuid_utoa_r (uuid_t uuid, char *dst)
 char *
 lkowner_utoa (gf_lkowner_t *lkowner)
 {
-        char *lkowner_buffer = glusterfs_lkowner_buf_get();
+        char *lkowner_buffer = glusterfs_lkowner_buf_get(THIS->ctx);
         lkowner_unparse (lkowner, lkowner_buffer, GF_LKOWNER_BUF_SIZE);
         return lkowner_buffer;
 }
@@ -1865,8 +1958,7 @@ gf_is_str_int (const char *value)
         }
 
 out:
-        if (fptr)
-                GF_FREE (fptr);
+        GF_FREE (fptr);
 
         return flag;
 }
@@ -1876,9 +1968,9 @@ out:
  */
 
 inline int32_t
-gf_roundup_power_of_two (uint32_t nr)
+gf_roundup_power_of_two (int32_t nr)
 {
-        uint32_t result = 1;
+        int32_t result = 1;
 
         if (nr < 0) {
                 gf_log ("common-utils", GF_LOG_WARNING,
@@ -1894,18 +1986,13 @@ out:
         return result;
 }
 
-/*
- * rounds up nr to next power of two. If nr is already a power of two, next
- * power of two is returned.
- */
-
 /*
  * rounds up nr to next power of two. If nr is already a power of two, next
  * power of two is returned.
  */
 
 inline int32_t
-gf_roundup_next_power_of_two (uint32_t nr)
+gf_roundup_next_power_of_two (int32_t nr)
 {
         int32_t result = 1;
 
@@ -2038,3 +2125,310 @@ gf_strip_whitespace (char *str, int len)
         GF_FREE (new_str);
         return new_len;
 }
+
+int
+gf_canonicalize_path (char *path)
+{
+        int             ret                  = -1;
+        int             path_len             = 0;
+        int             dir_path_len         = 0;
+        char           *tmppath              = NULL;
+        char           *dir                  = NULL;
+        char           *tmpstr               = NULL;
+
+        if (!path || *path != '/')
+                goto out;
+
+        tmppath = gf_strdup (path);
+        if (!tmppath)
+                goto out;
+
+        /* Strip the extra slashes and return */
+        bzero (path, strlen(path));
+        path[0] = '/';
+        dir = strtok_r(tmppath, "/", &tmpstr);
+
+        while (dir) {
+                dir_path_len = strlen(dir);
+                strncpy ((path + path_len + 1), dir, dir_path_len);
+                path_len += dir_path_len + 1;
+                dir = strtok_r (NULL, "/", &tmpstr);
+                if (dir)
+                        strncpy ((path + path_len), "/", 1);
+        }
+        path[path_len] = '\0';
+        ret = 0;
+
+ out:
+        if (ret)
+                gf_log ("common-utils", GF_LOG_ERROR,
+                        "Path manipulation failed");
+
+        GF_FREE(tmppath);
+
+        return ret;
+}
+
+static const char *__gf_timefmts[] = {
+        "%F %T",
+        "%Y/%m/%d-%T",
+        "%b %d %T",
+        "%F %H%M%S"
+};
+
+static const char *__gf_zerotimes[] = {
+        "0000-00-00 00:00:00",
+        "0000/00/00-00:00:00",
+        "xxx 00 00:00:00",
+        "0000-00-00 000000"
+};
+
+void
+_gf_timestuff (gf_timefmts *fmt, const char ***fmts, const char ***zeros)
+{
+        *fmt = gf_timefmt_last;
+        *fmts = __gf_timefmts;
+        *zeros = __gf_zerotimes;
+}
+
+
+char *
+generate_glusterfs_ctx_id (void)
+{
+        char           tmp_str[1024] = {0,};
+        char           hostname[256] = {0,};
+        struct timeval tv = {0,};
+        char           now_str[32];
+
+        if (gettimeofday (&tv, NULL) == -1) {
+                gf_log ("glusterfsd", GF_LOG_ERROR,
+                        "gettimeofday: failed %s",
+                        strerror (errno));
+        }
+
+        if (gethostname (hostname, 256) == -1) {
+                gf_log ("glusterfsd", GF_LOG_ERROR,
+                        "gethostname: failed %s",
+                        strerror (errno));
+        }
+
+        gf_time_fmt (now_str, sizeof now_str, tv.tv_sec, gf_timefmt_Ymd_T);
+        snprintf (tmp_str, sizeof tmp_str, "%s-%d-%s:%"
+#ifdef GF_DARWIN_HOST_OS
+                  PRId32,
+#else
+                  "ld",
+#endif
+                  hostname, getpid(), now_str, tv.tv_usec);
+
+        return gf_strdup (tmp_str);
+}
+
+char *
+gf_get_reserved_ports ()
+{
+        char    *ports_info  = NULL;
+#if defined GF_LINUX_HOST_OS
+        int     proc_fd      = -1;
+        char    *proc_file   = "/proc/sys/net/ipv4/ip_local_reserved_ports";
+        char    buffer[4096] = {0,};
+        int32_t ret          = -1;
+
+        proc_fd = open (proc_file, O_RDONLY);
+        if (proc_fd == -1) {
+                /* What should be done in this case? error out from here
+                 * and thus stop the glusterfs process from starting or
+                 * continue with older method of using any of the available
+                 * port? For now 2nd option is considered.
+                 */
+                gf_log ("glusterfs", GF_LOG_WARNING, "could not open "
+                        "the file /proc/sys/net/ipv4/ip_local_reserved_ports "
+                        "for getting reserved ports info (%s)",
+                        strerror (errno));
+                goto out;
+        }
+
+        ret = read (proc_fd, buffer, sizeof (buffer));
+        if (ret < 0) {
+                gf_log ("glusterfs", GF_LOG_WARNING, "could not "
+                        "read the file %s for getting reserved ports "
+                        "info (%s)", proc_file, strerror (errno));
+                goto out;
+        }
+        ports_info = gf_strdup (buffer);
+
+out:
+        if (proc_fd != -1)
+                close (proc_fd);
+#endif /* GF_LINUX_HOST_OS */
+        return ports_info;
+}
+
+int
+gf_process_reserved_ports (gf_boolean_t *ports)
+{
+        int      ret         = -1;
+#if defined GF_LINUX_HOST_OS
+        char    *ports_info  = NULL;
+        char    *tmp         = NULL;
+        char    *blocked_port = NULL;
+
+        ports_info = gf_get_reserved_ports ();
+        if (!ports_info) {
+                gf_log ("glusterfs", GF_LOG_WARNING, "Not able to get reserved "
+                        "ports, hence there is a possibility that glusterfs "
+                        "may consume reserved port");
+                goto out;
+        }
+
+        blocked_port = strtok_r (ports_info, ",\n",&tmp);
+
+        while (blocked_port) {
+                gf_ports_reserved (blocked_port, ports);
+                blocked_port = strtok_r (NULL, ",\n", &tmp);
+        }
+
+        ret = 0;
+
+out:
+        GF_FREE (ports_info);
+#endif /* GF_LINUX_HOST_OS */
+        return ret;
+}
+
+gf_boolean_t
+gf_ports_reserved (char *blocked_port, gf_boolean_t *ports)
+{
+        gf_boolean_t    result   = _gf_false;
+        char            *range_port = NULL;
+        int16_t         tmp_port1, tmp_port2 = -1;
+
+        if (strstr (blocked_port, "-") == NULL) {
+                /* get rid of the new line character*/
+                if (blocked_port[strlen(blocked_port) -1] == '\n')
+                        blocked_port[strlen(blocked_port) -1] = '\0';
+                if (gf_string2int16 (blocked_port, &tmp_port1) == 0) {
+                        if (tmp_port1 > (GF_CLIENT_PORT_CEILING - 1)
+                            || tmp_port1 < 0) {
+                                gf_log ("glusterfs-socket", GF_LOG_WARNING,
+                                        "invalid port %d", tmp_port1);
+                                result = _gf_true;
+                                goto out;
+                        } else {
+                                gf_log ("glusterfs", GF_LOG_DEBUG,
+                                        "blocking port %d", tmp_port1);
+                                ports[tmp_port1] = _gf_true;
+                        }
+                } else {
+                        gf_log ("glusterfs-socket", GF_LOG_WARNING, "%s is "
+                                "not a valid port identifier", blocked_port);
+                        result = _gf_true;
+                        goto out;
+                }
+        } else {
+                range_port = strtok (blocked_port, "-");
+                if (!range_port){
+                        result = _gf_true;
+                        goto out;
+                }
+                if (gf_string2int16 (range_port, &tmp_port1) == 0) {
+                        if (tmp_port1 > (GF_CLIENT_PORT_CEILING - 1))
+                                tmp_port1 = GF_CLIENT_PORT_CEILING - 1;
+                        if (tmp_port1 < 0)
+                                tmp_port1 = 0;
+                }
+                range_port = strtok (NULL, "-");
+                if (!range_port) {
+                        result = _gf_true;
+                        goto out;
+                }
+                /* get rid of the new line character*/
+                if (range_port[strlen(range_port) -1] == '\n')
+                        range_port[strlen(range_port) - 1] = '\0';
+                if (gf_string2int16 (range_port, &tmp_port2) == 0) {
+                        if (tmp_port2 >
+                            (GF_CLIENT_PORT_CEILING - 1))
+                                tmp_port2 = GF_CLIENT_PORT_CEILING - 1;
+                        if (tmp_port2 < 0)
+                                tmp_port2 = 0;
+                }
+                gf_log ("glusterfs", GF_LOG_DEBUG, "lower: %d, higher: %d",
+                        tmp_port1, tmp_port2);
+                for (; tmp_port1 <= tmp_port2; tmp_port1++)
+                        ports[tmp_port1] = _gf_true;
+        }
+
+out:
+        return result;
+}
+
+/* Takes in client ip{v4,v6} and returns associated hostname, if any
+ * Also, allocates memory for the hostname.
+ * Returns: 0 for success, -1 for failure
+ */
+int
+gf_get_hostname_from_ip (char *client_ip, char **hostname)
+{
+        int                      ret                          = -1;
+        struct sockaddr         *client_sockaddr              = NULL;
+        struct sockaddr_in       client_sock_in               = {0};
+        struct sockaddr_in6      client_sock_in6              = {0};
+        char                     client_hostname[NI_MAXHOST]  = {0};
+        char                    *client_ip_copy               = NULL;
+        char                    *tmp                          = NULL;
+        char                    *ip                           = NULL;
+
+        /* if ipv4, reverse lookup the hostname to
+         * allow FQDN based rpc authentication
+         */
+        if (valid_ipv4_address (client_ip, strlen (client_ip), 0) == _gf_false) {
+                /* most times, we get a.b.c.d:port form, so check that */
+                client_ip_copy = gf_strdup (client_ip);
+                if (!client_ip_copy)
+                        goto out;
+
+                ip = strtok_r (client_ip_copy, ":", &tmp);
+        } else {
+                ip = client_ip;
+        }
+
+        if (valid_ipv4_address (ip, strlen (ip), 0) == _gf_true) {
+                client_sockaddr = (struct sockaddr *)&client_sock_in;
+                client_sock_in.sin_family = AF_INET;
+                ret = inet_pton (AF_INET, ip,
+                                 (void *)&client_sock_in.sin_addr.s_addr);
+
+        } else if (valid_ipv6_address (ip, strlen (ip), 0) == _gf_true) {
+                client_sockaddr = (struct sockaddr *) &client_sock_in6;
+
+                client_sock_in6.sin6_family = AF_INET6;
+                ret = inet_pton (AF_INET6, ip,
+                                 (void *)&client_sock_in6.sin6_addr);
+        } else {
+                goto out;
+        }
+
+        if (ret != 1) {
+                ret = -1;
+                goto out;
+        }
+
+        ret = getnameinfo (client_sockaddr,
+                           sizeof (*client_sockaddr),
+                           client_hostname, sizeof (client_hostname),
+                           NULL, 0, 0);
+        if (ret) {
+                gf_log ("common-utils", GF_LOG_ERROR,
+                        "Could not lookup hostname of %s : %s",
+                        client_ip, gai_strerror (ret));
+                ret = -1;
+                goto out;
+        }
+
+        *hostname = gf_strdup ((char *)client_hostname);
+ out:
+        if (client_ip_copy)
+                GF_FREE (client_ip_copy);
+
+        return ret;
+}
