@@ -344,6 +344,9 @@ char *dirname_r(char *path);
 #define dirname(path) dirname_r(path)
 #endif /* THREAD_UNSAFE_DIRNAME */
 
+int gf_mkostemp (char *tmpl, int suffixlen, int flags);
+#define mkostemp(tmpl, flags) gf_mkostemp(tmpl, 0, flags);
+
 #ifdef HAVE_STRUCT_STAT_ST_ATIM_TV_NSEC
 /* Linux, Solaris, Cygwin */
 #define ST_ATIM_NSEC(stbuf) ((stbuf)->st_atim.tv_nsec)
