@@ -121,7 +121,7 @@ gf_fd_fdtable_alloc (void)
 }
 
 
-fdentry_t *
+static fdentry_t *
 __gf_fd_fdtable_get_all_fds (fdtable_t *fdtable, uint32_t *count)
 {
         fdentry_t       *fdentries = NULL;
@@ -159,7 +159,7 @@ gf_fd_fdtable_get_all_fds (fdtable_t *fdtable, uint32_t *count)
 }
 
 
-fdentry_t *
+static fdentry_t *
 __gf_fd_fdtable_copy_all_fds (fdtable_t *fdtable, uint32_t *count)
 {
         fdentry_t *fdentries = NULL;
@@ -513,6 +513,11 @@ fd_destroy (fd_t *fd)
         LOCK_DESTROY (&fd->lock);
 
         GF_FREE (fd->_ctx);
+        LOCK (&fd->inode->lock);
+        {
+                fd->inode->fd_count--;
+        }
+        UNLOCK (&fd->inode->lock);
         inode_unref (fd->inode);
         fd->inode = (inode_t *)0xaaaaaaaa;
         fd_lk_ctx_unref (fd->lk_ctx);
@@ -552,6 +557,7 @@ __fd_bind (fd_t *fd)
 {
         list_del_init (&fd->inode_list);
         list_add (&fd->inode_list, &fd->inode->fd_list);
+        fd->inode->fd_count++;
 
         return fd;
 }
@@ -659,6 +665,12 @@ __fd_lookup (inode_t *inode, uint64_t pid)
 
 
         list_for_each_entry (iter_fd, &inode->fd_list, inode_list) {
+                if (iter_fd->anonymous)
+                        /* If someone was interested in getting an
+                           anonymous fd (or was OK getting an anonymous fd),
+                           they can as well call fd_anonymous() directly */
+                        continue;
+
                 if (!pid || iter_fd->pid == pid) {
                         fd = __fd_ref (iter_fd);
                         break;
@@ -707,24 +719,44 @@ fd_lookup_uint64 (inode_t *inode, uint64_t pid)
         return fd;
 }
 
+static fd_t *
+__fd_lookup_anonymous (inode_t *inode)
+{
+        fd_t *iter_fd = NULL;
+        fd_t *fd = NULL;
 
-fd_t *
+        if (list_empty (&inode->fd_list))
+                return NULL;
+
+        list_for_each_entry (iter_fd, &inode->fd_list, inode_list) {
+                if (iter_fd->anonymous) {
+                        fd = __fd_ref (iter_fd);
+                        break;
+                }
+        }
+
+        return fd;
+}
+
+static fd_t *
 __fd_anonymous (inode_t *inode)
 {
         fd_t *fd = NULL;
 
-        fd = __fd_lookup (inode, (uint64_t)-1);
+        fd = __fd_lookup_anonymous (inode);
 
         /* if (fd); then we already have increased the refcount in
-           __fd_lookup(), so no need of one more fd_ref().
+           __fd_lookup_anonymous(), so no need of one more fd_ref().
            if (!fd); then both create and bind wont bump up the ref
            count, so we have to call fd_ref() after bind. */
         if (!fd) {
-                fd = __fd_create (inode, (uint64_t)-1);
+                fd = __fd_create (inode, 0);
 
                 if (!fd)
                         return NULL;
 
+                fd->anonymous = _gf_true;
+
                 __fd_bind (fd);
 
                 __fd_ref (fd);
@@ -748,11 +780,28 @@ fd_anonymous (inode_t *inode)
         return fd;
 }
 
+fd_t*
+fd_lookup_anonymous (inode_t *inode)
+{
+        fd_t *fd = NULL;
+
+        if (!inode) {
+                gf_log_callingfn ("fd", GF_LOG_WARNING, "!inode");
+                return NULL;
+        }
+
+        LOCK (&inode->lock);
+        {
+                fd = __fd_lookup_anonymous (inode);
+        }
+        UNLOCK (&inode->lock);
+        return fd;
+}
 
 gf_boolean_t
 fd_is_anonymous (fd_t *fd)
 {
-        return (fd && fd->pid == -1);
+        return (fd && fd->anonymous);
 }
 
 
@@ -960,6 +1009,13 @@ fd_dump (fd_t *fd, char *prefix)
         gf_proc_dump_write("pid", "%llu", fd->pid);
         gf_proc_dump_write("refcount", "%d", fd->refcount);
         gf_proc_dump_write("flags", "%d", fd->flags);
+
+        if (fd->inode) {
+                gf_proc_dump_build_key (key, "inode", NULL);
+                gf_proc_dump_add_section(key);
+                inode_dump (fd->inode, key);
+        }
+
 }
 
 
@@ -1061,9 +1117,7 @@ unlock:
         }
 
 out:
-        if (fd_ctx != NULL) {
-                GF_FREE (fd_ctx);
-        }
+        GF_FREE (fd_ctx);
 
         return;
 }
