@@ -32,10 +32,11 @@
 #include <arpa/inet.h>
 #include <sys/poll.h>
 #include <pthread.h>
-
+#include <limits.h> /* For PATH_MAX */
 
 #include "list.h"
 #include "logging.h"
+#include "lkowner.h"
 
 #define GF_YES 1
 #define GF_NO  0
@@ -72,14 +73,18 @@
 #define ZR_MOUNTPOINT_OPT       "mountpoint"
 #define ZR_ATTR_TIMEOUT_OPT     "attribute-timeout"
 #define ZR_ENTRY_TIMEOUT_OPT    "entry-timeout"
+#define ZR_NEGATIVE_TIMEOUT_OPT "negative-timeout"
 #define ZR_DIRECT_IO_OPT        "direct-io-mode"
 #define ZR_STRICT_VOLFILE_CHECK "strict-volfile-check"
 #define ZR_DUMP_FUSE            "dump-fuse"
+#define ZR_FUSE_MOUNTOPTS       "fuse-mountopts"
 
 #define GF_XATTR_CLRLK_CMD      "glusterfs.clrlk"
 #define GF_XATTR_PATHINFO_KEY   "trusted.glusterfs.pathinfo"
 #define GF_XATTR_NODE_UUID_KEY  "trusted.glusterfs.node-uuid"
 #define GF_XATTR_VOL_ID_KEY   "trusted.glusterfs.volume-id"
+#define GF_XATTR_LOCKINFO_KEY   "trusted.glusterfs.lockinfo"
+#define GF_XATTR_GET_REAL_FILENAME_KEY "user.glusterfs.get_real_filename:"
 
 #define GF_READDIR_SKIP_DIRS       "readdir-filter-directories"
 
@@ -87,6 +92,8 @@
                                         strlen (GF_XATTR_PATHINFO_KEY)) == 0)
 #define XATTR_IS_NODE_UUID(x) (strncmp (x, GF_XATTR_NODE_UUID_KEY,      \
                                         strlen (GF_XATTR_NODE_UUID_KEY)) == 0)
+#define XATTR_IS_LOCKINFO(x) (strncmp (x, GF_XATTR_LOCKINFO_KEY,        \
+                                       strlen (GF_XATTR_LOCKINFO_KEY)) == 0)
 
 #define GF_XATTR_LINKINFO_KEY   "trusted.distribute.linkinfo"
 #define GFID_XATTR_KEY "trusted.gfid"
@@ -129,11 +136,19 @@
 #define ZR_FILE_CONTENT_REQUEST(key) (!strncmp(key, ZR_FILE_CONTENT_STR, \
 					       ZR_FILE_CONTENT_STRLEN))
 
+#define DEFAULT_VAR_RUN_DIRECTORY    DATADIR "/run/gluster"
+
 /* GlusterFS's maximum supported Auxilary GIDs */
 /* TODO: Keeping it to 200, so that we can fit in 2KB buffer for auth data
  * in RPC server code, if there is ever need for having more aux-gids, then
  * we have to add aux-gid in payload of actors */
-#define GF_MAX_AUX_GROUPS   200
+#define GF_MAX_AUX_GROUPS   65536
+
+#define GF_UUID_BUF_SIZE 50
+
+#define GF_REBALANCE_TID_KEY     "rebalance-id"
+#define GF_REMOVE_BRICK_TID_KEY  "remove-brick-id"
+#define GF_REPLACE_BRICK_TID_KEY "replace-brick-id"
 
 /* NOTE: add members ONLY at the end (just before _MAXVALUE) */
 typedef enum {
@@ -246,7 +261,9 @@ typedef enum {
 
 typedef enum {
 	GF_XATTROP_ADD_ARRAY,
-        GF_XATTROP_ADD_ARRAY64
+        GF_XATTROP_ADD_ARRAY64,
+        GF_XATTROP_OR_ARRAY,
+        GF_XATTROP_AND_ARRAY
 } gf_xattrop_flags_t;
 
 
@@ -255,12 +272,6 @@ typedef enum {
 #define GF_SET_DIR_ONLY       0x4
 #define GF_SET_EPOCH_TIME     0x8 /* used by afr dir lookup selfheal */
 
-/* Directory into which replicate self-heal will move deleted files and
-   directories into. The storage/posix janitor thread will periodically
-   clean up this directory */
-
-#define GF_REPLICATE_TRASH_DIR          ".landfill"
-
 /* key value which quick read uses to get small files in lookup cbk */
 #define GF_CONTENT_KEY "glusterfs.content"
 
@@ -297,14 +308,19 @@ struct _cmd_args {
         int              read_only;
         int              acl;
         int              selinux;
+        int              enable_ino32;
         int              worm;
         int              mac_compat;
+	int		 fopen_keep_cache;
+	int		 gid_timeout;
 	struct list_head xlator_options;  /* list of xlator_option_t */
 
 	/* fuse options */
 	int              fuse_direct_io_mode;
+	char             *use_readdirp;
         int              volfile_check;
 	double           fuse_entry_timeout;
+	double           fuse_negative_timeout;
 	double           fuse_attribute_timeout;
 	char            *volume_name;
 	int              fuse_nodev;
@@ -313,7 +329,9 @@ struct _cmd_args {
         pid_t            client_pid;
         int              client_pid_set;
         unsigned         uid_map_root;
-
+        int              background_qlen;
+        int              congestion_threshold;
+        char            *fuse_mountopts;
 
 	/* key args */
 	char            *mount_point;
@@ -381,17 +399,18 @@ struct _glusterfs_ctx {
         struct mem_pool    *dict_pair_pool;
         struct mem_pool    *dict_data_pool;
 
-        int                 mem_accounting; /* if value is other than 0, it
-                                               will be set */
         glusterfsd_mgmt_event_notify_fn_t notify; /* Used for xlators to make
                                                      call to fsd-mgmt */
+        gf_log_handle_t     log; /* all logging related variables */
+
+        int           mem_acct_enable;
+
+        int                 daemon_pipe[2];
 };
 typedef struct _glusterfs_ctx glusterfs_ctx_t;
 
+glusterfs_ctx_t *glusterfs_ctx_new (void);
 
-/* If you edit this structure then, make a corresponding change in
- * globals.c in the eventstring.
- */
 typedef enum {
         GF_EVENT_PARENT_UP = 1,
         GF_EVENT_POLLIN,
@@ -413,9 +432,6 @@ typedef enum {
         GF_EVENT_MAXVAL,
 } glusterfs_event_t;
 
-/* gf_lkowner_t is defined in lkowner.h */
-#include "lkowner.h"
-
 struct gf_flock {
         short        l_type;
         short        l_whence;
@@ -425,8 +441,6 @@ struct gf_flock {
         gf_lkowner_t l_owner;
 };
 
-extern char *glusterfs_strevent (glusterfs_event_t ev);
-
 #define GF_MUST_CHECK __attribute__((warn_unused_result))
 /*
  * Some macros (e.g. ALLOC_OR_GOTO) set variables in function scope, but the
