@@ -134,8 +134,7 @@ __dentry_unset (dentry_t *dentry)
 
         list_del_init (&dentry->inode_list);
 
-        if (dentry->name)
-                GF_FREE (dentry->name);
+        GF_FREE (dentry->name);
 
         if (dentry->parent) {
                 __inode_unref (dentry->parent);
@@ -311,7 +310,7 @@ __inode_destroy (inode_t *inode)
                 goto noctx;
         }
 
-        for (index = 0; index < inode->table->xl->graph->xl_count; index++) {
+        for (index = 0; index < inode->table->ctxcount; index++) {
                 if (inode->_ctx[index].xl_key) {
                         xl = (xlator_t *)(long)inode->_ctx[index].xl_key;
                         old_THIS = THIS;
@@ -529,8 +528,7 @@ __inode_create (inode_table_t *table)
         INIT_LIST_HEAD (&newi->hash);
         INIT_LIST_HEAD (&newi->dentry_list);
 
-        newi->_ctx = GF_CALLOC (1, (sizeof (struct _inode_ctx) *
-                                    table->xl->graph->xl_count),
+        newi->_ctx = GF_CALLOC (1, (sizeof (struct _inode_ctx) * table->ctxcount),
                                 gf_common_mt_inode_ctx);
 
         if (newi->_ctx == NULL) {
@@ -830,14 +828,13 @@ __inode_link (inode_t *inode, inode_t *parent, const char *name,
                 if (uuid_is_null (iatt->ia_gfid))
                         return NULL;
 
-                uuid_copy (inode->gfid, iatt->ia_gfid);
-                inode->ia_type    = iatt->ia_type;
-
-                old_inode = __inode_find (table, inode->gfid);
+                old_inode = __inode_find (table, iatt->ia_gfid);
 
                 if (old_inode) {
                         link_inode = old_inode;
                 } else {
+                        uuid_copy (inode->gfid, iatt->ia_gfid);
+                        inode->ia_type    = iatt->ia_type;
                         __inode_hash (inode);
                 }
         }
@@ -950,6 +947,54 @@ inode_forget (inode_t *inode, uint64_t nlookup)
         return 0;
 }
 
+/*
+ * Invalidate an inode. This is invoked when a translator decides that an inode's
+ * cache is no longer valid. Any translator interested in taking action in this
+ * situation can define the invalidate callback.
+ */
+int
+inode_invalidate(inode_t *inode)
+{
+	int ret = 0;
+	xlator_t *xl = NULL;
+	xlator_t *old_THIS = NULL;
+
+	if (!inode) {
+		gf_log_callingfn(THIS->name, GF_LOG_WARNING, "inode not found");
+		return -1;
+	}
+
+	/*
+	 * The master xlator is not in the graph but it can define an invalidate
+	 * handler.
+	 */
+	xl = inode->table->xl->ctx->master;
+	if (xl && xl->cbks->invalidate) {
+		old_THIS = THIS;
+		THIS = xl;
+		ret = xl->cbks->invalidate(xl, inode);
+		THIS = old_THIS;
+		if (ret)
+			return ret;
+	}
+
+	xl = inode->table->xl->graph->first;
+	while (xl) {
+		old_THIS = THIS;
+		THIS = xl;
+		if (xl->cbks->invalidate)
+			ret = xl->cbks->invalidate(xl, inode);
+		THIS = old_THIS;
+
+		if (ret)
+			break;
+
+		xl = xl->next;
+	}
+
+	return ret;
+}
+
 
 static void
 __inode_unlink (inode_t *inode, inode_t *parent, const char *name)
@@ -1157,9 +1202,7 @@ __inode_path (inode_t *inode, const char *name, char **bufp)
 out:
         if (__is_root_gfid (inode->gfid) && !name) {
                 ret = 1;
-                if (buf) {
-                        GF_FREE (buf);
-                }
+                GF_FREE (buf);
                 buf = GF_CALLOC (ret + 1, sizeof (char), gf_common_mt_char);
                 if (buf) {
                         strcpy (buf, "/");
@@ -1255,8 +1298,8 @@ __inode_table_init_root (inode_table_t *table)
         iatt.ia_ino = 1;
         iatt.ia_type = IA_IFDIR;
 
-        table->root = root;
         __inode_link (root, NULL, NULL, &iatt);
+        table->root = root;
 }
 
 
@@ -1272,6 +1315,7 @@ inode_table_new (size_t lru_limit, xlator_t *xl)
                 return NULL;
 
         new->xl = xl;
+        new->ctxcount = xl->graph->xl_count + 1;
 
         new->lru_limit = lru_limit;
 
@@ -1337,10 +1381,8 @@ inode_table_new (size_t lru_limit, xlator_t *xl)
 out:
         if (ret) {
                 if (new) {
-                        if (new->inode_hash)
-                                GF_FREE (new->inode_hash);
-                        if (new->name_hash)
-                                GF_FREE (new->name_hash);
+                        GF_FREE (new->inode_hash);
+                        GF_FREE (new->name_hash);
                         if (new->dentry_pool)
                                 mem_pool_destroy (new->dentry_pool);
                         if (new->inode_pool)
@@ -1406,8 +1448,7 @@ inode_from_path (inode_table_t *itable, const char *path)
         if (parent)
                 inode_unref (parent);
 
-        if (pathname)
-                GF_FREE (pathname);
+        GF_FREE (pathname);
 
 out:
         return inode;
@@ -1425,7 +1466,7 @@ __inode_ctx_set2 (inode_t *inode, xlator_t *xlator, uint64_t *value1_p,
         if (!inode || !xlator)
                 return -1;
 
-        for (index = 0; index < xlator->graph->xl_count; index++) {
+        for (index = 0; index < inode->table->ctxcount; index++) {
                 if (!inode->_ctx[index].xl_key) {
                         if (set_idx == -1)
                                 set_idx = index;
@@ -1482,12 +1523,12 @@ __inode_ctx_get2 (inode_t *inode, xlator_t *xlator, uint64_t *value1,
         if (!inode || !xlator)
                 return -1;
 
-        for (index = 0; index < xlator->graph->xl_count; index++) {
+        for (index = 0; index < inode->table->ctxcount; index++) {
                 if (inode->_ctx[index].xl_key == xlator)
                         break;
         }
 
-        if (index == xlator->graph->xl_count) {
+        if (index == inode->table->ctxcount) {
                 ret = -1;
                 goto out;
         }
@@ -1534,12 +1575,12 @@ inode_ctx_del2 (inode_t *inode, xlator_t *xlator, uint64_t *value1,
 
         LOCK (&inode->lock);
         {
-                for (index = 0; index < xlator->graph->xl_count; index++) {
+                for (index = 0; index < inode->table->ctxcount; index++) {
                         if (inode->_ctx[index].xl_key == xlator)
                                 break;
                 }
 
-                if (index == xlator->graph->xl_count) {
+                if (index == inode->table->ctxcount) {
                         ret = -1;
                         goto unlock;
                 }
@@ -1583,17 +1624,18 @@ inode_dump (inode_t *inode, char *prefix)
         {
                 gf_proc_dump_write("gfid", "%s", uuid_utoa (inode->gfid));
                 gf_proc_dump_write("nlookup", "%ld", inode->nlookup);
+                gf_proc_dump_write("fd-count", "%u", inode->fd_count);
                 gf_proc_dump_write("ref", "%u", inode->ref);
                 gf_proc_dump_write("ia_type", "%d", inode->ia_type);
                 if (inode->_ctx) {
-                        inode_ctx = GF_CALLOC (inode->table->xl->graph->xl_count,
+                        inode_ctx = GF_CALLOC (inode->table->ctxcount,
                                                sizeof (*inode_ctx),
                                                gf_common_mt_inode_ctx);
                         if (inode_ctx == NULL) {
                                 goto unlock;
                         }
 
-                        for (i = 0; i < inode->table->xl->graph->xl_count; i++) {
+                        for (i = 0; i < inode->table->ctxcount; i++) {
                                 inode_ctx[i] = inode->_ctx[i];
                         }
                 }
@@ -1610,7 +1652,7 @@ unlock:
         UNLOCK(&inode->lock);
 
         if (inode_ctx && (dump_options.xl_options.dump_inodectx == _gf_true)) {
-                for (i = 0; i < inode->table->xl->graph->xl_count; i++) {
+                for (i = 0; i < inode->table->ctxcount; i++) {
                         if (inode_ctx[i].xl_key) {
                                 xl = (xlator_t *)(long)inode_ctx[i].xl_key;
                                 if (xl->dumpops && xl->dumpops->inodectx)
@@ -1619,9 +1661,7 @@ unlock:
                 }
         }
 
-        if (inode_ctx != NULL) {
-                GF_FREE (inode_ctx);
-        }
+        GF_FREE (inode_ctx);
 
         return;
 }
