@@ -82,6 +82,8 @@ struct _call_frame_t {
         const char      *unwind_to;
 };
 
+#define SMALL_GROUP_COUNT 128
+
 struct _call_stack_t {
         union {
                 struct list_head      all_frames;
@@ -99,13 +101,17 @@ struct _call_stack_t {
         gid_t                         gid;
         pid_t                         pid;
         uint16_t                      ngrps;
-        uint32_t                      groups[GF_MAX_AUX_GROUPS];
+        uint32_t                      groups_small[SMALL_GROUP_COUNT];
+	uint32_t                     *groups_large;
+	uint32_t                     *groups;
         gf_lkowner_t                  lk_owner;
+        glusterfs_ctx_t              *ctx;
 
         call_frame_t                  frames;
 
         int32_t                       op;
         int8_t                        type;
+        struct timeval                tv;
 };
 
 
@@ -122,11 +128,10 @@ struct _call_stack_t {
 struct xlator_fops;
 
 void
-gf_set_fop_from_fn_pointer (call_frame_t *frame, struct xlator_fops *fops,
-                            void *fn);
+gf_latency_begin (call_frame_t *frame, void *fn);
 
 void
-gf_update_latency (call_frame_t *frame);
+gf_latency_end (call_frame_t *frame);
 
 static inline void
 FRAME_DESTROY (call_frame_t *frame)
@@ -173,6 +178,9 @@ STACK_DESTROY (call_stack_t *stack)
         while (stack->frames.next) {
                 FRAME_DESTROY (stack->frames.next);
         }
+
+	GF_FREE (stack->groups_large);
+
         mem_put (stack);
 
         if (local)
@@ -236,6 +244,7 @@ STACK_RESET (call_stack_t *stack)
                 _new->wind_from = __FUNCTION__;                         \
                 _new->wind_to = #fn;                                    \
                 _new->unwind_to = #rfn;                                 \
+                                                                        \
                 LOCK_INIT (&_new->lock);                                \
                 LOCK(&frame->root->stack_lock);                         \
                 {                                                       \
@@ -249,11 +258,27 @@ STACK_RESET (call_stack_t *stack)
                 UNLOCK(&frame->root->stack_lock);                       \
                 old_THIS = THIS;                                        \
                 THIS = obj;                                             \
+                if (frame->this->ctx->measure_latency)                  \
+                        gf_latency_begin (_new, fn);                    \
                 fn (_new, obj, params);                                 \
                 THIS = old_THIS;                                        \
         } while (0)
 
 
+/* make a call without switching frames */
+#define STACK_WIND_TAIL(frame, obj, fn, params ...)                     \
+        do {                                                            \
+                xlator_t     *old_THIS = NULL;                          \
+                                                                        \
+                frame->this = obj;                                      \
+                frame->wind_to = #fn;                                   \
+                old_THIS = THIS;                                        \
+                THIS = obj;                                             \
+                fn (frame, obj, params);                                \
+                THIS = old_THIS;                                        \
+        } while (0)
+
+
 /* make a call with a cookie */
 #define STACK_WIND_COOKIE(frame, rfn, cky, obj, fn, params ...)         \
         do {                                                            \
@@ -288,6 +313,8 @@ STACK_RESET (call_stack_t *stack)
                 fn##_cbk = rfn;                                         \
                 old_THIS = THIS;                                        \
                 THIS = obj;                                             \
+                if (obj->ctx->measure_latency)                          \
+                        gf_latency_begin (_new, fn);                    \
                 fn (_new, obj, params);                                 \
                 THIS = old_THIS;                                        \
         } while (0)
@@ -314,6 +341,8 @@ STACK_RESET (call_stack_t *stack)
                 THIS = _parent->this;                                   \
                 frame->complete = _gf_true;                             \
                 frame->unwind_from = __FUNCTION__;                      \
+                if (frame->this->ctx->measure_latency)                  \
+                        gf_latency_end (frame);                         \
                 fn (_parent, frame->cookie, _parent->this, params);     \
                 THIS = old_THIS;                                        \
         } while (0)
@@ -341,11 +370,31 @@ STACK_RESET (call_stack_t *stack)
                 THIS = _parent->this;                                   \
                 frame->complete = _gf_true;                             \
                 frame->unwind_from = __FUNCTION__;                      \
+                if (frame->this->ctx->measure_latency)                  \
+                        gf_latency_end (frame);                         \
                 fn (_parent, frame->cookie, _parent->this, params);     \
                 THIS = old_THIS;                                        \
         } while (0)
 
 
+static inline int
+call_stack_alloc_groups (call_stack_t *stack, int ngrps)
+{
+	if (ngrps <= SMALL_GROUP_COUNT) {
+		stack->groups = stack->groups_small;
+	} else {
+		stack->groups_large = GF_CALLOC (sizeof (gid_t), ngrps,
+						 gf_common_mt_groups_t);
+		if (!stack->groups_large)
+			return -1;
+		stack->groups = stack->groups_large;
+	}
+
+	stack->ngrps = ngrps;
+
+	return 0;
+}
+
 static inline call_frame_t *
 copy_frame (call_frame_t *frame)
 {
@@ -369,14 +418,27 @@ copy_frame (call_frame_t *frame)
         newstack->ngrps = oldstack->ngrps;
         newstack->op  = oldstack->op;
         newstack->type = oldstack->type;
+	if (call_stack_alloc_groups (newstack, oldstack->ngrps) != 0) {
+		mem_put (newstack);
+		return NULL;
+	}
         memcpy (newstack->groups, oldstack->groups,
-                sizeof (gid_t) * GF_MAX_AUX_GROUPS);
+                sizeof (gid_t) * oldstack->ngrps);
         newstack->unique = oldstack->unique;
 
         newstack->frames.this = frame->this;
         newstack->frames.root = newstack;
         newstack->pool = oldstack->pool;
         newstack->lk_owner = oldstack->lk_owner;
+        newstack->ctx = oldstack->ctx;
+
+        if (newstack->ctx->measure_latency) {
+                if (gettimeofday (&newstack->tv, NULL) == -1)
+                        gf_log ("stack", GF_LOG_ERROR, "gettimeofday () failed."
+                                " (%s)", strerror (errno));
+                memcpy (&newstack->frames.begin, &newstack->tv,
+                        sizeof (newstack->tv));
+        }
 
         LOCK_INIT (&newstack->frames.lock);
         LOCK_INIT (&newstack->stack_lock);
@@ -391,39 +453,9 @@ copy_frame (call_frame_t *frame)
         return &newstack->frames;
 }
 
-
-static inline call_frame_t *
-create_frame (xlator_t *xl, call_pool_t *pool)
-{
-        call_stack_t    *stack = NULL;
-
-        if (!xl || !pool) {
-                return NULL;
-        }
-
-        stack = mem_get0 (pool->stack_mem_pool);
-        if (!stack)
-                return NULL;
-
-        stack->pool = pool;
-        stack->frames.root = stack;
-        stack->frames.this = xl;
-
-        LOCK (&pool->lock);
-        {
-                list_add (&stack->all_frames, &pool->all_frames);
-                pool->cnt++;
-        }
-        UNLOCK (&pool->lock);
-
-        LOCK_INIT (&stack->frames.lock);
-        LOCK_INIT (&stack->stack_lock);
-
-        return &stack->frames;
-}
-
 void gf_proc_dump_pending_frames(call_pool_t *call_pool);
 void gf_proc_dump_pending_frames_to_dict (call_pool_t *call_pool,
                                           dict_t *dict);
+call_frame_t *create_frame (xlator_t *xl, call_pool_t *pool);
 gf_boolean_t __is_fuse_call (call_frame_t *frame);
 #endif /* _STACK_H */
