@@ -1,20 +1,11 @@
 /*
-  Copyright (c) 2010-2011 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 #ifndef _CONFIG_H
@@ -241,7 +232,12 @@ sys_statvfs (const char *path, struct statvfs *buf)
 int
 sys_close (int fd)
 {
-        return close (fd);
+        int ret = -1;
+
+        if (fd >= 0)
+                ret = close (fd);
+
+        return ret;
 }
 
 
@@ -268,7 +264,7 @@ sys_lsetxattr (const char *path, const char *name, const void *value,
                size_t size, int flags)
 {
 
-#ifdef GF_LINUX_HOST_OS
+#if defined(GF_LINUX_HOST_OS) || defined(__NetBSD__)
         return lsetxattr (path, name, value, size, flags);
 #endif
 
@@ -293,7 +289,7 @@ ssize_t
 sys_llistxattr (const char *path, char *list, size_t size)
 {
 
-#ifdef GF_LINUX_HOST_OS
+#if defined(GF_LINUX_HOST_OS) || defined(__NetBSD__)
         return llistxattr (path, list, size);
 #endif
 
@@ -316,7 +312,7 @@ ssize_t
 sys_lgetxattr (const char *path, const char *name, void *value, size_t size)
 {
 
-#ifdef GF_LINUX_HOST_OS
+#if defined(GF_LINUX_HOST_OS) || defined(__NetBSD__)
         return lgetxattr (path, name, value, size);
 #endif
 
@@ -340,7 +336,7 @@ ssize_t
 sys_fgetxattr (int filedes, const char *name, void *value, size_t size)
 {
 
-#ifdef GF_LINUX_HOST_OS
+#if defined(GF_LINUX_HOST_OS) || defined(__NetBSD__)
         return fgetxattr (filedes, name, value, size);
 #endif
 
@@ -360,12 +356,38 @@ sys_fgetxattr (int filedes, const char *name, void *value, size_t size)
 }
 
 
+int
+sys_fremovexattr (int filedes, const char *name)
+{
+
+#if defined(GF_LINUX_HOST_OS) || defined(__NetBSD__)
+        return fremovexattr (filedes, name);
+#endif
+
+        errno = ENOSYS;
+        return -1;
+#if 0 /* TODO: to port to other OSes, fill in each of below */
+#ifdef GF_BSD_HOST_OS
+        return extattr_remove_fd (filedes, EXTATTR_NAMESPACE_USER, name);
+#endif
+
+#ifdef GF_SOLARIS_HOST_OS
+        return solaris_fremovexattr (filedes, name);
+#endif
+
+#ifdef GF_DARWIN_HOST_OS
+        return fremovexattr (filedes, name, 0);
+#endif
+#endif
+}
+
+
 int
 sys_fsetxattr (int filedes, const char *name, const void *value,
                size_t size, int flags)
 {
 
-#ifdef GF_LINUX_HOST_OS
+#if defined(GF_LINUX_HOST_OS) || defined(__NetBSD__)
         return fsetxattr (filedes, name, value, size, flags);
 #endif
 
@@ -389,7 +411,7 @@ ssize_t
 sys_flistxattr (int filedes, char *list, size_t size)
 {
 
-#ifdef GF_LINUX_HOST_OS
+#if defined(GF_LINUX_HOST_OS) || defined(__NetBSD__)
         return flistxattr (filedes, list, size);
 #endif
 
@@ -412,7 +434,7 @@ int
 sys_lremovexattr (const char *path, const char *name)
 {
 
-#ifdef GF_LINUX_HOST_OS
+#if defined(GF_LINUX_HOST_OS) || defined(__NetBSD__)
         return lremovexattr (path, name);
 #endif
 
