@@ -119,18 +119,9 @@ xlator_volopt_dynload (char *xlator_type, void **dl_handle,
         int                     ret = -1;
         char                    *name = NULL;
         void                    *handle = NULL;
-        volume_opt_list_t       *vol_opt = NULL;
 
         GF_VALIDATE_OR_GOTO ("xlator", xlator_type, out);
 
-        GF_ASSERT (dl_handle);
-
-        if (*dl_handle)
-                if (dlclose (*dl_handle))
-                        gf_log ("xlator", GF_LOG_WARNING, "Unable to close "
-                                  "previously opened handle( may be stale)."
-                                  "Ignoring the invalid handle");
-
         ret = gf_asprintf (&name, "%s/%s.so", XLATORDIR, xlator_type);
         if (-1 == ret) {
                 gf_log ("xlator", GF_LOG_ERROR, "asprintf failed");
@@ -146,28 +137,20 @@ xlator_volopt_dynload (char *xlator_type, void **dl_handle,
                 gf_log ("xlator", GF_LOG_WARNING, "%s", dlerror ());
                 goto out;
         }
-        *dl_handle = handle;
 
-
-        vol_opt = GF_CALLOC (1, sizeof (volume_opt_list_t),
-                         gf_common_mt_volume_opt_list_t);
-
-        if (!vol_opt) {
+        if (!(opt_list->given_opt = dlsym (handle, "options"))) {
+                dlerror ();
+                gf_log ("xlator", GF_LOG_ERROR,
+                        "Failed to load xlator opt table");
                 goto out;
         }
 
-        if (!(vol_opt->given_opt = dlsym (handle, "options"))) {
-                dlerror ();
-                gf_log ("xlator", GF_LOG_DEBUG,
-                         "Strict option validation not enforced -- neglecting");
-        }
-        opt_list->given_opt = vol_opt->given_opt;
-
-        INIT_LIST_HEAD (&vol_opt->list);
-        list_add_tail (&vol_opt->list, &opt_list->list);
+        *dl_handle = handle;
 
         ret = 0;
  out:
+        GF_FREE (name);
+
         gf_log ("xlator", GF_LOG_DEBUG, "Returning %d", ret);
         return ret;
 
@@ -216,19 +199,19 @@ xlator_dynload (xlator_t *xl)
                 goto out;
         }
 
-        if (!(xl->init = dlsym (handle, "init"))) {
+        if (!(*VOID(&xl->init) = dlsym (handle, "init"))) {
                 gf_log ("xlator", GF_LOG_WARNING, "dlsym(init) on %s",
                         dlerror ());
                 goto out;
         }
 
-        if (!(xl->fini = dlsym (handle, "fini"))) {
+        if (!(*VOID(&(xl->fini)) = dlsym (handle, "fini"))) {
                 gf_log ("xlator", GF_LOG_WARNING, "dlsym(fini) on %s",
                         dlerror ());
                 goto out;
         }
 
-        if (!(xl->notify = dlsym (handle, "notify"))) {
+        if (!(*VOID(&(xl->notify)) = dlsym (handle, "notify"))) {
                 gf_log ("xlator", GF_LOG_TRACE,
                         "dlsym(notify) on %s -- neglecting", dlerror ());
         }
@@ -238,13 +221,13 @@ xlator_dynload (xlator_t *xl)
                         "dlsym(dumpops) on %s -- neglecting", dlerror ());
         }
 
-        if (!(xl->mem_acct_init = dlsym (handle, "mem_acct_init"))) {
+        if (!(*VOID(&(xl->mem_acct_init)) = dlsym (handle, "mem_acct_init"))) {
                 gf_log (xl->name, GF_LOG_TRACE,
                         "dlsym(mem_acct_init) on %s -- neglecting",
                         dlerror ());
         }
 
-        if (!(xl->reconfigure = dlsym (handle, "reconfigure"))) {
+        if (!(*VOID(&(xl->reconfigure)) = dlsym (handle, "reconfigure"))) {
                 gf_log ("xlator", GF_LOG_TRACE,
                         "dlsym(reconfigure) on %s -- neglecting",
                         dlerror());
@@ -262,6 +245,7 @@ xlator_dynload (xlator_t *xl)
                 gf_log (xl->name, GF_LOG_TRACE,
                         "Strict option validation not enforced -- neglecting");
         }
+        INIT_LIST_HEAD (&vol_opt->list);
         list_add_tail (&vol_opt->list, &xl->volume_options);
 
         fill_defaults (xl);
@@ -269,8 +253,7 @@ xlator_dynload (xlator_t *xl)
         ret = 0;
 
 out:
-        if (name)
-                GF_FREE (name);
+        GF_FREE (name);
         return ret;
 }
 
@@ -343,6 +326,28 @@ out:
         return search;
 }
 
+xlator_t *
+xlator_search_by_xl_type (xlator_t *any, const char *type)
+{
+        xlator_t *search = NULL;
+
+        GF_VALIDATE_OR_GOTO ("xlator", any, out);
+        GF_VALIDATE_OR_GOTO ("xlator", type, out);
+
+        search = any;
+
+        while (search->prev)
+                search = search->prev;
+
+        while (search) {
+                if (!strcmp (search->type, type))
+                        break;
+                search = search->next;
+        }
+
+out:
+        return search;
+}
 
 static int
 __xlator_init(xlator_t *xl)
@@ -459,12 +464,12 @@ xlator_mem_acct_init (xlator_t *xl, int num_types)
         int             i = 0;
         int             ret = 0;
 
-        if (!gf_mem_acct_is_enabled())
-                return 0;
-
         if (!xl)
                 return -1;
 
+        if (!xl->ctx->mem_acct_enable)
+                return 0;
+
         xl->mem_acct.num_types = num_types;
 
         xl->mem_acct.rec = CALLOC(num_types, sizeof(struct mem_acct_rec));
@@ -580,6 +585,23 @@ out:
         return ret;
 }
 
+void
+loc_gfid (loc_t *loc, uuid_t gfid)
+{
+        if (!gfid)
+                goto out;
+        uuid_clear (gfid);
+
+        if (!loc)
+                goto out;
+        else if (!uuid_is_null (loc->gfid))
+                uuid_copy (gfid, loc->gfid);
+        else if (loc->inode && (!uuid_is_null (loc->inode->gfid)))
+                uuid_copy (gfid, loc->inode->gfid);
+out:
+        return;
+}
+
 int
 loc_copy (loc_t *dst, loc_t *src)
 {
@@ -644,10 +666,8 @@ xlator_destroy (xlator_t *xl)
         if (!xl)
                 return 0;
 
-        if (xl->name)
-                GF_FREE (xl->name);
-        if (xl->type)
-                GF_FREE (xl->type);
+        GF_FREE (xl->name);
+        GF_FREE (xl->type);
         if (xl->dlhandle)
                 dlclose (xl->dlhandle);
         if (xl->options)
@@ -722,7 +742,7 @@ is_gf_log_command (xlator_t *this, const char *name, char *value)
                 goto out;
         }
 
-        ctx = glusterfs_ctx_get();
+        ctx = this->ctx;
         if (!ctx)
                 goto out;
         if (!ctx->active)
