@@ -730,10 +730,13 @@ typedef int32_t (*cbk_forget_t) (xlator_t *this,
 typedef int32_t (*cbk_release_t) (xlator_t *this,
                                   fd_t *fd);
 
+typedef int32_t (*cbk_invalidate_t)(xlator_t *this, inode_t *inode);
+
 struct xlator_cbks {
         cbk_forget_t    forget;
         cbk_release_t   release;
         cbk_release_t   releasedir;
+	cbk_invalidate_t invalidate;
 };
 
 typedef int32_t (*dumpop_priv_t) (xlator_t *this);
@@ -821,6 +824,7 @@ struct _xlator {
 
         /* for the memory pool of 'frame->local' */
         struct mem_pool    *local_pool;
+        gf_boolean_t        is_autoloaded;
 };
 
 #define xlator_has_parent(xl) (xl->parents != NULL)
@@ -861,6 +865,7 @@ void xlator_foreach (xlator_t *this,
                      void *data);
 
 xlator_t *xlator_search_by_name (xlator_t *any, const char *name);
+xlator_t *xlator_search_by_xl_type (xlator_t *any, const char *type);
 
 void inode_destroy_notify (inode_t *inode, const char *xlname);
 
@@ -868,6 +873,7 @@ int loc_copy (loc_t *dst, loc_t *src);
 #define loc_dup(src, dst) loc_copy(dst, src)
 void loc_wipe (loc_t *loc);
 int loc_path (loc_t *loc, const char *bname);
+void loc_gfid (loc_t *loc, uuid_t gfid);
 int xlator_mem_acct_init (xlator_t *xl, int num_types);
 int is_gf_log_command (xlator_t *trans, const char *name, char *value);
 int glusterd_check_log_level (const char *value);
@@ -877,4 +883,5 @@ enum gf_hdsk_event_notify_op {
         GF_EN_DEFRAG_STATUS,
         GF_EN_MAX,
 };
+
 #endif /* _XLATOR_H */
