@@ -94,8 +94,10 @@ enum gf_probe_resp {
 	GF_PROBE_FRIEND,
         GF_PROBE_ANOTHER_CLUSTER,
         GF_PROBE_VOLUME_CONFLICT,
+        GF_PROBE_SAME_UUID,
         GF_PROBE_UNKNOWN_PEER,
-        GF_PROBE_ADD_FAILED
+        GF_PROBE_ADD_FAILED,
+        GF_PROBE_QUORUM_NOT_MET
 };
 
 enum gf_deprobe_resp {
@@ -103,7 +105,8 @@ enum gf_deprobe_resp {
         GF_DEPROBE_LOCALHOST,
         GF_DEPROBE_NOT_FRIEND,
         GF_DEPROBE_BRICK_EXIST,
-        GF_DEPROBE_FRIEND_DOWN
+        GF_DEPROBE_FRIEND_DOWN,
+        GF_DEPROBE_QUORUM_NOT_MET,
 };
 
 enum gf_cbk_procnum {
@@ -150,6 +153,9 @@ enum gluster_cli_procnum {
         GLUSTER_CLI_STATEDUMP_VOLUME,
         GLUSTER_CLI_LIST_VOLUME,
         GLUSTER_CLI_CLRLOCKS_VOLUME,
+        GLUSTER_CLI_UUID_RESET,
+        GLUSTER_CLI_BD_OP,
+        GLUSTER_CLI_UUID_GET,
         GLUSTER_CLI_MAXVALUE,
 };
 
@@ -181,9 +187,17 @@ enum glusterd_brick_procnum {
         GLUSTERD_BRICK_XLATOR_DEFRAG,
         GLUSTERD_NODE_PROFILE,
         GLUSTERD_NODE_STATUS,
+        GLUSTERD_BRICK_BD_OP,
         GLUSTERD_BRICK_MAXVALUE,
 };
 
+enum glusterd_mgmt_hndsk_procnum {
+        GD_MGMT_HNDSK_NULL,
+        GD_MGMT_HNDSK_VERSIONS,
+        GD_MGMT_HNDSK_VERSIONS_ACK,
+        GD_MGMT_HNDSK_MAXVALUE,
+};
+
 typedef enum {
         GF_AFR_OP_INVALID,
         GF_AFR_OP_HEAL_INDEX,
@@ -194,8 +208,16 @@ typedef enum {
         GF_AFR_OP_SPLIT_BRAIN_FILES
 } gf_xl_afr_op_t ;
 
+typedef enum {
+        GF_BD_OP_INVALID,
+        GF_BD_OP_NEW_BD,
+        GF_BD_OP_DELETE_BD,
+        GF_BD_OP_CLONE_BD,
+        GF_BD_OP_SNAPSHOT_BD,
+} gf_xl_bd_op_t ;
+
 #define GLUSTER_HNDSK_PROGRAM    14398633 /* Completely random */
-#define GLUSTER_HNDSK_VERSION    2   /* 0.0.1 */
+#define GLUSTER_HNDSK_VERSION    2   /* 0.0.2 */
 
 #define GLUSTER_PMAP_PROGRAM     34123456
 #define GLUSTER_PMAP_VERSION     1
@@ -203,9 +225,9 @@ typedef enum {
 #define GLUSTER_CBK_PROGRAM      52743234 /* Completely random */
 #define GLUSTER_CBK_VERSION      1   /* 0.0.1 */
 
-#define GLUSTER3_1_FOP_PROGRAM   1298437 /* Completely random */
-#define GLUSTER3_1_FOP_VERSION   330 /* 3.3.0 */
-#define GLUSTER3_1_FOP_PROCCNT   GFS3_OP_MAXVALUE
+#define GLUSTER_FOP_PROGRAM   1298437 /* Completely random */
+#define GLUSTER_FOP_VERSION   330 /* 3.3.0 */
+#define GLUSTER_FOP_PROCCNT   GFS3_OP_MAXVALUE
 
 /* Second version */
 #define GD_MGMT_PROGRAM          1238433 /* Completely random */
@@ -220,4 +242,8 @@ typedef enum {
 #define GD_BRICK_PROGRAM         4867634 /*Completely random*/
 #define GD_BRICK_VERSION         2
 
+/* OP-VERSION handshake */
+#define GD_MGMT_HNDSK_PROGRAM    1239873 /* Completely random */
+#define GD_MGMT_HNDSK_VERSION    1
+
 #endif /* !_PROTOCOL_COMMON_H */
