@@ -66,8 +66,8 @@ _is_lock_fop (struct saved_frame *sframe)
 {
         int     fop     = 0;
 
-        if (SFRAME_GET_PROGNUM (sframe) == GLUSTER3_1_FOP_PROGRAM &&
-            SFRAME_GET_PROGVER (sframe) == GLUSTER3_1_FOP_VERSION)
+        if (SFRAME_GET_PROGNUM (sframe) == GLUSTER_FOP_PROGRAM &&
+            SFRAME_GET_PROGVER (sframe) == GLUSTER_FOP_VERSION)
                 fop = SFRAME_GET_PROCNUM (sframe);
 
         return ((fop == GFS3_OP_LK) ||
@@ -144,7 +144,6 @@ call_bail (void *data)
         struct saved_frame    *saved_frame = NULL;
         struct saved_frame    *trav = NULL;
         struct saved_frame    *tmp = NULL;
-        struct tm              frame_sent_tm;
         char                   frame_sent[256] = {0,};
         struct timeval         timeout = {0,};
         struct iovec           iov = {0,};
@@ -191,8 +190,8 @@ call_bail (void *data)
         pthread_mutex_unlock (&conn->lock);
 
         list_for_each_entry_safe (trav, tmp, &list, list) {
-                localtime_r (&trav->saved_at.tv_sec, &frame_sent_tm);
-                strftime (frame_sent, 32, "%Y-%m-%d %H:%M:%S", &frame_sent_tm);
+                gf_time_fmt (frame_sent, sizeof frame_sent,
+                             trav->saved_at.tv_sec, gf_timefmt_FT);
                 snprintf (frame_sent + strlen (frame_sent),
                           256 - strlen (frame_sent),
                           ".%"GF_PRI_SUSECONDS, trav->saved_at.tv_usec);
@@ -340,20 +339,16 @@ out:
 void
 saved_frames_unwind (struct saved_frames *saved_frames)
 {
-        struct rpc_clnt      *clnt = NULL;
 	struct saved_frame   *trav = NULL;
 	struct saved_frame   *tmp = NULL;
-        struct tm            *frame_sent_tm = NULL;
-        char                 timestr[256] = {0,};
-
+        char                  timestr[1024] = {0,};
         struct iovec          iov = {0,};
 
         list_splice_init (&saved_frames->lk_sf.list, &saved_frames->sf.list);
 
 	list_for_each_entry_safe (trav, tmp, &saved_frames->sf.list, list) {
-                frame_sent_tm = localtime (&trav->saved_at.tv_sec);
-                strftime (timestr, sizeof(timestr), "%Y-%m-%d %H:%M:%S",
-                          frame_sent_tm);
+                gf_time_fmt (timestr, sizeof timestr,
+                             trav->saved_at.tv_sec, gf_timefmt_FT);
                 snprintf (timestr + strlen (timestr),
                           sizeof(timestr) - strlen (timestr),
                           ".%"GF_PRI_SUSECONDS, trav->saved_at.tv_usec);
@@ -373,14 +368,12 @@ saved_frames_unwind (struct saved_frames *saved_frames)
                                   trav->rpcreq->xid);
 		saved_frames->count--;
 
-                clnt = rpc_clnt_ref (trav->rpcreq->conn->rpc_clnt);
                 trav->rpcreq->rpc_status = -1;
                 trav->rpcreq->cbkfn (trav->rpcreq, &iov, 1, trav->frame);
 
                 rpc_clnt_reply_deinit (trav->rpcreq,
                                        trav->rpcreq->conn->rpc_clnt->reqpool);
 
-                clnt = rpc_clnt_unref (clnt);
 		list_del_init (&trav->list);
                 mem_put (trav);
 	}
@@ -429,15 +422,6 @@ rpc_clnt_reconnect (void *trans_ptr)
                                 "attempting reconnect");
                         ret = rpc_transport_connect (trans,
                                                      conn->config.remote_port);
-                        /* Every time there is a disconnection, processes
-                           should try to connect to 'glusterd' (ie, default
-                           port) or whichever port given as 'option remote-port'
-                           in volume file. */
-                        /* Below code makes sure the (re-)configured port lasts
-                           for just one successful attempt */
-                        if (!ret)
-                                conn->config.remote_port = 0;
-
                         conn->reconnect =
                                 gf_timer_call_after (clnt->ctx, tv,
                                                      rpc_clnt_reconnect,
@@ -460,7 +444,7 @@ rpc_clnt_reconnect (void *trans_ptr)
 int
 rpc_clnt_fill_request_info (struct rpc_clnt *clnt, rpc_request_info_t *info)
 {
-        struct saved_frame  saved_frame = {{}, 0};
+        struct saved_frame  saved_frame;
         int                 ret         = -1;
 
         pthread_mutex_lock (&clnt->conn.lock);
@@ -740,7 +724,8 @@ rpc_clnt_handle_cbk (struct rpc_clnt *clnt, rpc_transport_pollin_t *msg)
 
         if (found && (procnum < program->numactors) &&
             (program->actors[procnum].actor)) {
-                program->actors[procnum].actor (&progmsg);
+                program->actors[procnum].actor (clnt, program->mydata,
+                                                &progmsg);
         }
 
 out:
@@ -834,6 +819,9 @@ out:
         return;
 }
 
+static void
+rpc_clnt_destroy (struct rpc_clnt *rpc);
+
 int
 rpc_clnt_notify (rpc_transport_t *trans, void *mydata,
                  rpc_transport_event_t event, void *data, ...)
@@ -879,9 +867,7 @@ rpc_clnt_notify (rpc_transport_t *trans, void *mydata,
         }
 
         case RPC_TRANSPORT_CLEANUP:
-                /* this event should not be received on a client for, a
-                 * transport is only disconnected, but never destroyed.
-                 */
+                rpc_clnt_destroy (clnt);
                 ret = 0;
                 break;
 
@@ -925,6 +911,14 @@ rpc_clnt_notify (rpc_transport_t *trans, void *mydata,
 
         case RPC_TRANSPORT_CONNECT:
         {
+                /* Every time there is a disconnection, processes
+                   should try to connect to 'glusterd' (ie, default
+                   port) or whichever port given as 'option remote-port'
+                   in volume file. */
+                /* Below code makes sure the (re-)configured port lasts
+                   for just one successful attempt */
+                conn->config.remote_port = 0;
+
                 if (clnt->notifyfn)
                         ret = clnt->notifyfn (clnt, clnt->mydata,
                                               RPC_CLNT_CONNECT, NULL);
@@ -951,7 +945,7 @@ rpc_clnt_connection_deinit (rpc_clnt_connection_t *conn)
 }
 
 
-inline int
+static inline int
 rpc_clnt_connection_init (struct rpc_clnt *clnt, glusterfs_ctx_t *ctx,
                           dict_t *options, char *name)
 {
@@ -1324,7 +1318,7 @@ out:
 
 int
 rpcclnt_cbk_program_register (struct rpc_clnt *clnt,
-                              rpcclnt_cb_program_t *program)
+                              rpcclnt_cb_program_t *program, void *mydata)
 {
         int                   ret                = -1;
         char                  already_registered = 0;
@@ -1364,6 +1358,8 @@ rpcclnt_cbk_program_register (struct rpc_clnt *clnt,
         memcpy (tmp, program, sizeof (*tmp));
         INIT_LIST_HEAD (&tmp->program);
 
+        tmp->mydata = mydata;
+
         pthread_mutex_lock (&clnt->lock);
         {
                 list_add_tail (&tmp->program, &clnt->programs);
@@ -1479,10 +1475,6 @@ rpc_clnt_submit (struct rpc_clnt *rpc, rpc_clnt_prog_t *prog,
                 if (conn->connected == 0) {
                         ret = rpc_transport_connect (conn->trans,
                                                      conn->config.remote_port);
-                        /* Below code makes sure the (re-)configured port lasts
-                           for just one successful connect attempt */
-                        if (!ret)
-                                conn->config.remote_port = 0;
                 }
 
                 ret = rpc_transport_submit_request (rpc->conn.trans,
@@ -1549,19 +1541,23 @@ rpc_clnt_ref (struct rpc_clnt *rpc)
 }
 
 
+static void
+rpc_clnt_trigger_destroy (struct rpc_clnt *rpc)
+{
+        if (!rpc)
+                return;
+
+        rpc_clnt_disable (rpc);
+        rpc_transport_unref (rpc->conn.trans);
+}
+
 static void
 rpc_clnt_destroy (struct rpc_clnt *rpc)
 {
         if (!rpc)
                 return;
 
-        if (rpc->conn.trans) {
-                rpc_transport_unregister_notify (rpc->conn.trans);
-                rpc_transport_disconnect (rpc->conn.trans);
-                rpc_transport_unref (rpc->conn.trans);
-        }
-
-        rpc_clnt_reconnect_cleanup (&rpc->conn);
+        rpc_transport_unregister_notify (rpc->conn.trans);
         saved_frames_destroy (rpc->conn.saved_frames);
         pthread_mutex_destroy (&rpc->lock);
         pthread_mutex_destroy (&rpc->conn.lock);
@@ -1588,13 +1584,36 @@ rpc_clnt_unref (struct rpc_clnt *rpc)
         }
         pthread_mutex_unlock (&rpc->lock);
         if (!count) {
-                rpc_clnt_destroy (rpc);
+                rpc_clnt_trigger_destroy (rpc);
                 return NULL;
         }
         return rpc;
 }
 
 
+char
+rpc_clnt_is_disabled (struct rpc_clnt *rpc)
+{
+
+        rpc_clnt_connection_t *conn = NULL;
+        char                   disabled = 0;
+
+        if (!rpc) {
+                goto out;
+        }
+
+        conn = &rpc->conn;
+
+        pthread_mutex_lock (&conn->lock);
+        {
+                disabled = rpc->disabled;
+        }
+        pthread_mutex_unlock (&conn->lock);
+
+out:
+        return disabled;
+}
+
 void
 rpc_clnt_disable (struct rpc_clnt *rpc)
 {
@@ -1664,7 +1683,7 @@ rpc_clnt_reconfig (struct rpc_clnt *rpc, struct rpc_clnt_config *config)
                         if (strcmp (rpc->conn.config.remote_host,
                                     config->remote_host))
                                 gf_log (rpc->conn.trans->name, GF_LOG_INFO,
-                                        "changing port to %s (from %s)",
+                                        "changing hostname to %s (from %s)",
                                         config->remote_host,
                                         rpc->conn.config.remote_host);
                         FREE (rpc->conn.config.remote_host);
@@ -1728,8 +1747,7 @@ rpc_clnt_transport_unix_options_build (dict_t **options, char *filepath,
         *options = dict;
 out:
         if (ret) {
-                if (fpath)
-                        GF_FREE (fpath);
+                GF_FREE (fpath);
                 if (dict)
                         dict_unref (dict);
         }
