@@ -27,6 +27,7 @@
 #include "xdr-common.h"
 #include "xdr-generic.h"
 #include "rpc-common-xdr.h"
+#include "syncop.h"
 
 #include <errno.h>
 #include <pthread.h>
@@ -206,6 +207,8 @@ rpcsvc_program_actor (rpcsvc_request_t *req)
                 goto err;
         }
 
+	req->synctask = program->synctask;
+
         err = SUCCESS;
         gf_log (GF_RPCSVC, GF_LOG_TRACE, "Actor found: %s - %s",
                 program->progname, actor->procname);
@@ -220,7 +223,7 @@ err:
 /* this procedure can only pass 4 arguments to registered notifyfn. To send more
  * arguments call wrapper->notify directly.
  */
-inline void
+static inline void
 rpcsvc_program_notify (rpcsvc_listener_t *listener, rpcsvc_event_t event,
                        void *data)
 {
@@ -243,7 +246,7 @@ out:
 }
 
 
-inline int
+static inline int
 rpcsvc_accept (rpcsvc_t *svc, rpc_transport_t *listen_trans,
                rpc_transport_t *new_trans)
 {
@@ -273,8 +276,13 @@ rpcsvc_request_destroy (rpcsvc_request_t *req)
                 iobref_unref (req->iobref);
         }
 
+        if (req->hdr_iobuf)
+                iobuf_unref (req->hdr_iobuf);
+
         rpc_transport_unref (req->trans);
 
+	GF_FREE (req->auxgidlarge);
+
         mem_put (req);
 
 out:
@@ -430,11 +438,33 @@ err:
 }
 
 
+int
+rpcsvc_check_and_reply_error (int ret, call_frame_t *frame, void *opaque)
+{
+	rpcsvc_request_t  *req = NULL;
+
+	req = opaque;
+
+        if (ret)
+                gf_log ("rpcsvc", GF_LOG_ERROR,
+                        "rpc actor failed to complete successfully");
+
+        if (ret == RPCSVC_ACTOR_ERROR) {
+                ret = rpcsvc_error_reply (req);
+                if (ret)
+                        gf_log ("rpcsvc", GF_LOG_WARNING,
+                                "failed to queue error reply");
+        }
+
+	return 0;
+}
+
 int
 rpcsvc_handle_rpc_call (rpcsvc_t *svc, rpc_transport_t *trans,
                         rpc_transport_pollin_t *msg)
 {
         rpcsvc_actor_t          *actor = NULL;
+	rpcsvc_actor            actor_fn = NULL;
         rpcsvc_request_t        *req = NULL;
         int                     ret = -1;
         uint16_t                port = 0;
@@ -497,31 +527,34 @@ rpcsvc_handle_rpc_call (rpcsvc_t *svc, rpc_transport_t *trans,
                 /* Before going to xlator code, set the THIS properly */
                 THIS = svc->mydata;
 
-                if (req->count == 2) {
-                        if (actor->vector_actor) {
-                                ret = actor->vector_actor (req, &req->msg[1], 1,
-                                                           req->iobref);
-                        } else {
-                                rpcsvc_request_seterr (req, PROC_UNAVAIL);
-                                /* LOG TODO: print more info about procnum,
-                                   prognum etc, also print transport info */
-                                gf_log (GF_RPCSVC, GF_LOG_ERROR,
-                                        "No vectored handler present");
-                                ret = RPCSVC_ACTOR_ERROR;
-                        }
-                } else if (actor->actor) {
-                        ret = actor->actor (req);
+		actor_fn = actor->actor;
+
+		if (!actor_fn) {
+			rpcsvc_request_seterr (req, PROC_UNAVAIL);
+			/* LOG TODO: print more info about procnum,
+			   prognum etc, also print transport info */
+			gf_log (GF_RPCSVC, GF_LOG_ERROR,
+				"No vectored handler present");
+			ret = RPCSVC_ACTOR_ERROR;
+			goto err_reply;
+		}
+
+		if (req->synctask) {
+                        if (msg->hdr_iobuf)
+                                req->hdr_iobuf = iobuf_ref (msg->hdr_iobuf);
+
+			ret = synctask_new (THIS->ctx->env,
+					    (synctask_fn_t) actor_fn,
+					    rpcsvc_check_and_reply_error, NULL,
+                                            req);
+                } else {
+			ret = actor_fn (req);
                 }
         }
 
 err_reply:
-        if (ret == RPCSVC_ACTOR_ERROR) {
-                ret = rpcsvc_error_reply (req);
-        }
-
-        if (ret)
-                gf_log ("rpcsvc", GF_LOG_WARNING, "failed to queue error reply");
 
+        ret = rpcsvc_check_and_reply_error (ret, NULL, req);
         /* No need to propagate error beyond this function since the reply
          * has now been queued. */
         ret = 0;
@@ -687,7 +720,7 @@ err:
         return txrecord;
 }
 
-inline int
+static inline int
 rpcsvc_get_callid (rpcsvc_t *rpc)
 {
         return GF_UNIVERSAL_ANSWER;
@@ -873,7 +906,7 @@ out:
         return ret;
 }
 
-inline int
+static inline int
 rpcsvc_transport_submit (rpc_transport_t *trans, struct iovec *hdrvec,
                          int hdrcount, struct iovec *proghdr, int proghdrcount,
                          struct iovec *progpayload, int progpayloadcount,
@@ -1142,7 +1175,7 @@ out:
 }
 
 
-inline int
+static inline int
 rpcsvc_program_unregister_portmap (rpcsvc_program_t *prog)
 {
         int ret = 0;
@@ -1502,6 +1535,7 @@ rpcsvc_create_listeners (rpcsvc_t *svc, dict_t *options, char *name)
                 }
 
                 GF_FREE (transport_name);
+                transport_name = NULL;
                 count++;
         }
 
@@ -1513,17 +1547,13 @@ rpcsvc_create_listeners (rpcsvc_t *svc, dict_t *options, char *name)
         transport_type = NULL;
 
 out:
-        if (str != NULL) {
-                GF_FREE (str);
-        }
+        GF_FREE (str);
 
-        if (transport_type != NULL) {
-                GF_FREE (transport_type);
-        }
+        GF_FREE (transport_type);
 
-        if (tmp != NULL) {
-                GF_FREE (tmp);
-        }
+        GF_FREE (tmp);
+
+        GF_FREE (transport_name);
 
         return count;
 }
@@ -1700,15 +1730,17 @@ rpcsvc_dump (rpcsvc_request_t *req)
         uint32_t     dump_rsp_len      = 0;
 
         if (!req)
-                goto fail;
+                goto sendrsp;
 
         ret = build_prog_details (req, &rsp);
         if (ret < 0) {
                 op_errno = -ret;
-                goto fail;
+                goto sendrsp;
         }
 
-fail:
+        op_errno = 0;
+
+sendrsp:
         rsp.op_errno = gf_errno_to_error (op_errno);
         rsp.op_ret   = ret;
 
@@ -1720,15 +1752,12 @@ fail:
 
         ret = xdr_serialize_generic (iov, &rsp, (xdrproc_t)xdr_gf_dump_rsp);
         if (ret < 0) {
-                if (req)
-                        req->rpc_err = GARBAGE_ARGS;
-                op_errno = EINVAL;
-                goto fail;
+                ret = RPCSVC_ACTOR_ERROR;
+        } else {
+                rpcsvc_submit_generic (req, &iov, 1, NULL, 0, NULL);
+                ret = 0;
         }
 
-        ret = rpcsvc_submit_generic (req, &iov, 1, NULL, 0,
-                                     NULL);
-
         free_prog_details (&rsp);
 
         return ret;
@@ -1811,8 +1840,7 @@ rpcsvc_transport_unix_options_build (dict_t **options, char *filepath)
         *options = dict;
 out:
         if (ret) {
-                if (fpath)
-                        GF_FREE (fpath);
+                GF_FREE (fpath);
                 if (dict)
                         dict_unref (dict);
         }
@@ -1898,10 +1926,13 @@ rpcsvc_transport_peer_check_search (dict_t *options, char *pattern, char *clstr)
         char                    *addrstr = NULL;
         char                    *dup_addrstr = NULL;
         char                    *svptr = NULL;
+        char                    *fqdn        = NULL;
 
         if ((!options) || (!clstr))
                 return -1;
 
+        ret = dict_get_str (options, "fqdn", &fqdn);
+
         if (!dict_get (options, pattern))
                 return -1;
 
@@ -1929,13 +1960,23 @@ rpcsvc_transport_peer_check_search (dict_t *options, char *pattern, char *clstr)
                 if (ret == 0)
                         goto err;
 
+                /* compare hostnames if applicable */
+                if (fqdn) {
+#ifdef FNM_CASEFOLD
+                        ret = fnmatch (addrtok, fqdn, FNM_CASEFOLD);
+#else
+                        ret = fnmatch (addrtok, fqdn, 0);
+#endif
+                        if (ret == 0)
+                                goto err;
+                }
+
                 addrtok = strtok_r (NULL, ",", &svptr);
         }
 
         ret = -1;
 err:
-        if (dup_addrstr)
-                GF_FREE (dup_addrstr);
+        GF_FREE (dup_addrstr);
 
         return ret;
 }
@@ -2148,6 +2189,7 @@ rpcsvc_transport_peer_check_name (dict_t *options, char *volname,
         int     aret = RPCSVC_AUTH_REJECT;
         int     rjret = RPCSVC_AUTH_REJECT;
         char    clstr[RPCSVC_PEER_STRLEN];
+        char   *hostname    = NULL;
 
         if (!trans)
                 return ret;
@@ -2160,6 +2202,11 @@ rpcsvc_transport_peer_check_name (dict_t *options, char *volname,
                 goto err;
         }
 
+        ret = gf_get_hostname_from_ip (clstr, &hostname);
+        if (!ret)
+                ret = dict_set_dynstr (options, "fqdn",
+                                       hostname);
+
         aret = rpcsvc_transport_peer_check_allow (options, volname, clstr);
         rjret = rpcsvc_transport_peer_check_reject (options, volname, clstr);
 
@@ -2179,14 +2226,14 @@ rpcsvc_transport_peer_check_addr (dict_t *options, char *volname,
         int     rjret = RPCSVC_AUTH_REJECT;
         char    clstr[RPCSVC_PEER_STRLEN];
         char   *tmp   = NULL;
-        struct sockaddr_storage sastorage = {0,};
-        struct sockaddr        *sockaddr  = NULL;
+        union gf_sock_union sock_union;
 
         if (!trans)
                 return ret;
 
         ret = rpcsvc_transport_peeraddr (trans, clstr, RPCSVC_PEER_STRLEN,
-                                         &sastorage, sizeof (sastorage));
+                                         &sock_union.storage,
+                                         sizeof (sock_union.storage));
         if (ret != 0) {
                 gf_log (GF_RPCSVC, GF_LOG_ERROR, "Failed to get remote addr: "
                         "%s", gai_strerror (ret));
@@ -2194,8 +2241,7 @@ rpcsvc_transport_peer_check_addr (dict_t *options, char *volname,
                 goto err;
         }
 
-        sockaddr = (struct sockaddr *) &sastorage;
-        switch (sockaddr->sa_family) {
+        switch (sock_union.sa.sa_family) {
 
         case AF_INET:
         case AF_INET6:
@@ -2313,10 +2359,9 @@ int
 rpcsvc_transport_privport_check (rpcsvc_t *svc, char *volname,
                                  rpc_transport_t *trans)
 {
-        struct sockaddr_storage sastorage = {0,};
-        struct sockaddr_in     *sa = NULL;
+        union gf_sock_union     sock_union;
         int                     ret = RPCSVC_AUTH_REJECT;
-        socklen_t               sasize = sizeof (sa);
+        socklen_t               sinsize = sizeof (&sock_union.sin);
         char                    *srchstr = NULL;
         char                    *valstr = NULL;
         int                     globalinsecure = RPCSVC_AUTH_REJECT;
@@ -2324,12 +2369,13 @@ rpcsvc_transport_privport_check (rpcsvc_t *svc, char *volname,
         uint16_t                port = 0;
         gf_boolean_t            insecure = _gf_false;
 
+        memset (&sock_union, 0, sizeof (sock_union));
+
         if ((!svc) || (!volname) || (!trans))
                 return ret;
 
-        sa = (struct sockaddr_in*) &sastorage;
-        ret = rpcsvc_transport_peeraddr (trans, NULL, 0, &sastorage,
-                                         sasize);
+        ret = rpcsvc_transport_peeraddr (trans, NULL, 0, &sock_union.storage,
+                                         sinsize);
         if (ret != 0) {
                 gf_log (GF_RPCSVC, GF_LOG_ERROR, "Failed to get peer addr: %s",
                         gai_strerror (ret));
@@ -2337,7 +2383,7 @@ rpcsvc_transport_privport_check (rpcsvc_t *svc, char *volname,
                 goto err;
         }
 
-        port = ntohs (sa->sin_port);
+        port = ntohs (sock_union.sin.sin_port);
         gf_log (GF_RPCSVC, GF_LOG_TRACE, "Client port: %d", (int)port);
         /* If the port is already a privileged one, dont bother with checking
          * options.
@@ -2398,6 +2444,8 @@ rpcsvc_transport_privport_check (rpcsvc_t *svc, char *volname,
                         " allowed");
 
 err:
+        GF_FREE (srchstr);
+
         return ret;
 }
 
@@ -2425,17 +2473,16 @@ rpcsvc_volume_allowed (dict_t *options, char *volname)
                 ret = dict_get_str (options, srchstr, &addrstr);
 
 out:
-        if (srchstr)
-                GF_FREE (srchstr);
+        GF_FREE (srchstr);
 
         return addrstr;
 }
 
 
 rpcsvc_actor_t gluster_dump_actors[] = {
-        [GF_DUMP_NULL] = {"NULL", GF_DUMP_NULL, NULL, NULL, NULL, 0},
-        [GF_DUMP_DUMP] = {"DUMP", GF_DUMP_DUMP, rpcsvc_dump, NULL, NULL, 0},
-        [GF_DUMP_MAXVALUE] = {"MAXVALUE", GF_DUMP_MAXVALUE, NULL, NULL, NULL, 0},
+        [GF_DUMP_NULL] = {"NULL", GF_DUMP_NULL, NULL, NULL, 0},
+        [GF_DUMP_DUMP] = {"DUMP", GF_DUMP_DUMP, rpcsvc_dump, NULL, 0},
+        [GF_DUMP_MAXVALUE] = {"MAXVALUE", GF_DUMP_MAXVALUE, NULL, NULL, 0},
 };
 
 
