@@ -19,8 +19,6 @@
 #include "protocol-common.h"
 #include "event-history.h"
 
-#define AFR_POLL_TIMEOUT 600
-
 typedef enum {
         STOP_CRAWL_ON_SINGLE_SUBVOL = 1
 } afr_crawl_flags_t;
@@ -59,6 +57,11 @@ afr_start_crawl (xlator_t *this, int idx, afr_crawl_type_t crawl,
 static int
 _crawl_directory (fd_t *fd, loc_t *loc, afr_crawl_data_t *crawl_data);
 
+/* For calling straight through (e.g. already in a synctask). */
+int
+afr_find_child_position (xlator_t *this, int child, afr_child_pos_t *pos);
+
+/* For deferring through a new synctask. */
 int
 afr_syncop_find_child_position (void *data);
 
@@ -87,8 +90,7 @@ shd_cleanup_event (void *event)
 
         if (!shd_event)
                 goto out;
-        if (shd_event->path)
-                GF_FREE (shd_event->path);
+        GF_FREE (shd_event->path);
         GF_FREE (shd_event);
 out:
         return;
@@ -180,16 +182,18 @@ out:
 
 int
 _get_path_from_gfid_loc (xlator_t *this, xlator_t *readdir_xl, loc_t *child,
-                         char **fpath)
+                         char **fpath, gf_boolean_t *missing)
 {
         dict_t          *xattr = NULL;
         char            *path = NULL;
         int             ret = -1;
 
-        ret = syncop_getxattr (readdir_xl, child, &xattr,
-                               GFID_TO_PATH_KEY);
-        if (ret)
+        ret = syncop_getxattr (readdir_xl, child, &xattr, GFID_TO_PATH_KEY);
+        if (ret < 0) {
+                if ((errno == ENOENT) && missing)
+                        *missing = _gf_true;
                 goto out;
+        }
         ret = dict_get_str (xattr, GFID_TO_PATH_KEY, &path);
         if (ret) {
                 gf_log (this->name, GF_LOG_ERROR, "Failed to get path for "
@@ -240,34 +244,6 @@ _add_eh_to_dict (xlator_t *this, eh_t *eh, dict_t *dict, int child)
         return 0;
 }
 
-int
-_add_summary_to_dict (xlator_t *this, afr_crawl_data_t *crawl_data,
-                      gf_dirent_t *entry,
-                      loc_t *childloc, loc_t *parentloc, struct iatt *iattr)
-{
-        dict_t          *output = NULL;
-        xlator_t        *readdir_xl = NULL;
-        int             ret = -1;
-        char            *path = NULL;
-
-        if (uuid_is_null (childloc->gfid))
-                goto out;
-
-        output = crawl_data->op_data;
-        readdir_xl = crawl_data->readdir_xl;
-
-        ret = _get_path_from_gfid_loc (this, readdir_xl, childloc, &path);
-        if (ret)
-                goto out;
-
-        ret = _add_path_to_dict (this, output, crawl_data->child, path, NULL,
-                                 _gf_true);
-out:
-        if (ret && path)
-                GF_FREE (path);
-        return ret;
-}
-
 void
 _remove_stale_index (xlator_t *this, xlator_t *readdir_xl,
                      loc_t *parent, char *fname)
@@ -281,10 +257,10 @@ _remove_stale_index (xlator_t *this, xlator_t *readdir_xl,
         gf_log (this->name, GF_LOG_DEBUG, "Removing stale index "
                 "for %s on %s", index_loc.name, readdir_xl->name);
         ret = syncop_unlink (readdir_xl, &index_loc);
-        if (ret && (errno != ENOENT)) {
-                gf_log (this->name, GF_LOG_ERROR, "%s: Failed to remove"
-                        " index on %s - %s", index_loc.name,
-                        readdir_xl->name, strerror (errno));
+        if(ret && (errno != ENOENT)) {
+                gf_log(this->name, GF_LOG_ERROR, "%s: Failed to remove index "
+                       "on %s - %s",index_loc.name, readdir_xl->name,
+                       strerror (errno));
         }
         index_loc.path = NULL;
         loc_wipe (&index_loc);
@@ -292,6 +268,40 @@ out:
         return;
 }
 
+int
+_add_summary_to_dict (xlator_t *this, afr_crawl_data_t *crawl_data,
+                      gf_dirent_t *entry,
+                      loc_t *childloc, loc_t *parentloc, struct iatt *iattr)
+{
+        dict_t          *output = NULL;
+        xlator_t        *readdir_xl = NULL;
+        int             ret = -1;
+        char            *path = NULL;
+        gf_boolean_t    missing = _gf_false;
+        char            gfid_str[64] = {0};
+
+        if (uuid_is_null (childloc->gfid))
+                goto out;
+
+        output = crawl_data->op_data;
+        readdir_xl = crawl_data->readdir_xl;
+
+        ret = _get_path_from_gfid_loc (this, readdir_xl, childloc, &path,
+                                       &missing);
+        if (ret == 0) {
+                ret = _add_path_to_dict (this, output, crawl_data->child, path,
+                                         NULL, _gf_true);
+        } else if (missing) {
+                _remove_stale_index (this, readdir_xl, parentloc,
+                                     uuid_utoa_r (childloc->gfid, gfid_str));
+        }
+
+out:
+        if (ret && path)
+                GF_FREE (path);
+        return ret;
+}
+
 void
 _crawl_post_sh_action (xlator_t *this, loc_t *parent, loc_t *child,
                        int32_t op_ret, int32_t op_errno, dict_t *xattr_rsp,
@@ -302,20 +312,22 @@ _crawl_post_sh_action (xlator_t *this, loc_t *parent, loc_t *child,
         afr_self_heald_t *shd = NULL;
         eh_t             *eh = NULL;
         char             *path = NULL;
+        char             gfid_str[64] = {0};
         shd_event_t      *event = NULL;
         int32_t          sh_failed = 0;
         gf_boolean_t     split_brain = 0;
-
+        int32_t          actual_sh_done = 0;
         priv = this->private;
         shd  = &priv->shd;
         if (crawl_data->crawl == INDEX) {
                 if ((op_ret < 0) && (op_errno == ENOENT)) {
                         _remove_stale_index (this, crawl_data->readdir_xl,
-                                             parent, uuid_utoa (child->gfid));
+                                             parent, uuid_utoa_r (child->gfid,
+                                                                  gfid_str));
                         goto out;
                 }
                 ret = _get_path_from_gfid_loc (this, crawl_data->readdir_xl,
-                                               child, &path);
+                                               child, &path, NULL);
                 if (ret)
                         goto out;
         } else {
@@ -326,26 +338,42 @@ _crawl_post_sh_action (xlator_t *this, loc_t *parent, loc_t *child,
                 }
         }
 
-        if (xattr_rsp)
+        if (xattr_rsp) {
                 ret = dict_get_int32 (xattr_rsp, "sh-failed", &sh_failed);
+                ret = dict_get_int32 (xattr_rsp, "actual-sh-done", &actual_sh_done);
+        }
+
         split_brain = afr_is_split_brain (this, child->inode);
-        if ((op_ret < 0 && op_errno == EIO) || split_brain)
+
+        if ((op_ret < 0 && op_errno == EIO) || split_brain) {
                 eh = shd->split_brain;
-        else if ((op_ret < 0) || sh_failed)
+        } else if ((op_ret < 0) || sh_failed) {
                 eh = shd->heal_failed;
-        else
-                eh = shd->healed;
+        } else if (actual_sh_done == 1) {
+                      eh = shd->healed;
+        }
+
         ret = -1;
-        event = GF_CALLOC (1, sizeof (*event), gf_afr_mt_shd_event_t);
-        if (!event)
-                goto out;
-        event->child = crawl_data->child;
-        event->path = path;
-        ret = eh_save_history (eh, event);
-        if (ret < 0) {
-                gf_log (this->name, GF_LOG_ERROR, "%s:Failed to save to "
-                        "event history, (%d, %s)", path, op_ret, strerror (op_errno));
-                goto out;
+
+        if (eh != NULL) {
+                event = GF_CALLOC (1, sizeof (*event), gf_afr_mt_shd_event_t);
+                if (!event)
+                        goto out;
+                event->child = crawl_data->child;
+                event->path = path;
+
+                ret = eh_save_history (eh, event);
+                if (ret < 0) {
+                        gf_log (this->name, GF_LOG_ERROR, "%s:Failed to save "
+                                "to event history, (%d, %s)", path, op_ret,
+                                strerror (op_errno));
+
+                        goto out;
+                }
+        } else {
+                gf_log (this->name, GF_LOG_DEBUG, "%s:Self heal already done ",
+                        path);
+
         }
         ret = 0;
 out:
@@ -354,6 +382,25 @@ out:
         return;
 }
 
+int
+_link_inode_update_loc (xlator_t *this, loc_t *loc, struct iatt *iattr)
+{
+        inode_t       *link_inode = NULL;
+        int           ret = -1;
+
+        link_inode = inode_link (loc->inode, NULL, NULL, iattr);
+        if (link_inode == NULL) {
+                gf_log (this->name, GF_LOG_ERROR, "inode link failed "
+                        "on the inode (%s)", uuid_utoa (iattr->ia_gfid));
+                goto out;
+        }
+        inode_unref (loc->inode);
+        loc->inode = link_inode;
+        ret = 0;
+out:
+        return ret;
+}
+
 int
 _self_heal_entry (xlator_t *this, afr_crawl_data_t *crawl_data, gf_dirent_t *entry,
                   loc_t *child, loc_t *parent, struct iatt *iattr)
@@ -370,6 +417,9 @@ _self_heal_entry (xlator_t *this, afr_crawl_data_t *crawl_data, gf_dirent_t *ent
                                crawl_data);
         if (xattr_rsp)
                 dict_unref (xattr_rsp);
+        if (ret == 0)
+                ret = _link_inode_update_loc (this, child, iattr);
+
         return ret;
 }
 
@@ -401,11 +451,11 @@ _crawl_proceed (xlator_t *this, int child, int crawl_flags, char **reason)
         shd  = &priv->shd;
         if (!shd->enabled) {
                 msg = "Self-heal daemon is not enabled";
-                gf_log (this->name, GF_LOG_ERROR, "%s", msg);
+                gf_log (this->name, GF_LOG_DEBUG, "%s", msg);
                 goto out;
         }
         if (!priv->child_up[child]) {
-                gf_log (this->name, GF_LOG_ERROR, "Stopping crawl for %s , "
+                gf_log (this->name, GF_LOG_DEBUG, "Stopping crawl for %s , "
                         "subvol went down", priv->children[child]->name);
                 msg = "Brick is Not connected";
                 goto out;
@@ -414,7 +464,7 @@ _crawl_proceed (xlator_t *this, int child, int crawl_flags, char **reason)
         if (crawl_flags & STOP_CRAWL_ON_SINGLE_SUBVOL) {
                 if (afr_up_children_count (priv->child_up,
                                            priv->child_count) < 2) {
-                        gf_log (this->name, GF_LOG_ERROR, "Stopping crawl as "
+                        gf_log (this->name, GF_LOG_DEBUG, "Stopping crawl as "
                                 "< 2 children are up");
                         msg = "< 2 bricks in replica are running";
                         goto out;
@@ -446,20 +496,28 @@ _do_crawl_op_on_local_subvols (xlator_t *this, afr_crawl_type_t crawl,
         if (op == HEAL)
                 crawl_flags |= STOP_CRAWL_ON_SINGLE_SUBVOL;
 
-        ret = dict_get_int32 (output, this->name, &xl_id);
-        if (ret) {
-                gf_log (this->name, GF_LOG_ERROR, "Invalid input, "
-                        "translator-id is not available");
-                goto out;
+        if (output) {
+                ret = dict_get_int32 (output, this->name, &xl_id);
+                if (ret) {
+                        gf_log (this->name, GF_LOG_ERROR, "Invalid input, "
+                                "translator-id is not available");
+                        goto out;
+                }
         }
         pos_data.this = this;
         subkey = "status";
         for (i = 0; i < priv->child_count; i++) {
                 if (_crawl_proceed (this, i, crawl_flags, &status)) {
                         pos_data.child = i;
-                        ret = synctask_new (this->ctx->env,
-                                            afr_syncop_find_child_position,
-                                            NULL, NULL, &pos_data);
+                        /*
+                         * We're already in a synctask in this case, so we
+                         * don't need to defer through a second (and in fact
+                         * that can cause deadlock).  Just call straight
+                         * through instead.
+                         */
+                        ret = afr_find_child_position(pos_data.this,
+                                                      pos_data.child,
+                                                      &pos_data.pos);
                         if (ret) {
                                 status = "Not able to find brick location";
                         } else if (pos_data.pos == AFR_POS_REMOTE) {
@@ -470,7 +528,7 @@ _do_crawl_op_on_local_subvols (xlator_t *this, afr_crawl_type_t crawl,
                                         status = "Started self-heal";
                                         _do_self_heal_on_subvol (this, i,
                                                                  crawl);
-                                } else {
+                                } else if (output) {
                                         status = "";
                                         afr_start_crawl (this, i, INDEX,
                                                          _add_summary_to_dict,
@@ -478,14 +536,19 @@ _do_crawl_op_on_local_subvols (xlator_t *this, afr_crawl_type_t crawl,
                                                          NULL);
                                 }
                         }
-                        snprintf (key, sizeof (key), "%d-%d-%s", xl_id,
-                                  i, subkey);
-                        ret = dict_set_str (output, key, status);
+                        if (output) {
+                                snprintf (key, sizeof (key), "%d-%d-%s", xl_id,
+                                          i, subkey);
+                                ret = dict_set_str (output, key, status);
+                        }
                         if (!op_ret && (crawl == FULL))
                                 break;
                 }
-                snprintf (key, sizeof (key), "%d-%d-%s", xl_id, i, subkey);
-                ret = dict_set_str (output, key, status);
+                if (output) {
+                        snprintf (key, sizeof (key), "%d-%d-%s", xl_id, i,
+                                  subkey);
+                        ret = dict_set_str (output, key, status);
+                }
         }
 out:
         return op_ret;
@@ -602,7 +665,7 @@ afr_poll_self_heal (void *data)
         }
         if (shd->enabled && (shd->pos[child] == AFR_POS_LOCAL))
                 _do_self_heal_on_subvol (this, child, INDEX);
-        timeout.tv_sec = AFR_POLL_TIMEOUT;
+        timeout.tv_sec = shd->timeout;
         timeout.tv_usec = 0;
         //notify and previous timer should be synchronized.
         LOCK (&priv->lock);
@@ -629,7 +692,7 @@ unlock:
 }
 
 static int
-afr_local_child_poll_self_heal  (int ret, call_frame_t *sync_frame, void *data)
+afr_handle_child_up  (int ret, call_frame_t *sync_frame, void *data)
 {
         afr_self_heald_t *shd = NULL;
         shd_pos_t        *pos_data = data;
@@ -643,6 +706,7 @@ afr_local_child_poll_self_heal  (int ret, call_frame_t *sync_frame, void *data)
         shd->pos[pos_data->child] = pos_data->pos;
         if (pos_data->pos != AFR_POS_REMOTE)
                 afr_poll_self_heal ((void*)(long)pos_data->child);
+        _do_self_heal_on_local_subvols (THIS, INDEX, NULL);
 out:
         GF_FREE (data);
         return 0;
@@ -666,28 +730,65 @@ afr_proactive_self_heal (void *data)
         pos_data->this = this;
         pos_data->child = child;
         ret = synctask_new (this->ctx->env, afr_syncop_find_child_position,
-                            afr_local_child_poll_self_heal, NULL, pos_data);
+                            afr_handle_child_up, NULL, pos_data);
         if (ret)
                 goto out;
 out:
         return;
 }
 
-int
-_link_inode_update_loc (xlator_t *this, loc_t *loc, struct iatt *iattr)
+static int
+get_pathinfo_host (char *pathinfo, char *hostname, size_t size)
 {
-        inode_t       *link_inode = NULL;
-        int           ret = -1;
+        char    *start = NULL;
+        char    *end = NULL;
+        int     ret  = -1;
+        int     i    = 0;
 
-        link_inode = inode_link (loc->inode, NULL, NULL, iattr);
-        if (link_inode == NULL) {
-                gf_log (this->name, GF_LOG_ERROR, "inode link failed "
-                        "on the inode (%s)", uuid_utoa (iattr->ia_gfid));
+        if (!pathinfo)
+                goto out;
+
+        start = strchr (pathinfo, ':');
+        if (!start)
+                goto out;
+        end = strrchr (pathinfo, ':');
+        if (start == end)
+                goto out;
+
+        memset (hostname, 0, size);
+        i = 0;
+        while (++start != end)
+                hostname[i++] = *start;
+        ret = 0;
+out:
+        return ret;
+}
+
+int
+afr_local_pathinfo (char *pathinfo, gf_boolean_t *local)
+{
+        int             ret   = 0;
+        char            pathinfohost[1024] = {0};
+        char            localhost[1024] = {0};
+        xlator_t        *this = THIS;
+
+        *local = _gf_false;
+        ret = get_pathinfo_host (pathinfo, pathinfohost, sizeof (pathinfohost));
+        if (ret) {
+                gf_log (this->name, GF_LOG_ERROR, "Invalid pathinfo: %s",
+                        pathinfo);
                 goto out;
         }
-        inode_unref (loc->inode);
-        loc->inode = link_inode;
-        ret = 0;
+
+        ret = gethostname (localhost, sizeof (localhost));
+        if (ret) {
+                gf_log (this->name, GF_LOG_ERROR, "gethostname() failed, "
+                        "reason: %s", strerror (errno));
+                goto out;
+        }
+
+        if (!strcmp (localhost, pathinfohost))
+                *local = _gf_true;
 out:
         return ret;
 }
@@ -858,9 +959,6 @@ _process_entries (xlator_t *this, loc_t *parentloc, gf_dirent_t *entries,
                 if (ret)
                         continue;
 
-                ret = _link_inode_update_loc (this, &entry_loc, &iattr);
-                if (ret)
-                        goto out;
                 if (crawl_data->crawl == INDEX)
                         continue;
 
@@ -964,7 +1062,7 @@ afr_find_child_position (xlator_t *this, int child, afr_child_pos_t *pos)
 
         ret = syncop_getxattr (priv->children[child], &loc, &xattr_rsp,
                                GF_XATTR_NODE_UUID_KEY);
-        if (ret) {
+        if (ret < 0) {
                 gf_log (this->name, GF_LOG_ERROR, "getxattr failed on %s - "
                         "(%s)", priv->children[child]->name, strerror (errno));
                 goto out;
