@@ -34,8 +34,10 @@ dht_layout_dump (dht_layout_t  *layout, const char *prefix)
         char    key[GF_DUMP_MAX_BUF_LEN];
         int     i = 0;
 
-        GF_VALIDATE_OR_GOTO ("dht", layout, out);
-        GF_VALIDATE_OR_GOTO ("dht", prefix, out);
+        if (!layout)
+                goto out;
+        if (!prefix)
+                goto out;
 
         gf_proc_dump_build_key(key, prefix, "cnt");
         gf_proc_dump_write(key, "%d", layout->cnt);
@@ -43,8 +45,13 @@ dht_layout_dump (dht_layout_t  *layout, const char *prefix)
         gf_proc_dump_write(key, "%d", layout->preset);
         gf_proc_dump_build_key(key, prefix, "gen");
         gf_proc_dump_write(key, "%d", layout->gen);
-        gf_proc_dump_build_key(key, prefix, "type");
-        gf_proc_dump_write(key, "%d", layout->type);
+        if (layout->type != IA_INVAL) {
+                gf_proc_dump_build_key(key, prefix, "inode type");
+                gf_proc_dump_write(key, "%d", layout->type);
+        }
+
+        if  (!IA_ISDIR (layout->type))
+                goto out;
 
         for (i = 0; i < layout->cnt; i++) {
                 gf_proc_dump_build_key(key, prefix,"list[%d].err", i);
@@ -79,18 +86,15 @@ dht_priv_dump (xlator_t *this)
         dht_conf_t      *conf = NULL;
         int             ret = -1;
 
-        GF_VALIDATE_OR_GOTO ("dht", this, out);
+        if (!this)
+                goto out;
 
         conf = this->private;
-
         if (!conf)
-                return -1;
+                goto out;
 
         ret = TRY_LOCK(&conf->subvolume_lock);
-
         if (ret != 0) {
-                gf_log("", GF_LOG_WARNING, "Unable to lock dht subvolume %s",
-                       this->name);
                 return ret;
         }
 
@@ -121,8 +125,8 @@ dht_priv_dump (xlator_t *this)
 
         gf_proc_dump_write("search_unhashed", "%d", conf->search_unhashed);
         gf_proc_dump_write("gen", "%d", conf->gen);
-        gf_proc_dump_write("min_free_disk", "%lu", conf->min_free_disk);
-	gf_proc_dump_write("min_free_inodes", "%lu", conf->min_free_inodes);
+        gf_proc_dump_write("min_free_disk", "%lf", conf->min_free_disk);
+	gf_proc_dump_write("min_free_inodes", "%lf", conf->min_free_inodes);
         gf_proc_dump_write("disk_unit", "%c", conf->disk_unit);
         gf_proc_dump_write("refresh_interval", "%d", conf->refresh_interval);
         gf_proc_dump_write("unhashed_sticky_bit", "%d", conf->unhashed_sticky_bit);
@@ -151,21 +155,17 @@ dht_inodectx_dump (xlator_t *this, inode_t *inode)
 {
         int             ret = -1;
         dht_layout_t    *layout = NULL;
-        uint64_t        tmp_layout = 0;
 
-        GF_VALIDATE_OR_GOTO ("dht", this, out);
-        GF_VALIDATE_OR_GOTO ("dht", inode, out);
+        if (!this)
+                goto out;
+        if (!inode)
+                goto out;
 
-        ret = inode_ctx_get (inode, this, &tmp_layout);
+        ret = dht_inode_ctx_layout_get (inode, this, &layout);
 
-        if (ret != 0)
+        if ((ret != 0) || !layout)
                 return ret;
 
-        layout = (dht_layout_t *)(long)tmp_layout;
-
-        if (!layout)
-                return -1;
-
         gf_proc_dump_add_section("xlator.cluster.dht.%s.inode", this->name);
         dht_layout_dump(layout, "layout");
 
@@ -213,11 +213,9 @@ fini (xlator_t *this)
                         GF_FREE (conf->file_layouts);
                 }
 
-                if (conf->subvolumes)
-                        GF_FREE (conf->subvolumes);
+                GF_FREE (conf->subvolumes);
 
-                if (conf->subvolume_status)
-                        GF_FREE (conf->subvolume_status);
+                GF_FREE (conf->subvolume_status);
 
                 GF_FREE (conf);
         }
@@ -264,6 +262,7 @@ dht_parse_decommissioned_bricks (xlator_t *this, dht_conf_t *conf,
                         if (!strcmp (conf->subvolumes[i]->name, node)) {
                                 conf->decommissioned_bricks[i] =
                                         conf->subvolumes[i];
+                                        conf->decommission_subvols_cnt++;
                                 gf_log (this->name, GF_LOG_INFO,
                                         "decommissioning subvolume %s",
                                         conf->subvolumes[i]->name);
@@ -280,8 +279,7 @@ dht_parse_decommissioned_bricks (xlator_t *this, dht_conf_t *conf,
         ret = 0;
         conf->decommission_in_progress = 1;
 out:
-        if (dup_brick)
-                GF_FREE (dup_brick);
+        GF_FREE (dup_brick);
 
         return ret;
 }
@@ -327,12 +325,24 @@ reconfigure (xlator_t *this, dict_t *options)
 
 	GF_OPTION_RECONF ("min-free-disk", conf->min_free_disk, options,
                           percent_or_size, out);
+        /* option can be any one of percent or bytes */
+        conf->disk_unit = 0;
+        if (conf->min_free_disk < 100.0)
+                conf->disk_unit = 'p';
+
 	GF_OPTION_RECONF ("min-free-inodes", conf->min_free_inodes, options,
                           percent, out);
+
         GF_OPTION_RECONF ("directory-layout-spread", conf->dir_spread_cnt,
                           options, uint32, out);
+
         GF_OPTION_RECONF ("readdir-optimize", conf->readdir_optimize, options,
                           bool, out);
+        if (conf->defrag) {
+                GF_OPTION_RECONF ("rebalance-stats", conf->defrag->stats,
+                                  options, bool, out);
+        }
+
         if (dict_get_str (options, "decommissioned-bricks", &temp_str) == 0) {
                 ret = dht_parse_decommissioned_bricks (this, conf, temp_str);
                 if (ret == -1)
@@ -387,6 +397,8 @@ init (xlator_t *this)
 
                 defrag->is_exiting = 0;
 
+                conf->defrag = defrag;
+
                 ret = dict_get_str (this->options, "node-uuid", &node_uuid);
                 if (ret) {
                         gf_log (this->name, GF_LOG_ERROR, "node-uuid not "
@@ -402,10 +414,9 @@ init (xlator_t *this)
 
                 defrag->cmd = cmd;
 
-                conf->defrag = defrag;
+                defrag->stats = _gf_false;
         }
 
-
         conf->search_unhashed = GF_DHT_LOOKUP_UNHASHED_ON;
         if (dict_get_str (this->options, "lookup-unhashed", &temp_str) == 0) {
                 /* If option is not "auto", other options _should_ be boolean */
@@ -432,8 +443,18 @@ init (xlator_t *this)
 
         GF_OPTION_INIT ("assert-no-child-down", conf->assert_no_child_down,
                         bool, err);
+
         GF_OPTION_INIT ("readdir-optimize", conf->readdir_optimize, bool, err);
 
+        if (defrag) {
+                GF_OPTION_INIT ("rebalance-stats", defrag->stats, bool, err);
+        }
+
+        /* option can be any one of percent or bytes */
+        conf->disk_unit = 0;
+        if (conf->min_free_disk < 100)
+                conf->disk_unit = 'p';
+
         ret = dht_init_subvolumes (this, conf);
         if (ret == -1) {
                 goto err;
@@ -475,14 +496,13 @@ err:
                         GF_FREE (conf->file_layouts);
                 }
 
-                if (conf->subvolumes)
-                        GF_FREE (conf->subvolumes);
+                GF_FREE (conf->subvolumes);
 
-                if (conf->subvolume_status)
-                        GF_FREE (conf->subvolume_status);
+                GF_FREE (conf->subvolume_status);
 
-                if (conf->du_stats)
-                        GF_FREE (conf->du_stats);
+                GF_FREE (conf->du_stats);
+
+                GF_FREE (conf->defrag);
 
                 GF_FREE (conf);
         }
@@ -558,18 +578,23 @@ struct volume_options options[] = {
                     "on", "off"},
           .type = GF_OPTION_TYPE_STR,
           .default_value = "on",
+          .description = "This option if set to ON, does a lookup through "
+          "all the sub-volumes, in case a lookup didn't return any result "
+          "from the hash subvolume. If set to OFF, it does not do a lookup "
+          "on the remaining subvolumes."
         },
         { .key  = {"min-free-disk"},
           .type = GF_OPTION_TYPE_PERCENT_OR_SIZET,
           .default_value = "10%",
-          .description = "Percentage/Size of disk space that must be "
-                         "kept free."
+          .description = "Percentage/Size of disk space, after which the "
+          "process starts balancing out the cluster, and logs will appear "
+          "in log files",
         },
 	{ .key  = {"min-free-inodes"},
           .type = GF_OPTION_TYPE_PERCENT,
           .default_value = "5%",
-          .description = "Percentage inodes that must be "
-                         "kept free."
+          .description = "after system has only N% of inodes, warnings "
+          "starts to appear in log files",
         },
         { .key = {"unhashed-sticky-bit"},
           .type = GF_OPTION_TYPE_BOOL,
@@ -578,16 +603,26 @@ struct volume_options options[] = {
         { .key = {"use-readdirp"},
           .type = GF_OPTION_TYPE_BOOL,
           .default_value = "on",
+          .description = "This option if set to ON, forces the use of "
+          "readdirp, and hence also displays the stats of the files."
         },
         { .key = {"assert-no-child-down"},
           .type = GF_OPTION_TYPE_BOOL,
           .default_value = "off",
+          .description = "This option if set to ON, in the event of "
+          "CHILD_DOWN, will call exit."
         },
         { .key  = {"directory-layout-spread"},
           .type = GF_OPTION_TYPE_INT,
+          .min  = 1,
+          .validate = GF_OPT_VALIDATE_MIN,
+          .description = "Specifies the directory layout spread."
         },
         { .key  = {"decommissioned-bricks"},
           .type = GF_OPTION_TYPE_ANY,
+          .description = "This option if set to ON, decommissions "
+          "the brick, so that no new data is allowed to be created "
+          "on that brick."
         },
         { .key  = {"rebalance-cmd"},
           .type = GF_OPTION_TYPE_INT,
@@ -595,9 +630,20 @@ struct volume_options options[] = {
         { .key = {"node-uuid"},
           .type = GF_OPTION_TYPE_STR,
         },
+        { .key = {"rebalance-stats"},
+          .type = GF_OPTION_TYPE_BOOL,
+          .default_value = "off",
+          .description = "This option if set to ON displays and logs the "
+          " time taken for migration of each file, during the rebalance "
+          "process. If set to OFF, the rebalance logs will only display the "
+          "time spent in each directory."
+        },
         { .key = {"readdir-optimize"},
           .type = GF_OPTION_TYPE_BOOL,
           .default_value = "off",
+          .description = "This option if set to ON enables the optimization "
+          "that allows DHT to requests non-first subvolumes to filter out "
+          "directory entries."
         },
 
         { .key  = {NULL} },
