@@ -320,7 +320,8 @@ nufa_create (call_frame_t *frame, xlator_t *this,
         if (dht_is_subvol_filled (this, (xlator_t *)conf->private)) {
                 avail_subvol =
                         dht_free_disk_available_subvol (this,
-                                                        (xlator_t *)conf->private);
+                                                        (xlator_t *)conf->private,
+                                                        local);
         }
 
         if (subvol != avail_subvol) {
@@ -362,17 +363,22 @@ nufa_mknod_linkfile_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         dht_local_t  *local = NULL;
 
         local = frame->local;
+        if (!local || !local->cached_subvol) {
+                op_errno = EINVAL;
+                op_ret = -1;
+                goto err;
+        }
 
         if (op_ret >= 0) {
-                STACK_WIND (frame, dht_newfile_cbk,
-                            local->cached_subvol,
+                STACK_WIND_COOKIE (frame, dht_newfile_cbk,
+                            (void *)local->cached_subvol, local->cached_subvol,
                             local->cached_subvol->fops->mknod,
                             &local->loc, local->mode, local->rdev,
                             local->umask, local->params);
 
                 return 0;
         }
-
+err:
         WIPE (postparent);
         WIPE (preparent);
 
@@ -420,7 +426,8 @@ nufa_mknod (call_frame_t *frame, xlator_t *this,
         if (dht_is_subvol_filled (this, (xlator_t *)conf->private)) {
                 avail_subvol =
                         dht_free_disk_available_subvol (this,
-                                                        (xlator_t *)conf->private);
+                                                        (xlator_t *)conf->private,
+                                                        local);
         }
 
         if (avail_subvol != subvol) {
@@ -440,9 +447,9 @@ nufa_mknod (call_frame_t *frame, xlator_t *this,
         gf_log (this->name, GF_LOG_TRACE,
                 "creating %s on %s", loc->path, subvol->name);
 
-        STACK_WIND (frame, dht_newfile_cbk,
-                    subvol, subvol->fops->mknod,
-                    loc, mode, rdev, umask, params);
+        STACK_WIND_COOKIE (frame, dht_newfile_cbk, (void *)subvol, subvol,
+                           subvol->fops->mknod, loc, mode, rdev, umask,
+                           params);
 
         return 0;
 
@@ -481,11 +488,9 @@ fini (xlator_t *this)
                         GF_FREE (conf->file_layouts);
                 }
 
-                if (conf->subvolumes)
-                        GF_FREE (conf->subvolumes);
+                GF_FREE (conf->subvolumes);
 
-                if (conf->subvolume_status)
-                        GF_FREE (conf->subvolume_status);
+                GF_FREE (conf->subvolume_status);
 
                 GF_FREE (conf);
         }
@@ -493,6 +498,26 @@ fini (xlator_t *this)
         return;
 }
 
+gf_boolean_t
+same_first_part (char *str1, char term1, char *str2, char term2)
+{
+        gf_boolean_t    ended1;
+        gf_boolean_t    ended2;
+
+        for (;;) {
+                ended1 = ((*str1 == '\0') || (*str1 == term1));
+                ended2 = ((*str2 == '\0') || (*str2 == term2));
+                if (ended1 && ended2) {
+                        return _gf_true;
+                }
+                if (ended1 || ended2 || (*str1 != *str2)) {
+                        return _gf_false;
+                }
+                ++str1;
+                ++str2;
+        }
+}
+
 int
 init (xlator_t *this)
 {
@@ -504,7 +529,11 @@ init (xlator_t *this)
         int            ret = -1;
         int            i = 0;
         char           my_hostname[256];
-        uint32_t       temp_free_disk = 0;
+        double         temp_free_disk = 0;
+        uint64_t       size = 0;
+        xlator_t      *local_subvol = NULL;
+        char          *brick_host = NULL;
+        xlator_t      *kid = NULL;
 
         if (!this->children) {
                 gf_log (this->name, GF_LOG_CRITICAL,
@@ -563,23 +592,58 @@ init (xlator_t *this)
                 local_volname = data->data;
         }
 
-        trav = this->children;
-        while (trav) {
+        for (trav = this->children; trav; trav = trav->next) {
                 if (strcmp (trav->xlator->name, local_volname) == 0)
                         break;
-                trav = trav->next;
+                if (local_subvol) {
+                        continue;
+                }
+                kid = trav->xlator;
+                for (;;) {
+                        if (dict_get_str(trav->xlator->options,"remote-host",
+                                         &brick_host) == 0) {
+                                /* Found it. */
+                                break;
+                        }
+                        if (!kid->children) {
+                                /* Nowhere further to look. */
+                                gf_log (this->name, GF_LOG_ERROR,
+                                        "could not get remote-host");
+                                goto err;
+                        }
+                        if (kid->children->next) {
+                                /* Multiple choices, can't/shouldn't decide. */
+                                gf_log (this->name, GF_LOG_ERROR,
+                                        "NUFA found fan-out (type %s) volume",
+                                        kid->type);
+                                goto err;
+                        }
+                        /* One-to-one xlators are OK, try the next one. */
+                        kid = kid->children->xlator;
+                }
+                if (same_first_part(my_hostname,'.',brick_host,'.')) {
+                        local_subvol = trav->xlator;
+                }
         }
 
-        if (!trav) {
-                gf_log (this->name, GF_LOG_ERROR,
-                        "Could not find subvolume named '%s'. "
-                        "Please define volume with the name as the hostname "
-                        "or override it with 'option local-volume-name'",
-                        local_volname);
-                goto err;
+        if (trav) {
+                gf_log (this->name, GF_LOG_INFO,
+                        "Using specified subvol %s", local_volname);
+                conf->private = trav->xlator;
         }
+        else if (local_subvol) {
+                gf_log (this->name, GF_LOG_INFO,
+                        "Using first local subvol %s", local_subvol->name);
+                conf->private = local_subvol;
+        }
+        else {
+                gf_log (this->name, GF_LOG_ERROR,
+                        "Could not find specified or local subvol");
+                goto err;
+
+        }
+
         /* The volume specified exists */
-        conf->private = trav->xlator;
 
         conf->min_free_disk = 10;
         conf->disk_unit = 'p';
@@ -589,16 +653,16 @@ init (xlator_t *this)
                 if (gf_string2percent (temp_str,
                                        &temp_free_disk) == 0) {
                         if (temp_free_disk > 100) {
-                                gf_string2bytesize (temp_str,
-                                                    &conf->min_free_disk);
+                                gf_string2bytesize (temp_str, &size);
+                                conf->min_free_disk = size;
                                 conf->disk_unit = 'b';
                         } else {
-                                conf->min_free_disk = (uint64_t)temp_free_disk;
+                                conf->min_free_disk = temp_free_disk;
                                 conf->disk_unit = 'p';
                         }
                 } else {
-                        gf_string2bytesize (temp_str,
-                                            &conf->min_free_disk);
+                        gf_string2bytesize (temp_str, &size);
+                        conf->min_free_disk = size;
                         conf->disk_unit = 'b';
                 }
         }
@@ -629,14 +693,11 @@ err:
                         GF_FREE (conf->file_layouts);
                 }
 
-                if (conf->subvolumes)
-                        GF_FREE (conf->subvolumes);
+                GF_FREE (conf->subvolumes);
 
-                if (conf->subvolume_status)
-                        GF_FREE (conf->subvolume_status);
+                GF_FREE (conf->subvolume_status);
 
-                if (conf->du_stats)
-                        GF_FREE (conf->du_stats);
+                GF_FREE (conf->du_stats);
 
                 GF_FREE (conf);
         }
