@@ -1,22 +1,12 @@
 /*
-  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
-  This file is part of GlusterFS.
+   Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
+   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+   This file is licensed to you under your choice of the GNU Lesser
+   General Public License, version 3 or any later version (LGPLv3 or
+   later), or the GNU General Public License, version 2 (GPLv2), in all
+   cases as published by the Free Software Foundation.
 */
-
 #ifndef _CONFIG_H
 #define _CONFIG_H
 #include "config.h"
@@ -441,23 +431,29 @@ out:
         return ret;
 }
 
+static int
+_check_key_is_zero_filled (dict_t *d, char *k, data_t *v,
+                           void *tmp)
+{
+        if (mem_0filled ((const char*)v->data, v->len)) {
+                /* -1 means, no more iterations, treat as 'break' */
+                return -1;
+        }
+        return 0;
+}
+
+
 void
 _xattrop_index_action (xlator_t *this, inode_t *inode,  dict_t *xattr)
 {
-        data_pair_t       *trav = NULL;
         gf_boolean_t      zero_xattr = _gf_true;
         index_inode_ctx_t *ctx = NULL;
         int               ret = 0;
 
-        trav = xattr->members_list;
-        while (trav && inode) {
-                if (mem_0filled ((const char*)trav->value->data,
-                                 trav->value->len)) {
-                        zero_xattr = _gf_false;
-                        break;
-                }
-                trav = trav->next;
-        }
+        ret = dict_foreach (xattr, _check_key_is_zero_filled, NULL);
+        if (ret == -1)
+                zero_xattr = _gf_false;
+
         ret = index_inode_ctx_get (inode, this, &ctx);
         if (ret) {
                 gf_log (this->name, GF_LOG_ERROR, "Not able to %s %s -> index",
@@ -1029,6 +1025,10 @@ init (xlator_t *this)
         int ret = -1;
         index_priv_t *priv = NULL;
         pthread_t thread;
+        pthread_attr_t  w_attr;
+        gf_boolean_t    mutex_inited = _gf_false;
+        gf_boolean_t    cond_inited  = _gf_false;
+        gf_boolean_t    attr_inited  = _gf_false;
 
 	if (!this->children || this->children->next) {
 		gf_log (this->name, GF_LOG_ERROR,
@@ -1051,16 +1051,23 @@ init (xlator_t *this)
                         "pthread_cond_init failed (%d)", ret);
                 goto out;
         }
+        cond_inited = _gf_true;
 
         if ((ret = pthread_mutex_init(&priv->mutex, NULL)) != 0) {
                 gf_log (this->name, GF_LOG_ERROR,
                         "pthread_mutex_init failed (%d)", ret);
                 goto out;
         }
+        mutex_inited = _gf_true;
 
-        pthread_attr_init (&priv->w_attr);
-        ret = pthread_attr_setstacksize (&priv->w_attr,
-                                         INDEX_THREAD_STACK_SIZE);
+        if ((ret = pthread_attr_init (&w_attr)) != 0) {
+                gf_log (this->name, GF_LOG_ERROR,
+                        "pthread_attr_init failed (%d)", ret);
+                goto out;
+        }
+        attr_inited = _gf_true;
+
+        ret = pthread_attr_setstacksize (&w_attr, INDEX_THREAD_STACK_SIZE);
         if (ret == EINVAL) {
                 gf_log (this->name, GF_LOG_WARNING,
                         "Using default thread stack size");
@@ -1071,7 +1078,7 @@ init (xlator_t *this)
         INIT_LIST_HEAD (&priv->callstubs);
 
         this->private = priv;
-        ret = pthread_create (&thread, &priv->w_attr, index_worker, this);
+        ret = pthread_create (&thread, &w_attr, index_worker, this);
         if (ret) {
                 gf_log (this->name, GF_LOG_WARNING, "Failed to create "
                         "worker thread, aborting");
@@ -1079,9 +1086,17 @@ init (xlator_t *this)
         }
         ret = 0;
 out:
-        if (!this->private && priv)
-                GF_FREE (priv);
-
+        if (ret) {
+                if (cond_inited)
+                        pthread_cond_destroy (&priv->cond);
+                if (mutex_inited)
+                        pthread_mutex_destroy (&priv->mutex);
+                if (priv)
+                        GF_FREE (priv);
+                this->private = NULL;
+        }
+        if (attr_inited)
+                pthread_attr_destroy (&w_attr);
         return ret;
 }
 
@@ -1095,6 +1110,8 @@ fini (xlator_t *this)
                 goto out;
         this->private = NULL;
         LOCK_DESTROY (&priv->lock);
+        pthread_cond_destroy (&priv->cond);
+        pthread_mutex_destroy (&priv->mutex);
         GF_FREE (priv);
 out:
         return;
@@ -1166,8 +1183,7 @@ struct xlator_fops fops = {
         .unlink      = index_unlink
 };
 
-struct xlator_dumpops dumpops = {
-};
+struct xlator_dumpops dumpops;
 
 struct xlator_cbks cbks = {
         .forget         = index_forget,
