@@ -1,22 +1,12 @@
 /*
-  Copyright (c) 2006-2011 Gluster, Inc. <http://www.gluster.com>
-  This file is part of GlusterFS.
+   Copyright (c) 2006-2012 Red Hat, Inc. <http://www.redhat.com>
+   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+   This file is licensed to you under your choice of the GNU Lesser
+   General Public License, version 3 or any later version (LGPLv3 or
+   later), or the GNU General Public License, version 2 (GPLv2), in all
+   cases as published by the Free Software Foundation.
 */
-
 #ifndef _CONFIG_H
 #define _CONFIG_H
 #include "config.h"
@@ -341,8 +331,7 @@ __lock_name (pl_inode_t *pinode, const char *basename, entrylk_type type,
         if (conf) {
                 ret = -EAGAIN;
                 if (nonblock){
-                        if (lock->basename)
-                                GF_FREE ((char *)lock->basename);
+                        GF_FREE ((char *)lock->basename);
                         GF_FREE (lock);
                         goto out;
 
@@ -361,8 +350,7 @@ __lock_name (pl_inode_t *pinode, const char *basename, entrylk_type type,
         if ( __blocked_lock_conflict (dom, basename, type) && !(__owner_has_lock (dom, lock))) {
                 ret = -EAGAIN;
                 if (nonblock) {
-                        if (lock->basename)
-                                GF_FREE ((char *) lock->basename);
+                        GF_FREE ((char *) lock->basename);
                         GF_FREE (lock);
                         goto out;
 
@@ -499,8 +487,7 @@ __grant_blocked_entry_locks (xlator_t *this, pl_inode_t *pl_inode,
                 } else {
                         gf_log (this->name, GF_LOG_DEBUG,
                                 "should never happen");
-                        if (bl->basename)
-                                GF_FREE ((char *)bl->basename);
+                        GF_FREE ((char *)bl->basename);
                         GF_FREE (bl);
                 }
         }
@@ -601,8 +588,7 @@ release_entry_locks_for_transport (xlator_t *this, pl_inode_t *pinode,
 
                 STACK_UNWIND_STRICT (entrylk, lock->frame, -1, EAGAIN, NULL);
 
-                if (lock->basename)
-                        GF_FREE ((char *)lock->basename);
+                GF_FREE ((char *)lock->basename);
                 GF_FREE (lock);
 
         }
@@ -612,8 +598,7 @@ release_entry_locks_for_transport (xlator_t *this, pl_inode_t *pinode,
 
                 STACK_UNWIND_STRICT (entrylk, lock->frame, 0, 0, NULL);
 
-                if (lock->basename)
-                        GF_FREE ((char *)lock->basename);
+                GF_FREE ((char *)lock->basename);
                 GF_FREE (lock);
         }
 
