@@ -1,22 +1,12 @@
 /*
-  Copyright (c) 2008-2011 Gluster, Inc. <http://www.gluster.com>
-  This file is part of GlusterFS.
+   Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
+   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+   This file is licensed to you under your choice of the GNU Lesser
+   General Public License, version 3 or any later version (LGPLv3 or
+   later), or the GNU General Public License, version 2 (GPLv2), in all
+   cases as published by the Free Software Foundation.
 */
-
 #include <fnmatch.h>
 
 #include "quota.h"
@@ -136,6 +126,7 @@ quota_local_cleanup (xlator_t *this, quota_local_t *local)
         inode_unref (local->inode);
         LOCK_DESTROY (&local->lock);
 
+        mem_put (local);
 out:
         return 0;
 }
@@ -497,8 +488,7 @@ quota_get_limit_value (inode_t *inode, xlator_t *this, int64_t *n)
         }
 
 out:
-        if (path)
-                GF_FREE (path);
+        GF_FREE (path);
 
         return ret;
 }
@@ -2704,7 +2694,6 @@ int
 quota_setxattr (call_frame_t *frame, xlator_t *this,
                 loc_t *loc, dict_t *dict, int flags, dict_t *xdata)
 {
-        data_pair_t    *trav     = NULL;
         int             op_errno = EINVAL;
         int             op_ret   = -1;
 
@@ -2713,7 +2702,7 @@ quota_setxattr (call_frame_t *frame, xlator_t *this,
         VALIDATE_OR_GOTO (loc, err);
 
         GF_IF_INTERNAL_XATTR_GOTO ("trusted.glusterfs.quota*", dict,
-                                   trav, op_errno, err);
+                                   op_errno, err);
 
         STACK_WIND (frame, quota_setxattr_cbk,
                     FIRST_CHILD(this),
@@ -2737,7 +2726,6 @@ int
 quota_fsetxattr (call_frame_t *frame, xlator_t *this, fd_t *fd,
                  dict_t *dict, int flags, dict_t *xdata)
 {
-        data_pair_t    *trav     = NULL;
         int32_t         op_ret   = -1;
         int32_t         op_errno = EINVAL;
 
@@ -2746,7 +2734,7 @@ quota_fsetxattr (call_frame_t *frame, xlator_t *this, fd_t *fd,
         VALIDATE_OR_GOTO (fd, err);
 
         GF_IF_INTERNAL_XATTR_GOTO ("trusted.glusterfs.quota*", dict,
-                                   trav, op_errno, err);
+                                   op_errno, err);
 
         STACK_WIND (frame, quota_fsetxattr_cbk,
                     FIRST_CHILD(this),
@@ -3037,14 +3025,17 @@ quota_parse_limits (quota_priv_t *priv, xlator_t *this, dict_t *xl_options,
         char         *path      = NULL, *saveptr = NULL;
         uint64_t      value     = 0;
         limits_t     *quota_lim = NULL, *old = NULL;
+        char         *last_colon= NULL;
 
         ret = dict_get_str (xl_options, "limit-set", &str);
 
         if (str) {
-                path = strtok_r (str, ":", &saveptr);
+                path = strtok_r (str, ",", &saveptr);
 
                 while (path) {
-                        str_val = strtok_r (NULL, ",", &saveptr);
+                        last_colon = strrchr (path, ':');
+                        *last_colon = '\0';
+                        str_val = last_colon + 1;
 
                         ret = gf_string2bytesize (str_val, &value);
                         if (ret != 0)
@@ -3078,7 +3069,7 @@ quota_parse_limits (quota_priv_t *priv, xlator_t *this, dict_t *xl_options,
                         }
                         UNLOCK (&priv->lock);
 
-                        path = strtok_r (NULL, ":", &saveptr);
+                        path = strtok_r (NULL, ",", &saveptr);
                 }
         } else {
                 gf_log (this->name, GF_LOG_INFO,
