@@ -1,21 +1,13 @@
 /*
-  Copyright (c) 2007-2012 Gluster, Inc. <http://www.gluster.com>
-  This file is part of GlusterFS.
+   Copyright (c) 2007-2012 Red Hat, Inc. <http://www.redhat.com>
+   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+   This file is licensed to you under your choice of the GNU Lesser
+   General Public License, version 3 or any later version (LGPLv3 or
+   later), or the GNU General Public License, version 2 (GPLv2), in all
+   cases as published by the Free Software Foundation.
 */
+
 #ifndef _CONFIG_H
 #define _CONFIG_H
 #include "config.h"
@@ -93,7 +85,7 @@ glusterd_hooks_create_hooks_directory (char *basedir)
         priv = THIS->private;
 
         snprintf (path, sizeof (path), "%s/hooks", basedir);
-        ret = mkdir_if_missing (path, NULL);
+        ret = mkdir_p (path, 0777, _gf_true);
         if (ret) {
                 gf_log (THIS->name, GF_LOG_CRITICAL, "Unable to create %s due"
                          "to %s", path, strerror (errno));
@@ -101,7 +93,7 @@ glusterd_hooks_create_hooks_directory (char *basedir)
         }
 
         GLUSTERD_GET_HOOKS_DIR (version_dir, GLUSTERD_HOOK_VER, priv);
-        ret = mkdir_if_missing (version_dir, NULL);
+        ret = mkdir_p (version_dir, 0777, _gf_true);
         if (ret) {
                 gf_log (THIS->name, GF_LOG_CRITICAL, "Unable to create %s due "
                         "to %s", version_dir, strerror (errno));
@@ -115,7 +107,7 @@ glusterd_hooks_create_hooks_directory (char *basedir)
 
                 snprintf (path, sizeof (path), "%s/%s", version_dir,
                           cmd_subdir);
-                ret = mkdir_if_missing (path, NULL);
+                ret = mkdir_p (path, 0777, _gf_true);
                 if (ret) {
                         gf_log (THIS->name, GF_LOG_CRITICAL,
                                 "Unable to create %s due to %s",
@@ -127,7 +119,7 @@ glusterd_hooks_create_hooks_directory (char *basedir)
                      type++) {
                         snprintf (path, sizeof (path), "%s/%s/%s",
                                   version_dir, cmd_subdir, type_subdir[type]);
-                        ret = mkdir_if_missing (path, NULL);
+                        ret = mkdir_p (path, 0777, _gf_true);
                         if (ret) {
                                 gf_log (THIS->name, GF_LOG_CRITICAL,
                                         "Unable to create %s due to %s",
@@ -346,8 +338,7 @@ glusterd_hooks_run_hooks (char *hooks_path, glusterd_op_t op, dict_t *op_ctx,
 out:
         if (lines) {
                 for (lineno = 0; lineno < line_count+1; lineno++)
-                        if (lines[lineno])
-                                GF_FREE (lines[lineno]);
+                        GF_FREE (lines[lineno]);
 
                 GF_FREE (lines);
         }
@@ -437,11 +428,9 @@ glusterd_hooks_stub_cleanup (glusterd_hooks_stub_t *stub)
         if (stub->op_ctx)
                 dict_unref (stub->op_ctx);
 
-        if (stub->scriptdir)
-                GF_FREE (stub->scriptdir);
+        GF_FREE (stub->scriptdir);
 
-        if (stub)
-                GF_FREE (stub);
+        GF_FREE (stub);
 }
 
 static void*
