@@ -1,22 +1,12 @@
 /*
-  Copyright (c) 2006-2012 Gluster, Inc. <http://www.gluster.com>
-  This file is part of GlusterFS.
+   Copyright (c) 2006-2012 Red Hat, Inc. <http://www.redhat.com>
+   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+   This file is licensed to you under your choice of the GNU Lesser
+   General Public License, version 3 or any later version (LGPLv3 or
+   later), or the GNU General Public License, version 2 (GPLv2), in all
+   cases as published by the Free Software Foundation.
 */
-
 #ifndef _GLUSTERD_HOOKS_H_
 #define _GLUSTERD_HOOKS_H_
 
@@ -25,12 +15,16 @@
 #include "config.h"
 #endif
 
+#include <fnmatch.h>
+
 #define GLUSTERD_GET_HOOKS_DIR(path, version, priv) \
         snprintf (path, PATH_MAX, "%s/hooks/%d", priv->workdir,\
                   version);
 
 #define GLUSTERD_HOOK_VER       1
 
+#define GD_HOOKS_SPECIFIC_KEY   "user.*"
+
 typedef enum glusterd_commit_hook_type {
         GD_COMMIT_HOOK_NONE = 0,
         GD_COMMIT_HOOK_PRE,
@@ -55,6 +49,20 @@ typedef struct hooks_stub {
 } glusterd_hooks_stub_t;
 
 
+static inline gf_boolean_t
+is_key_glusterd_hooks_friendly (char *key)
+{
+        gf_boolean_t is_friendly = _gf_false;
+
+        /* This is very specific to hooks friendly behavior */
+        if (fnmatch (GD_HOOKS_SPECIFIC_KEY, key, FNM_NOESCAPE) == 0) {
+                gf_log (THIS->name, GF_LOG_DEBUG, "user namespace key %s", key);
+                is_friendly = _gf_true;
+        }
+
+        return is_friendly;
+}
+
 int
 glusterd_hooks_create_hooks_directory (char *basedir);
 
