@@ -1,22 +1,12 @@
 /*
-  Copyright (c) 2011 Gluster, Inc. <http://www.gluster.com>
-  This file is part of GlusterFS.
+   Copyright (c) 2011-2012 Red Hat, Inc. <http://www.redhat.com>
+   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+   This file is licensed to you under your choice of the GNU Lesser
+   General Public License, version 3 or any later version (LGPLv3 or
+   later), or the GNU General Public License, version 2 (GPLv2), in all
+   cases as published by the Free Software Foundation.
 */
-
 #ifndef _CONFIG_H
 #define _CONFIG_H
 #include "config.h"
@@ -276,6 +266,7 @@ make_georep_mountspec (gf_mount_spec_t *mspec, const char *volnames,
         char *vols            = NULL;
         char *vol             = NULL;
         char *p               = NULL;
+        char *savetok         = NULL;
         char *fa[3]           = {0,};
         size_t siz            = 0;
         int vc                = 0;
@@ -296,7 +287,7 @@ make_georep_mountspec (gf_mount_spec_t *mspec, const char *volnames,
                 goto out;
 
         for (p = vols;;) {
-                vol = strtok (p, ",");
+                vol = strtok_r (p, ",", &savetok);
                 if (!vol) {
                         GF_ASSERT (vc == 0);
                         break;
@@ -323,10 +314,10 @@ make_georep_mountspec (gf_mount_spec_t *mspec, const char *volnames,
         fa[2] = georep_mnt_desc;
 
         for (i = 0; i < 3; i++) {
-                if (fa[i])
-                        GF_FREE (fa[i]);
-                else
+                if (fa[i] == NULL)
                         ret = -1;
+                else
+                        GF_FREE (fa[i]);
         }
 
         return ret;
@@ -696,8 +687,7 @@ glusterd_do_mount (char *label, dict_t *argdict, char **path, int *op_errno)
                 *path = cookie;
         }
 
-        if (mtptemp)
-                GF_FREE (mtptemp);
+        GF_FREE (mtptemp);
 
         return ret;
 }
