@@ -2,19 +2,10 @@
   Copyright (c) 2012 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 
@@ -78,12 +69,9 @@ mountudpproc3_mnt_3_svc(dirpath **dpp, struct svc_req *req)
         return res;
 
  err:
-        if (fh)
-                GF_FREE (fh);
-        if (res)
-                GF_FREE (res);
-        if (autharr)
-                GF_FREE (autharr);
+        GF_FREE (fh);
+        GF_FREE (res);
+        GF_FREE (autharr);
         return NULL;
 }
 
