@@ -2,19 +2,10 @@
    Copyright (c) 2008-2011 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
-   GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
-   by the Free Software Foundation; either version 3 of the License,
-   or (at your option) any later version.
-
-   GlusterFS is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program.  If not, see
-   <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 
@@ -52,6 +43,8 @@ enum gf_nfs_mem_types_ {
         gf_nfs_mt_nlm4_fde,
         gf_nfs_mt_nlm4_nlmclnt,
         gf_nfs_mt_nlm4_share,
+        gf_nfs_mt_aux_gids,
+        gf_nfs_mt_inode_ctx,
         gf_nfs_mt_end
 };
 #endif
