@@ -31,7 +31,7 @@ ioc_get_priority (ioc_table_t *table, const char *path);
 struct volume_options options[];
 
 
-inline uint32_t
+static inline uint32_t
 ioc_hashfn (void *data, int len)
 {
         off_t offset;
@@ -41,7 +41,7 @@ ioc_hashfn (void *data, int len)
         return (offset >> ioc_log2_page_size);
 }
 
-inline ioc_inode_t *
+static inline ioc_inode_t *
 ioc_inode_reupdate (ioc_inode_t *ioc_inode)
 {
         ioc_table_t *table = NULL;
@@ -54,7 +54,7 @@ ioc_inode_reupdate (ioc_inode_t *ioc_inode)
         return ioc_inode;
 }
 
-inline ioc_inode_t *
+static inline ioc_inode_t *
 ioc_get_inode (dict_t *dict, char *name)
 {
         ioc_inode_t *ioc_inode      = NULL;
@@ -313,6 +313,20 @@ ioc_forget (xlator_t *this, inode_t *inode)
         return 0;
 }
 
+static int32_t
+ioc_invalidate(xlator_t *this, inode_t *inode)
+{
+	uint64_t     ioc_addr = 0;
+	ioc_inode_t *ioc_inode = NULL;
+
+	inode_ctx_get(inode, this, (uint64_t *) &ioc_addr);
+	ioc_inode = (void *) ioc_addr;
+
+	if (ioc_inode)
+		ioc_inode_flush(ioc_inode);
+
+	return 0;
+}
 
 /*
  * ioc_cache_validate_cbk -
@@ -477,7 +491,7 @@ out:
         return ret;
 }
 
-inline uint32_t
+static inline uint32_t
 is_match (const char *path, const char *pattern)
 {
         int32_t ret = 0;
@@ -539,6 +553,13 @@ ioc_open_cbk (call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
                 inode_ctx_get (fd->inode, this, &tmp_ioc_inode);
                 ioc_inode = (ioc_inode_t *)(long)tmp_ioc_inode;
 
+                //TODO: see why inode context is NULL and handle it.
+                if (!ioc_inode) {
+                        gf_log (this->name, GF_LOG_ERROR, "inode context is "
+                                "NULL (%s)", uuid_utoa (fd->inode->gfid));
+                        goto out;
+                }
+
                 ioc_table_lock (ioc_inode->table);
                 {
                         list_move_tail (&ioc_inode->inode_lru,
@@ -1483,13 +1504,9 @@ ioc_get_priority_list (const char *opt_str, struct list_head *first)
                 stripe_str = strtok_r (NULL, ",", &tmp_str);
         }
 out:
-        if (string != NULL) {
-                GF_FREE (string);
-        }
+        GF_FREE (string);
 
-        if (dup_str != NULL) {
-                GF_FREE (dup_str);
-        }
+        GF_FREE (dup_str);
 
         if (max_pri == -1) {
                 list_for_each_entry_safe (curr, tmp, first, list) {
@@ -1522,7 +1539,7 @@ mem_acct_init (xlator_t *this)
 }
 
 
-gf_boolean_t
+static gf_boolean_t
 check_cache_size_ok (xlator_t *this, uint64_t cache_size)
 {
         gf_boolean_t            ret = _gf_true;
@@ -1545,7 +1562,7 @@ check_cache_size_ok (xlator_t *this, uint64_t cache_size)
         else
                 max_cache_size = total_mem;
 
-        gf_log (this->name, GF_LOG_INFO, "Max cache size is %"PRIu64,
+        gf_log (this->name, GF_LOG_DEBUG, "Max cache size is %"PRIu64,
                 max_cache_size);
 
         if (cache_size > max_cache_size) {
@@ -1832,7 +1849,6 @@ __ioc_cache_dump (ioc_inode_t *ioc_inode, char *prefix)
         ioc_table_t *table                    = NULL;
         ioc_page_t  *page                     = NULL;
         int          i                        = 0;
-        struct tm   *tm                       = NULL;
         char         key[GF_DUMP_MAX_BUF_LEN] = {0, };
         char         timestr[256]             = {0, };
 
@@ -1843,9 +1859,9 @@ __ioc_cache_dump (ioc_inode_t *ioc_inode, char *prefix)
         table = ioc_inode->table;
 
         if (ioc_inode->cache.tv.tv_sec) {
-                tm = localtime (&ioc_inode->cache.tv.tv_sec);
-                strftime (timestr, 256, "%Y-%m-%d %H:%M:%S", tm);
-                snprintf (timestr + strlen (timestr), 256 - strlen (timestr),
+                gf_time_fmt (timestr, sizeof timestr,
+                             ioc_inode->cache.tv.tv_sec, gf_timefmt_FT);
+                snprintf (timestr + strlen (timestr), sizeof timestr - strlen (timestr),
                           ".%"GF_PRI_SUSECONDS, ioc_inode->cache.tv.tv_usec);
 
                 gf_proc_dump_write ("last-cache-validation-time", "%s",
@@ -1871,19 +1887,18 @@ int
 ioc_inode_dump (xlator_t *this, inode_t *inode)
 {
 
-        char    *path = NULL;
-        int     ret   = -1;
-        ioc_inode_t *ioc_inode = NULL;
-        char         key_prefix[GF_DUMP_MAX_BUF_LEN] = {0, };
-        uint64_t    tmp_ioc_inode = 0;
-        gf_boolean_t section_added = _gf_false;
+        char          *path                           = NULL;
+        int           ret                             = -1;
+        char          key_prefix[GF_DUMP_MAX_BUF_LEN] = {0, };
+        uint64_t      tmp_ioc_inode                   = 0;
+        ioc_inode_t   *ioc_inode                      = NULL;
+        gf_boolean_t  section_added                   = _gf_false;
+        char          uuid_str[64]                    = {0,};
 
-        if (this == NULL || inode == NULL) {
+        if (this == NULL || inode == NULL)
                 goto out;
-        }
 
-        gf_proc_dump_build_key (key_prefix, "xlator.performance.io-cache",
-                                "inode");
+        gf_proc_dump_build_key (key_prefix, "io-cache", "inode");
 
         inode_ctx_get (inode, this, &tmp_ioc_inode);
         ioc_inode = (ioc_inode_t *)(long)tmp_ioc_inode;
@@ -1911,8 +1926,8 @@ ioc_inode_dump (xlator_t *this, inode_t *inode)
                         gf_proc_dump_write ("path", "%s", path);
                         GF_FREE (path);
                 }
-                gf_proc_dump_write ("uuid", "%s", uuid_utoa
-                                    (ioc_inode->inode->gfid));
+                gf_proc_dump_write ("uuid", "%s", uuid_utoa_r
+                                    (ioc_inode->inode->gfid, uuid_str));
                 __ioc_cache_dump (ioc_inode, key_prefix);
                 __ioc_inode_waitq_dump (ioc_inode, key_prefix);
 
@@ -1942,8 +1957,8 @@ ioc_priv_dump (xlator_t *this)
                 goto out;
 
         priv = this->private;
-        gf_proc_dump_build_key (key_prefix, "xlator.performance.io-cache",
-                                "priv");
+
+        gf_proc_dump_build_key (key_prefix, "io-cache", "priv");
         gf_proc_dump_add_section (key_prefix);
         add_section = _gf_true;
 
@@ -2041,7 +2056,8 @@ struct xlator_dumpops dumpops = {
 
 struct xlator_cbks cbks = {
         .forget      = ioc_forget,
-        .release     = ioc_release
+        .release     = ioc_release,
+	.invalidate  = ioc_invalidate,
 };
 
 struct volume_options options[] = {
