@@ -804,7 +804,7 @@ ioc_frame_unwind (call_frame_t *frame)
         int32_t        copied = 0;
         struct iobref *iobref = NULL;
         struct iatt    stbuf  = {0,};
-        int32_t        op_ret = 0;
+        int32_t        op_ret = 0, op_errno = 0;
 
         GF_ASSERT (frame);
 
@@ -813,16 +813,21 @@ ioc_frame_unwind (call_frame_t *frame)
                 gf_log (frame->this->name, GF_LOG_WARNING,
                         "local is NULL");
                 op_ret = -1;
-                local->op_errno = ENOMEM;
+                op_errno = ENOMEM;
+                goto unwind;
+        }
+
+        if (local->op_ret < 0) {
+                op_ret = local->op_ret;
+                op_errno = local->op_errno;
                 goto unwind;
         }
 
         //  ioc_local_lock (local);
-        frame->local = NULL;
         iobref = iobref_new ();
         if (iobref == NULL) {
                 op_ret = -1;
-                local->op_errno = ENOMEM;
+                op_errno = ENOMEM;
         }
 
         if (list_empty (&local->fill_list)) {
@@ -839,7 +844,7 @@ ioc_frame_unwind (call_frame_t *frame)
         vector = GF_CALLOC (count, sizeof (*vector), gf_ioc_mt_iovec);
         if (vector == NULL) {
                 op_ret = -1;
-                local->op_errno = ENOMEM;
+                op_errno = ENOMEM;
         }
 
         list_for_each_entry_safe (fill, next, &local->fill_list, list) {
@@ -869,7 +874,8 @@ unwind:
 
         //  ioc_local_unlock (local);
 
-        STACK_UNWIND_STRICT (readv, frame, op_ret, local->op_errno, vector,
+        frame->local = NULL;
+        STACK_UNWIND_STRICT (readv, frame, op_ret, op_errno, vector,
                              count, &stbuf, iobref, NULL);
 
         if (iobref != NULL) {
@@ -882,7 +888,8 @@ unwind:
         }
 
         pthread_mutex_destroy (&local->local_lock);
-        mem_put (local);
+        if (local)
+                mem_put (local);
 
         return;
 }
