@@ -30,32 +30,35 @@
 
 struct mdc_conf {
 	int  timeout;
+	gf_boolean_t cache_posix_acl;
+	gf_boolean_t cache_selinux;
+	gf_boolean_t force_readdirp;
 };
 
 
-struct mdc_key {
+static struct mdc_key {
 	const char *name;
 	int         load;
 	int         check;
 } mdc_keys[] = {
 	{
 		.name = "system.posix_acl_access",
-		.load = 1,
+		.load = 0,
 		.check = 1,
 	},
 	{
 		.name = "system.posix_acl_default",
-		.load = 1,
+		.load = 0,
 		.check = 1,
 	},
 	{
 		.name = "security.selinux",
-		.load = 1,
+		.load = 0,
 		.check = 1,
 	},
 	{
 		.name = "security.capability",
-		.load = 1,
+		.load = 0,
 		.check = 1,
 	},
 	{
@@ -63,7 +66,11 @@ struct mdc_key {
 		.load = 0,
 		.check = 1,
 	},
-	{},
+        {
+                .name = NULL,
+                .load = 0,
+                .check = 0,
+        }
 };
 
 
@@ -125,6 +132,7 @@ struct mdc_local {
         loc_t   loc2;
         fd_t   *fd;
         char   *linkname;
+	char   *key;
         dict_t *xattr;
 };
 
@@ -220,8 +228,9 @@ mdc_local_wipe (xlator_t *this, mdc_local_t *local)
         if (local->fd)
                 fd_unref (local->fd);
 
-        if (local->linkname)
-                GF_FREE (local->linkname);
+        GF_FREE (local->linkname);
+
+        GF_FREE (local->key);
 
         if (local->xattr)
                 dict_unref (local->xattr);
@@ -247,8 +256,7 @@ mdc_inode_wipe (xlator_t *this, inode_t *inode)
         if (mdc->xattr)
                 dict_unref (mdc->xattr);
 
-        if (mdc->linkname)
-                GF_FREE (mdc->linkname);
+        GF_FREE (mdc->linkname);
 
         GF_FREE (mdc);
 
@@ -376,7 +384,8 @@ mdc_to_iatt (struct md_cache *mdc, struct iatt *iatt)
 
 
 int
-mdc_inode_iatt_set (xlator_t *this, inode_t *inode, struct iatt *iatt)
+mdc_inode_iatt_set_validate(xlator_t *this, inode_t *inode, struct iatt *prebuf,
+			    struct iatt *iatt)
 {
         int              ret = -1;
         struct md_cache *mdc = NULL;
@@ -392,6 +401,19 @@ mdc_inode_iatt_set (xlator_t *this, inode_t *inode, struct iatt *iatt)
                         goto unlock;
                 }
 
+		/*
+		 * Invalidate the inode if the mtime or ctime has changed
+		 * and the prebuf doesn't match the value we have cached.
+		 * TODO: writev returns with a NULL iatt due to
+		 * performance/write-behind, causing invalidation on writes.
+		 */
+		if (IA_ISREG(inode->ia_type) &&
+		    ((iatt->ia_mtime != mdc->md_mtime) ||
+		    (iatt->ia_ctime != mdc->md_ctime)))
+			if (!prebuf || (prebuf->ia_ctime != mdc->md_ctime) ||
+			    (prebuf->ia_mtime != mdc->md_mtime))
+				inode_invalidate(inode);
+
                 mdc_from_iatt (mdc, iatt);
 
                 time (&mdc->ia_time);
@@ -403,6 +425,10 @@ out:
         return ret;
 }
 
+int mdc_inode_iatt_set(xlator_t *this, inode_t *inode, struct iatt *iatt)
+{
+	return mdc_inode_iatt_set_validate(this, inode, NULL, iatt);
+}
 
 int
 mdc_inode_iatt_get (xlator_t *this, inode_t *inode, struct iatt *iatt)
@@ -437,7 +463,7 @@ struct updatedict {
 	int ret;
 };
 
-static void
+static int
 updatefn(dict_t *dict, char *key, data_t *value, void *data)
 {
 	struct updatedict *u = data;
@@ -454,17 +480,18 @@ updatefn(dict_t *dict, char *key, data_t *value, void *data)
 			u->dict = dict_new();
 			if (!u->dict) {
 				u->ret = -1;
-				return;
+				return -1;
 			}
 		}
 
 		if (dict_set(u->dict, key, value) < 0) {
 			u->ret = -1;
-			return;
+			return -1;
 		}
 
 		break;
 	}
+        return 0;
 }
 
 static int
@@ -560,6 +587,31 @@ out:
 }
 
 
+int
+mdc_inode_xatt_unset (xlator_t *this, inode_t *inode, char *name)
+{
+        int              ret = -1;
+        struct md_cache *mdc = NULL;
+
+        mdc = mdc_inode_prep (this, inode);
+        if (!mdc)
+                goto out;
+
+        if (!name)
+                goto out;
+
+        LOCK (&mdc->lock);
+        {
+		dict_del (mdc->xattr, name);
+        }
+        UNLOCK (&mdc->lock);
+
+        ret = 0;
+out:
+        return ret;
+}
+
+
 int
 mdc_inode_xatt_get (xlator_t *this, inode_t *inode, dict_t **dict)
 {
@@ -574,13 +626,15 @@ mdc_inode_xatt_get (xlator_t *this, inode_t *inode, dict_t **dict)
 
         LOCK (&mdc->lock);
         {
+                ret = 0;
+		/* Missing xattr only means no keys were there, i.e
+		   a negative cache for the "loaded" keys
+		*/
                 if (!mdc->xattr)
                         goto unlock;
 
                 if (dict)
                         *dict = dict_ref (mdc->xattr);
-
-                ret = 0;
         }
 unlock:
         UNLOCK (&mdc->lock);
@@ -590,6 +644,46 @@ out:
 }
 
 
+int
+mdc_inode_iatt_invalidate (xlator_t *this, inode_t *inode)
+{
+        int              ret = -1;
+        struct md_cache *mdc = NULL;
+
+        if (mdc_inode_ctx_get (this, inode, &mdc) != 0)
+                goto out;
+
+        LOCK (&mdc->lock);
+        {
+		mdc->ia_time = 0;
+        }
+        UNLOCK (&mdc->lock);
+
+out:
+        return ret;
+}
+
+
+int
+mdc_inode_xatt_invalidate (xlator_t *this, inode_t *inode)
+{
+        int              ret = -1;
+        struct md_cache *mdc = NULL;
+
+        if (mdc_inode_ctx_get (this, inode, &mdc) != 0)
+                goto out;
+
+        LOCK (&mdc->lock);
+        {
+		mdc->xa_time = 0;
+        }
+        UNLOCK (&mdc->lock);
+
+out:
+        return ret;
+}
+
+
 void
 mdc_load_reqs (xlator_t *this, dict_t *dict)
 {
@@ -623,7 +717,7 @@ is_mdc_key_satisfied (const char *key)
 		return 0;
 
 	for (mdc_key = mdc_keys[i].name; (mdc_key = mdc_keys[i].name); i++) {
-		if (!mdc_keys[i].check)
+		if (!mdc_keys[i].load)
 			continue;
 		if (strcmp (mdc_key, key) == 0)
 			return 1;
@@ -633,13 +727,15 @@ is_mdc_key_satisfied (const char *key)
 }
 
 
-static void
+static int
 checkfn (dict_t *this, char *key, data_t *value, void *data)
 {
         struct checkpair *pair = data;
 
 	if (!is_mdc_key_satisfied (key))
 		pair->ret = 0;
+
+        return 0;
 }
 
 
@@ -695,6 +791,7 @@ mdc_lookup (call_frame_t *frame, xlator_t *this, loc_t *loc,
         struct iatt  stbuf = {0, };
         struct iatt  postparent = {0, };
         dict_t      *xattr_rsp = NULL;
+        dict_t      *xattr_alloc = NULL;
         mdc_local_t *local = NULL;
 
 
@@ -702,6 +799,13 @@ mdc_lookup (call_frame_t *frame, xlator_t *this, loc_t *loc,
         if (!local)
                 goto uncached;
 
+	if (!loc->name)
+		/* A nameless discovery is dangerous to cache. We
+		   perform nameless lookup with the intention of
+		   re-establishing an inode "properly"
+		*/
+		goto uncached;
+
         loc_copy (&local->loc, loc);
 
         ret = mdc_inode_iatt_get (this, loc->inode, &stbuf);
@@ -726,6 +830,8 @@ mdc_lookup (call_frame_t *frame, xlator_t *this, loc_t *loc,
         return 0;
 
 uncached:
+	if (!xdata)
+		xdata = xattr_alloc = dict_new ();
 	if (xdata)
 		mdc_load_reqs (this, xdata);
 
@@ -734,7 +840,8 @@ uncached:
 
         if (xattr_rsp)
                 dict_unref (xattr_rsp);
-
+	if (xattr_alloc)
+		dict_unref (xattr_alloc);
         return 0;
 }
 
@@ -857,7 +964,7 @@ mdc_truncate_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         if (!local)
                 goto out;
 
-        mdc_inode_iatt_set (this, local->loc.inode, postbuf);
+        mdc_inode_iatt_set_validate(this, local->loc.inode, prebuf, postbuf);
 
 out:
         MDC_STACK_UNWIND (truncate, frame, op_ret, op_errno, prebuf, postbuf,
@@ -899,7 +1006,7 @@ mdc_ftruncate_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         if (!local)
                 goto out;
 
-        mdc_inode_iatt_set (this, local->fd->inode, postbuf);
+        mdc_inode_iatt_set_validate(this, local->fd->inode, prebuf, postbuf);
 
 out:
         MDC_STACK_UNWIND (ftruncate, frame, op_ret, op_errno, prebuf, postbuf,
@@ -1375,7 +1482,7 @@ mdc_writev_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         if (!local)
                 goto out;
 
-        mdc_inode_iatt_set (this, local->fd->inode, postbuf);
+        mdc_inode_iatt_set_validate(this, local->fd->inode, prebuf, postbuf);
 
 out:
         MDC_STACK_UNWIND (writev, frame, op_ret, op_errno, prebuf, postbuf,
@@ -1420,7 +1527,7 @@ mdc_setattr_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         if (!local)
                 goto out;
 
-        mdc_inode_iatt_set (this, local->loc.inode, postbuf);
+	mdc_inode_iatt_set_validate(this, local->loc.inode, prebuf, postbuf);
 
 out:
         MDC_STACK_UNWIND (setattr, frame, op_ret, op_errno, prebuf, postbuf,
@@ -1462,7 +1569,7 @@ mdc_fsetattr_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         if (!local)
                 goto out;
 
-        mdc_inode_iatt_set (this, local->fd->inode, postbuf);
+        mdc_inode_iatt_set_validate(this, local->fd->inode, prebuf, postbuf);
 
 out:
         MDC_STACK_UNWIND (fsetattr, frame, op_ret, op_errno, prebuf, postbuf,
@@ -1504,7 +1611,7 @@ mdc_fsync_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         if (!local)
                 goto out;
 
-        mdc_inode_iatt_set (this, local->fd->inode, postbuf);
+        mdc_inode_iatt_set_validate(this, local->fd->inode, prebuf, postbuf);
 
 out:
         MDC_STACK_UNWIND (fsync, frame, op_ret, op_errno, prebuf, postbuf,
@@ -1547,6 +1654,8 @@ mdc_setxattr_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
         mdc_inode_xatt_update (this, local->loc.inode, local->xattr);
 
+	mdc_inode_iatt_invalidate (this, local->loc.inode);
+
 out:
         MDC_STACK_UNWIND (setxattr, frame, op_ret, op_errno, xdata);
 
@@ -1588,6 +1697,7 @@ mdc_fsetxattr_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
         mdc_inode_xatt_update (this, local->fd->inode, local->xattr);
 
+	mdc_inode_iatt_invalidate (this, local->fd->inode);
 out:
         MDC_STACK_UNWIND (fsetxattr, frame, op_ret, op_errno, xdata);
 
@@ -1640,6 +1750,7 @@ mdc_getxattr (call_frame_t *frame, xlator_t *this, loc_t *loc, const char *key,
               dict_t *xdata)
 {
         int           ret;
+	int           op_errno = ENODATA;
         mdc_local_t  *local = NULL;
 	dict_t       *xattr = NULL;
 
@@ -1656,10 +1767,12 @@ mdc_getxattr (call_frame_t *frame, xlator_t *this, loc_t *loc, const char *key,
 	if (ret != 0)
 		goto uncached;
 
-	if (!dict_get (xattr, (char *)key))
-		goto uncached;
+	if (!xattr || !dict_get (xattr, (char *)key)) {
+		ret = -1;
+		op_errno = ENODATA;
+	}
 
-        MDC_STACK_UNWIND (getxattr, frame, 0, 0, xattr, xdata);
+        MDC_STACK_UNWIND (getxattr, frame, ret, op_errno, xattr, xdata);
 
         return 0;
 
@@ -1701,6 +1814,7 @@ mdc_fgetxattr (call_frame_t *frame, xlator_t *this, fd_t *fd, const char *key,
         int           ret;
         mdc_local_t  *local = NULL;
 	dict_t       *xattr = NULL;
+	int           op_errno = ENODATA;
 
         local = mdc_local_get (frame);
         if (!local)
@@ -1715,10 +1829,12 @@ mdc_fgetxattr (call_frame_t *frame, xlator_t *this, fd_t *fd, const char *key,
 	if (ret != 0)
 		goto uncached;
 
-	if (!dict_get (xattr, (char *)key))
-		goto uncached;
+	if (!xattr || !dict_get (xattr, (char *)key)) {
+		ret = -1;
+		op_errno = ENODATA;
+	}
 
-        MDC_STACK_UNWIND (fgetxattr, frame, 0, 0, xattr, xdata);
+        MDC_STACK_UNWIND (fgetxattr, frame, ret, op_errno, xattr, xdata);
 
         return 0;
 
@@ -1729,6 +1845,97 @@ uncached:
         return 0;
 }
 
+int
+mdc_removexattr_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+		     int32_t op_ret, int32_t op_errno, dict_t *xdata)
+{
+        mdc_local_t  *local = NULL;
+
+        local = frame->local;
+
+        if (op_ret != 0)
+                goto out;
+
+        if (!local)
+                goto out;
+
+	if (local->key)
+		mdc_inode_xatt_unset (this, local->loc.inode, local->key);
+	else
+		mdc_inode_xatt_invalidate (this, local->loc.inode);
+
+	mdc_inode_iatt_invalidate (this, local->loc.inode);
+out:
+        MDC_STACK_UNWIND (removexattr, frame, op_ret, op_errno, xdata);
+
+        return 0;
+}
+
+
+int
+mdc_removexattr (call_frame_t *frame, xlator_t *this, loc_t *loc,
+		 const char *name, dict_t *xdata)
+{
+        mdc_local_t  *local = NULL;
+
+        local = mdc_local_get (frame);
+
+	loc_copy (&local->loc, loc);
+
+	local->key = gf_strdup (name);
+
+        STACK_WIND (frame, mdc_removexattr_cbk,
+                    FIRST_CHILD(this), FIRST_CHILD(this)->fops->removexattr,
+                    loc, name, xdata);
+        return 0;
+}
+
+
+int
+mdc_fremovexattr_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+		      int32_t op_ret, int32_t op_errno, dict_t *xdata)
+{
+        mdc_local_t  *local = NULL;
+
+        local = frame->local;
+
+        if (op_ret != 0)
+                goto out;
+
+        if (!local)
+                goto out;
+
+	if (local->key)
+		mdc_inode_xatt_unset (this, local->fd->inode, local->key);
+	else
+		mdc_inode_xatt_invalidate (this, local->fd->inode);
+
+	mdc_inode_iatt_invalidate (this, local->fd->inode);
+out:
+        MDC_STACK_UNWIND (fremovexattr, frame, op_ret, op_errno, xdata);
+
+        return 0;
+}
+
+
+int
+mdc_fremovexattr (call_frame_t *frame, xlator_t *this, fd_t *fd,
+		  const char *name, dict_t *xdata)
+{
+        mdc_local_t  *local = NULL;
+
+        local = mdc_local_get (frame);
+
+	local->fd = fd_ref (fd);
+
+	local->key = gf_strdup (name);
+
+        STACK_WIND (frame, mdc_fremovexattr_cbk,
+                    FIRST_CHILD(this), FIRST_CHILD(this)->fops->fremovexattr,
+                    fd, name, xdata);
+        return 0;
+}
+
 
 int
 mdc_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
@@ -1756,18 +1963,42 @@ int
 mdc_readdirp (call_frame_t *frame, xlator_t *this, fd_t *fd,
 	      size_t size, off_t offset, dict_t *xdata)
 {
+	dict_t *xattr_alloc = NULL;
+
+	if (!xdata)
+		xdata = xattr_alloc = dict_new ();
+	if (xdata)
+		mdc_load_reqs (this, xdata);
+
 	STACK_WIND (frame, mdc_readdirp_cbk,
 		    FIRST_CHILD (this), FIRST_CHILD (this)->fops->readdirp,
 		    fd, size, offset, xdata);
+	if (xattr_alloc)
+		dict_unref (xattr_alloc);
 	return 0;
 }
 
+int
+mdc_readdir_cbk(call_frame_t *frame, void *cookie, xlator_t *this, int op_ret,
+		int op_errno, gf_dirent_t *entries, dict_t *xdata)
+{
+	STACK_UNWIND_STRICT (readdir, frame, op_ret, op_errno, entries, xdata);
+	return 0;
+}
 
 int
 mdc_readdir (call_frame_t *frame, xlator_t *this, fd_t *fd,
 	     size_t size, off_t offset, dict_t *xdata)
 {
         int need_unref = 0;
+	struct mdc_conf *conf = this->private;
+
+	if (!conf->force_readdirp) {
+		STACK_WIND(frame, mdc_readdir_cbk, FIRST_CHILD(this),
+			   FIRST_CHILD(this)->fops->readdir, fd, size, offset,
+			   xdata);
+		return 0;
+	}
 
 	if (!xdata) {
                 xdata = dict_new ();
@@ -1777,9 +2008,9 @@ mdc_readdir (call_frame_t *frame, xlator_t *this, fd_t *fd,
         if (xdata)
 		mdc_load_reqs (this, xdata);
 
-	STACK_WIND (frame, mdc_readdirp_cbk,
-		    FIRST_CHILD (this), FIRST_CHILD (this)->fops->readdirp,
-		    fd, size, offset, xdata);
+	STACK_WIND(frame, mdc_readdirp_cbk, FIRST_CHILD(this),
+		   FIRST_CHILD(this)->fops->readdirp, fd, size, offset,
+		   xdata);
 
         if (need_unref && xdata)
                 dict_unref (xdata);
@@ -1797,6 +2028,35 @@ mdc_forget (xlator_t *this, inode_t *inode)
 }
 
 
+int
+is_strpfx (const char *str1, const char *str2)
+{
+	/* is one of the string a prefix of the other? */
+	int i;
+
+	for (i = 0; str1[i] == str2[i]; i++) {
+		if (!str1[i] || !str2[i])
+			break;
+	}
+
+	return !(str1[i] && str2[i]);
+}
+
+
+int
+mdc_key_load_set (struct mdc_key *keys, char *pattern, gf_boolean_t val)
+{
+	struct mdc_key *key = NULL;
+
+	for (key = keys; key->name; key++) {
+		if (is_strpfx (key->name, pattern))
+			key->load = val;
+	}
+
+	return 0;
+}
+
+
 int
 reconfigure (xlator_t *this, dict_t *options)
 {
@@ -1804,7 +2064,16 @@ reconfigure (xlator_t *this, dict_t *options)
 
 	conf = this->private;
 
-	GF_OPTION_RECONF ("timeout", conf->timeout, options, int32, out);
+	GF_OPTION_RECONF ("md-cache-timeout", conf->timeout, options, int32, out);
+
+	GF_OPTION_RECONF ("cache-selinux", conf->cache_selinux, options, bool, out);
+	mdc_key_load_set (mdc_keys, "security.", conf->cache_selinux);
+
+	GF_OPTION_RECONF ("cache-posix-acl", conf->cache_posix_acl, options, bool, out);
+	mdc_key_load_set (mdc_keys, "system.posix_acl_", conf->cache_posix_acl);
+
+	GF_OPTION_RECONF("force-readdirp", conf->force_readdirp, options, bool, out);
+
 out:
 	return 0;
 }
@@ -1830,8 +2099,15 @@ init (xlator_t *this)
 		return -1;
 	}
 
-        GF_OPTION_INIT ("timeout", conf->timeout, int32, out);
+        GF_OPTION_INIT ("md-cache-timeout", conf->timeout, int32, out);
 
+	GF_OPTION_INIT ("cache-selinux", conf->cache_selinux, bool, out);
+	mdc_key_load_set (mdc_keys, "security.", conf->cache_selinux);
+
+	GF_OPTION_INIT ("cache-posix-acl", conf->cache_posix_acl, bool, out);
+	mdc_key_load_set (mdc_keys, "system.posix_acl_", conf->cache_posix_acl);
+
+	GF_OPTION_INIT("force-readdirp", conf->force_readdirp, bool, out);
 out:
 	this->private = conf;
 
@@ -1869,6 +2145,8 @@ struct xlator_fops fops = {
         .fsetxattr   = mdc_fsetxattr,
         .getxattr    = mdc_getxattr,
         .fgetxattr   = mdc_fgetxattr,
+	.removexattr = mdc_removexattr,
+	.fremovexattr= mdc_fremovexattr,
 	.readdirp    = mdc_readdirp,
 	.readdir     = mdc_readdir
 };
@@ -1879,11 +2157,26 @@ struct xlator_cbks cbks = {
 };
 
 struct volume_options options[] = {
-        { .key = {"timeout"},
+	{ .key = {"cache-selinux"},
+	  .type = GF_OPTION_TYPE_BOOL,
+	  .default_value = "false",
+	},
+	{ .key = {"cache-posix-acl"},
+	  .type = GF_OPTION_TYPE_BOOL,
+	  .default_value = "false",
+	},
+        { .key = {"md-cache-timeout"},
           .type = GF_OPTION_TYPE_INT,
           .min = 0,
           .max = 60,
           .default_value = "1",
           .description = "Time period after which cache has to be refreshed",
         },
+	{ .key = {"force-readdirp"},
+	  .type = GF_OPTION_TYPE_BOOL,
+	  .default_value = "true",
+	  .description = "Convert all readdir requests to readdirplus to "
+			 "collect stat info on each entry.",
+	},
+    { .key = {NULL} },
 };
