@@ -815,12 +815,16 @@ client_create (call_frame_t *frame, xlator_t *this, loc_t *loc,  int32_t flags,
                 goto out;
 
         args.loc = loc;
-        args.flags = flags;
         args.mode = mode;
         args.fd = fd;
         args.umask = umask;
         args.xdata = xdata;
 
+        if (!conf->filter_o_direct)
+                args.flags = flags;
+        else
+                args.flags = (flags & ~O_DIRECT);
+
         proc = &conf->fops->proctable[GF_FOP_CREATE];
         if (!proc) {
                 gf_log (this->name, GF_LOG_ERROR,
@@ -854,10 +858,14 @@ client_open (call_frame_t *frame, xlator_t *this, loc_t *loc,
                 goto out;
 
         args.loc = loc;
-        args.flags = flags;
         args.fd = fd;
         args.xdata = xdata;
 
+        if (!conf->filter_o_direct)
+                args.flags = flags;
+        else
+                args.flags = (flags & ~O_DIRECT);
+
         proc = &conf->fops->proctable[GF_FOP_OPEN];
         if (!proc) {
                 gf_log (this->name, GF_LOG_ERROR,
@@ -1269,8 +1277,7 @@ client_set_remote_options (char *value, xlator_t *this)
 
         ret = _gf_true;
 out:
-        if (dup_value)
-                GF_FREE (dup_value);
+        GF_FREE (dup_value);
 
         return ret;
 }
@@ -2111,6 +2118,11 @@ client_rpc_notify (struct rpc_clnt *rpc, void *mydata, rpc_clnt_event_t event,
                 conf->connected = 0;
                 conf->skip_notify = 0;
 
+		if (conf->quick_reconnect) {
+			conf->quick_reconnect = 0;
+			rpc_clnt_start (conf->rpc);
+		}
+
                 break;
 
         default:
@@ -2194,6 +2206,9 @@ build_client_config (xlator_t *this, clnt_conf_t *conf)
                 gf_log (this->name, GF_LOG_WARNING,
                         "option 'remote-subvolume' not given");
 
+        GF_OPTION_INIT ("filter-O_DIRECT", conf->filter_o_direct,
+                        bool, out);
+
         ret = 0;
 out:
         return ret;
@@ -2278,7 +2293,8 @@ client_init_rpc (xlator_t *this)
         conf->handshake = &clnt_handshake_prog;
         conf->dump      = &clnt_dump_prog;
 
-        ret = rpcclnt_cbk_program_register (conf->rpc, &gluster_cbk_prog);
+        ret = rpcclnt_cbk_program_register (conf->rpc, &gluster_cbk_prog,
+                                            this);
         if (ret) {
                 gf_log (this->name, GF_LOG_ERROR,
                         "failed to register callback program");
@@ -2377,6 +2393,9 @@ reconfigure (xlator_t *this, dict_t *options)
                 }
         }
 
+        GF_OPTION_RECONF ("filter-O_DIRECT", conf->filter_o_direct,
+                          options, bool, out);
+
         ret = client_init_grace_timer (this, options, conf);
         if (ret)
                 goto out;
@@ -2701,16 +2720,27 @@ struct volume_options options[] = {
         { .key   = {"lk-heal"},
           .type  = GF_OPTION_TYPE_BOOL,
           .default_value = "off",
+          .description = "Enables or disables the lock heal."
         },
         { .key   = {"grace-timeout"},
           .type  = GF_OPTION_TYPE_INT,
           .min   = 10,
-          .max   = 1800
+          .max   = 1800,
+          .description = "Sets the grace-timeout value. Valid range 10-1800."
         },
         {.key  = {"tcp-window-size"},
          .type = GF_OPTION_TYPE_SIZET,
          .min  = GF_MIN_SOCKET_WINDOW_SIZE,
-         .max  = GF_MAX_SOCKET_WINDOW_SIZE
+         .max  = GF_MAX_SOCKET_WINDOW_SIZE,
+         .description = "Specifies the window size for tcp socket."
+        },
+        { .key   = {"filter-O_DIRECT"},
+          .type  = GF_OPTION_TYPE_BOOL,
+          .default_value = "disable",
+          .description = "If enabled, in open() and creat() calls, O_DIRECT "
+          "flag will be filtered at the client protocol level so server will "
+          "still continue to cache the file. This works similar to NFS's "
+          "behavior of O_DIRECT",
         },
         { .key   = {NULL} },
 };
