@@ -1,22 +1,12 @@
 /*
-  Copyright (c) 2006-2012 Red Hat, Inc. <http://www.redhat.com>
-  This file is part of GlusterFS.
+   Copyright (c) 2006-2012 Red Hat, Inc. <http://www.redhat.com>
+   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+   This file is licensed to you under your choice of the GNU Lesser
+   General Public License, version 3 or any later version (LGPLv3 or
+   later), or the GNU General Public License, version 2 (GPLv2), in all
+   cases as published by the Free Software Foundation.
 */
-
 #ifndef _CONFIG_H
 #define _CONFIG_H
 #include "config.h"
@@ -31,6 +21,47 @@
 #include <libaio.h>
 
 
+void
+__posix_fd_set_odirect (fd_t *fd, struct posix_fd *pfd, int opflags,
+			off_t offset, size_t size)
+{
+	int odirect = 0;
+	int flags = 0;
+	int ret = 0;
+
+	odirect = pfd->odirect;
+
+	if ((fd->flags|opflags) & O_DIRECT) {
+		/* if instructed, use O_DIRECT always */
+		odirect = 1;
+	} else {
+		/* else use O_DIRECT when feasible */
+		if ((offset|size) & 0xfff)
+			odirect = 0;
+		else
+			odirect = 1;
+	}
+
+	if (!odirect && pfd->odirect) {
+		flags = fcntl (pfd->fd, F_GETFL);
+		ret = fcntl (pfd->fd, F_SETFL, (flags & (~O_DIRECT)));
+		pfd->odirect = 0;
+	}
+
+	if (odirect && !pfd->odirect) {
+		flags = fcntl (pfd->fd, F_GETFL);
+		ret = fcntl (pfd->fd, F_SETFL, (flags | O_DIRECT));
+		pfd->odirect = 1;
+	}
+
+	if (ret) {
+		gf_log (THIS->name, GF_LOG_WARNING,
+			"fcntl() failed (%s). fd=%d flags=%d pfd->odirect=%d",
+			strerror (errno), pfd->fd, flags, pfd->odirect);
+	}
+}
+
+
 struct posix_aio_cb {
         struct iocb     iocb;
         call_frame_t   *frame;
@@ -49,7 +80,6 @@ posix_aio_readv_complete (struct posix_aio_cb *paiocb, int res, int res2)
         call_frame_t   *frame = NULL;
         xlator_t       *this = NULL;
         struct iobuf   *iobuf = NULL;
-        struct iatt     prebuf = {0,};
         struct iatt     postbuf = {0,};
         int             _fd = -1;
         int             op_ret = -1;
@@ -65,7 +95,6 @@ posix_aio_readv_complete (struct posix_aio_cb *paiocb, int res, int res2)
         this = frame->this;
         priv = this->private;
         iobuf = paiocb->iobuf;
-        prebuf = paiocb->prebuf;
         _fd = paiocb->fd;
         offset = paiocb->offset;
 
@@ -154,7 +183,7 @@ posix_aio_readv (call_frame_t *frame, xlator_t *this, fd_t *fd,
 
         priv = this->private;
 
-        ret = posix_fd_ctx_get_off (fd, this, &pfd, offset);
+        ret = posix_fd_ctx_get (fd, this, &pfd);
         if (ret < 0) {
                 op_errno = -ret;
                 gf_log (this->name, GF_LOG_WARNING,
@@ -198,7 +227,14 @@ posix_aio_readv (call_frame_t *frame, xlator_t *this, fd_t *fd,
 
         iocb = &paiocb->iocb;
 
-        ret = io_submit (priv->ctxp, 1, &iocb);
+	LOCK (&fd->lock);
+	{
+		__posix_fd_set_odirect (fd, pfd, flags, offset, size);
+
+		ret = io_submit (priv->ctxp, 1, &iocb);
+	}
+	UNLOCK (&fd->lock);
+
         if (ret != 1) {
                 gf_log (this->name, GF_LOG_ERROR,
                         "io_submit() returned %d", ret);
@@ -304,7 +340,7 @@ posix_aio_writev (call_frame_t *frame, xlator_t *this, fd_t *fd,
 
         priv = this->private;
 
-        ret = posix_fd_ctx_get_off (fd, this, &pfd, offset);
+        ret = posix_fd_ctx_get (fd, this, &pfd);
         if (ret < 0) {
                 op_errno = -ret;
                 gf_log (this->name, GF_LOG_WARNING,
@@ -346,7 +382,15 @@ posix_aio_writev (call_frame_t *frame, xlator_t *this, fd_t *fd,
 	}
 
 
-        ret = io_submit (priv->ctxp, 1, &iocb);
+	LOCK (&fd->lock);
+	{
+		__posix_fd_set_odirect (fd, pfd, flags, offset,
+					iov_length (iov, count));
+
+		ret = io_submit (priv->ctxp, 1, &iocb);
+	}
+	UNLOCK (&fd->lock);
+
         if (ret != 1) {
                 gf_log (this->name, GF_LOG_ERROR,
                         "io_submit() returned %d", ret);
@@ -516,4 +560,14 @@ posix_aio_off (xlator_t *this)
         return 0;
 }
 
+void
+__posix_fd_set_odirect (fd_t *fd, struct posix_fd *pfd, int opflags,
+                        off_t offset, size_t size)
+{
+        xlator_t        *this = THIS;
+        gf_log (this->name, GF_LOG_INFO,
+                "Linux AIO not available at build-time."
+                " Continuing with synchronous IO");
+        return;
+}
 #endif
