@@ -1,22 +1,12 @@
 /*
-   Copyright (c) 2011 Gluster, Inc. <http://www.gluster.com>
+   Copyright (c) 2011-2012 Red Hat, Inc. <http://www.redhat.com>
    This file is part of GlusterFS.
 
-   GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
-   by the Free Software Foundation; either version 3 of the License,
-   or (at your option) any later version.
-
-   GlusterFS is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program.  If not, see
-   <http://www.gnu.org/licenses/>.
+   This file is licensed to you under your choice of the GNU Lesser
+   General Public License, version 3 or any later version (LGPLv3 or
+   later), or the GNU General Public License, version 2 (GPLv2), in all
+   cases as published by the Free Software Foundation.
 */
-
 #ifndef _POSIX_HANDLE_H
 #define _POSIX_HANDLE_H
 
@@ -148,4 +138,6 @@ int posix_handle_init (xlator_t *this);
 int posix_create_link_if_gfid_exists (xlator_t *this, uuid_t gfid,
                                       char *real_path);
 
+int
+posix_handle_trash_init (xlator_t *this);
 #endif /* !_POSIX_HANDLE_H */
