@@ -1,22 +1,12 @@
 /*
-  Copyright (c) 2006-2011 Gluster, Inc. <http://www.gluster.com>
-  This file is part of GlusterFS.
+   Copyright (c) 2006-2012 Red Hat, Inc. <http://www.redhat.com>
+   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+   This file is licensed to you under your choice of the GNU Lesser
+   General Public License, version 3 or any later version (LGPLv3 or
+   later), or the GNU General Public License, version 2 (GPLv2), in all
+   cases as published by the Free Software Foundation.
 */
-
 #ifndef _CONFIG_H
 #define _CONFIG_H
 #include "config.h"
@@ -56,14 +46,6 @@
 #include "hashfn.h"
 #include <fnmatch.h>
 
-typedef struct {
-        xlator_t    *this;
-        const char  *real_path;
-        dict_t      *xattr;
-        struct iatt *stbuf;
-        loc_t       *loc;
-} posix_xattr_filler_t;
-
 char *marker_xattrs[] = {"trusted.glusterfs.quota.*",
                          "trusted.glusterfs.*.xtime",
                          NULL};
@@ -117,7 +99,7 @@ out:
         return ignore;
 }
 
-static void
+static int
 _posix_xattr_get_set (dict_t *xattr_req,
                       char *key,
                       data_t *data,
@@ -187,19 +169,13 @@ _posix_xattr_get_set (dict_t *xattr_req,
                 err:
                         if (_fd != -1)
                                 close (_fd);
-                        if (databuf)
-                                GF_FREE (databuf);
+                        GF_FREE (databuf);
                 }
         } else if (!strcmp (key, GLUSTERFS_OPEN_FD_COUNT)) {
                 loc = filler->loc;
-                if (loc && !list_empty (&loc->inode->fd_list)) {
-                        ret = dict_set_uint32 (filler->xattr, key, 1);
-                        if (ret < 0)
-                                gf_log (filler->this->name, GF_LOG_WARNING,
-                                        "Failed to set dictionary value for %s",
-                                        key);
-                } else {
-                        ret = dict_set_uint32 (filler->xattr, key, 0);
+                if (loc) {
+                        ret = dict_set_uint32 (filler->xattr, key,
+                                               loc->inode->fd_count);
                         if (ret < 0)
                                 gf_log (filler->this->name, GF_LOG_WARNING,
                                         "Failed to set dictionary value for %s",
@@ -212,16 +188,16 @@ _posix_xattr_get_set (dict_t *xattr_req,
                         value = GF_CALLOC (1, xattr_size + 1,
                                            gf_posix_mt_char);
                         if (!value)
-                                return;
+                                return -1;
 
-                        xattr_size = sys_lgetxattr (filler->real_path, key,
-                                                    value, xattr_size);
+                        xattr_size = sys_lgetxattr (filler->real_path, key, value,
+                                                    xattr_size);
                         if (xattr_size <= 0) {
                                 gf_log (filler->this->name, GF_LOG_WARNING,
                                         "getxattr failed. path: %s, key: %s",
                                         filler->real_path, key);
                                 GF_FREE (value);
-                                return;
+                                return -1;
                         }
 
                         value[xattr_size] = '\0';
@@ -236,7 +212,7 @@ _posix_xattr_get_set (dict_t *xattr_req,
                 }
         }
 out:
-        return;
+        return 0;
 }
 
 
@@ -244,7 +220,7 @@ int
 posix_fill_gfid_path (xlator_t *this, const char *path, struct iatt *iatt)
 {
         int ret = 0;
-        ssize_t  size = 0;
+        ssize_t size = 0;
 
         if (!iatt)
                 return 0;
@@ -264,7 +240,7 @@ int
 posix_fill_gfid_fd (xlator_t *this, int fd, struct iatt *iatt)
 {
         int ret = 0;
-        ssize_t  size = 0;
+        ssize_t size = 0;
 
         if (!iatt)
                 return 0;
@@ -347,11 +323,21 @@ posix_istat (xlator_t *this, uuid_t gfid, const char *basename,
 
         ret = lstat (real_path, &lstatbuf);
 
-        if (ret == -1) {
-                if (errno != ENOENT && errno != ELOOP)
-                        gf_log (this->name, GF_LOG_WARNING,
-                                "lstat failed on %s (%s)",
-                                real_path, strerror (errno));
+        if (ret != 0) {
+                if (ret == -1) {
+                        if (errno != ENOENT && errno != ELOOP)
+                                gf_log (this->name, GF_LOG_WARNING,
+                                        "lstat failed on %s (%s)",
+                                        real_path, strerror (errno));
+                } else {
+                        // may be some backend filesystem issue
+                        gf_log (this->name, GF_LOG_ERROR, "lstat failed on "
+                                "%s and return value is %d instead of -1. "
+                                "Please see dmesg output to check whether the "
+                                "failure is due to backend filesystem issue",
+                                real_path, ret);
+                        ret = -1;
+                }
                 goto out;
         }
 
@@ -395,11 +381,21 @@ posix_pstat (xlator_t *this, uuid_t gfid, const char *path,
 
         ret = lstat (path, &lstatbuf);
 
-        if (ret == -1) {
-                if (errno != ENOENT)
-                        gf_log (this->name, GF_LOG_WARNING,
-                                "lstat failed on %s (%s)",
-                                path, strerror (errno));
+        if (ret != 0) {
+                if (ret == -1) {
+                        if (errno != ENOENT)
+                                gf_log (this->name, GF_LOG_WARNING,
+                                        "lstat failed on %s (%s)",
+                                        path, strerror (errno));
+                } else {
+                        // may be some backend filesytem issue
+                        gf_log (this->name, GF_LOG_ERROR, "lstat failed on "
+                                "%s and return value is %d instead of -1. "
+                                "Please see dmesg output to check whether the "
+                                "failure is due to backend filesystem issue",
+                                path, ret);
+                        ret = -1;
+                }
                 goto out;
         }
 
@@ -503,8 +499,8 @@ out:
 
 
 int
-posix_set_file_contents (xlator_t *this, const char *path, data_pair_t *trav,
-                         int flags)
+posix_set_file_contents (xlator_t *this, const char *path, char *keyp,
+                         data_t *value, int flags)
 {
         char *      key                        = NULL;
         char        real_path[PATH_MAX];
@@ -516,7 +512,7 @@ posix_set_file_contents (xlator_t *this, const char *path, data_pair_t *trav,
         /* XXX: does not handle assigning GFID to created files */
         return -1;
 
-        key = &(trav->key[15]);
+        key = &(keyp[15]);
         sprintf (real_path, "%s/%s", path, key);
 
         if (flags & XATTR_REPLACE) {
@@ -528,9 +524,8 @@ posix_set_file_contents (xlator_t *this, const char *path, data_pair_t *trav,
                         goto create;
                 }
 
-                if (trav->value->len) {
-                        ret = write (file_fd, trav->value->data,
-                                     trav->value->len);
+                if (value->len) {
+                        ret = write (file_fd, value->data, value->len);
                         if (ret == -1) {
                                 op_ret = -errno;
                                 gf_log (this->name, GF_LOG_ERROR,
@@ -562,7 +557,7 @@ posix_set_file_contents (xlator_t *this, const char *path, data_pair_t *trav,
                         goto out;
                 }
 
-                ret = write (file_fd, trav->value->data, trav->value->len);
+                ret = write (file_fd, value->data, value->len);
                 if (ret == -1) {
                         op_ret = -errno;
                         gf_log (this->name, GF_LOG_ERROR,
@@ -646,8 +641,7 @@ posix_get_file_contents (xlator_t *this, uuid_t pargfid,
 
 out:
         if (op_ret < 0) {
-                if (*contents)
-                        GF_FREE (*contents);
+                GF_FREE (*contents);
                 if (file_fd != -1)
                         close (file_fd);
         }
@@ -659,17 +653,17 @@ static int gf_xattr_enotsup_log;
 
 int
 posix_handle_pair (xlator_t *this, const char *real_path,
-                   data_pair_t *trav, int flags)
+                   char *key, data_t *value, int flags)
 {
         int sys_ret = -1;
         int ret     = 0;
 
-        if (ZR_FILE_CONTENT_REQUEST(trav->key)) {
-                ret = posix_set_file_contents (this, real_path, trav, flags);
+        if (ZR_FILE_CONTENT_REQUEST(key)) {
+                ret = posix_set_file_contents (this, real_path, key, value,
+                                               flags);
         } else {
-                sys_ret = sys_lsetxattr (real_path, trav->key,
-                                         trav->value->data,
-                                         trav->value->len, flags);
+                sys_ret = sys_lsetxattr (real_path, key, value->data,
+                                         value->len, flags);
 
                 if (sys_ret < 0) {
                         if (errno == ENOTSUP) {
@@ -679,12 +673,13 @@ posix_handle_pair (xlator_t *this, const char *real_path,
                                                     "supported (try remounting "
                                                     "brick with 'user_xattr' "
                                                     "flag)");
-                        } else if (errno == ENOENT &&
-                                   !posix_special_xattr (marker_xattrs,
-                                                         trav->key)) {
-                                gf_log (this->name, GF_LOG_ERROR,
-                                        "setxattr on %s failed: %s", real_path,
-                                        strerror (errno));
+                        } else if (errno == ENOENT) {
+                                if (!posix_special_xattr (marker_xattrs,
+                                                          key)) {
+                                        gf_log (this->name, GF_LOG_ERROR,
+                                                "setxattr on %s failed: %s",
+                                                real_path, strerror (errno));
+                                }
                         } else {
 
 #ifdef GF_DARWIN_HOST_OS
@@ -692,12 +687,12 @@ posix_handle_pair (xlator_t *this, const char *real_path,
                                         ((errno == EINVAL) ?
                                          GF_LOG_DEBUG : GF_LOG_ERROR),
                                         "%s: key:%s error:%s",
-                                        real_path, trav->key,
+                                        real_path, key,
                                         strerror (errno));
 #else /* ! DARWIN */
                                 gf_log (this->name, GF_LOG_ERROR,
                                         "%s: key:%s error:%s",
-                                        real_path, trav->key,
+                                        real_path, key,
                                         strerror (errno));
 #endif /* DARWIN */
                         }
@@ -712,13 +707,13 @@ out:
 
 int
 posix_fhandle_pair (xlator_t *this, int fd,
-                    data_pair_t *trav, int flags)
+                    char *key, data_t *value, int flags)
 {
         int sys_ret = -1;
         int ret     = 0;
 
-        sys_ret = sys_fsetxattr (fd, trav->key, trav->value->data,
-                                 trav->value->len, flags);
+        sys_ret = sys_fsetxattr (fd, key, value->data,
+                                 value->len, flags);
 
         if (sys_ret < 0) {
                 if (errno == ENOTSUP) {
@@ -739,13 +734,11 @@ posix_fhandle_pair (xlator_t *this, int fd,
                                 ((errno == EINVAL) ?
                                  GF_LOG_DEBUG : GF_LOG_ERROR),
                                 "fd=%d: key:%s error:%s",
-                                fd, trav->key,
-                                strerror (errno));
+                                fd, key, strerror (errno));
 #else /* ! DARWIN */
                         gf_log (this->name, GF_LOG_ERROR,
                                 "fd=%d: key:%s error:%s",
-                                fd, trav->key,
-                                strerror (errno));
+                                fd, key, strerror (errno));
 #endif /* DARWIN */
                 }
 
@@ -849,7 +842,7 @@ posix_janitor_thread_proc (void *data)
                 time (&now);
                 if ((now - priv->last_landfill_check) > priv->janitor_sleep_duration) {
                         gf_log (this->name, GF_LOG_TRACE,
-                                "janitor cleaning out /" GF_REPLICATE_TRASH_DIR);
+                                "janitor cleaning out %s", priv->trash_path);
 
                         nftw (priv->trash_path,
                               janitor_walker,
@@ -940,37 +933,47 @@ out:
         return ret;
 }
 
+static int
+_handle_entry_create_keyvalue_pair (dict_t *d, char *k, data_t *v,
+                                    void *tmp)
+{
+        int                   ret = -1;
+        posix_xattr_filler_t *filler = NULL;
+
+        filler = tmp;
+
+        if (!strcmp (GFID_XATTR_KEY, k) ||
+            !strcmp ("gfid-req", k) ||
+            !strcmp ("system.posix_acl_default", k) ||
+            !strcmp ("system.posix_acl_access", k) ||
+            ZR_FILE_CONTENT_REQUEST(k)) {
+                return 0;
+        }
+
+        ret = posix_handle_pair (filler->this, filler->real_path, k, v,
+                                 XATTR_CREATE);
+        if (ret < 0) {
+                errno = -ret;
+                return -1;
+        }
+        return 0;
+}
+
 int
 posix_entry_create_xattr_set (xlator_t *this, const char *path,
                              dict_t *dict)
 {
-        data_pair_t *trav = NULL;
         int ret = -1;
 
+        posix_xattr_filler_t filler = {0,};
+
         if (!dict)
                 goto out;
 
-        trav = dict->members_list;
-        while (trav) {
-                if (!strcmp (GFID_XATTR_KEY, trav->key) ||
-                    !strcmp ("gfid-req", trav->key) ||
-                    !strcmp ("system.posix_acl_default", trav->key) ||
-                    !strcmp ("system.posix_acl_access", trav->key) ||
-                    ZR_FILE_CONTENT_REQUEST(trav->key)) {
-                        trav = trav->next;
-                        continue;
-                }
+        filler.this = this;
+        filler.real_path = path;
 
-                ret = posix_handle_pair (this, path, trav, XATTR_CREATE);
-                if (ret < 0) {
-                        errno = -ret;
-                        ret = -1;
-                        goto out;
-                }
-                trav = trav->next;
-        }
-
-        ret = 0;
+        ret = dict_foreach (dict, _handle_entry_create_keyvalue_pair, &filler);
 
 out:
         return ret;
@@ -994,7 +997,7 @@ __posix_fd_ctx_get (fd_t *fd, xlator_t *this, struct posix_fd **pfd_p)
                 goto out;
         }
 
-        if (fd->pid != -1)
+        if (!fd_is_anonymous(fd))
                 /* anonymous fd */
                 goto out;
 
@@ -1060,35 +1063,3 @@ posix_fd_ctx_get (fd_t *fd, xlator_t *this, struct posix_fd **pfd)
 
         return ret;
 }
-
-
-int
-posix_fd_ctx_get_off (fd_t *fd, xlator_t *this, struct posix_fd **pfd,
-                      off_t offset)
-{
-        int   ret;
-        int   flags;
-
-        LOCK (&fd->inode->lock);
-        {
-                ret = __posix_fd_ctx_get (fd, this, pfd);
-                if (ret)
-                        goto unlock;
-
-                if ((offset & 0xfff) && (*pfd)->odirect) {
-                        flags = fcntl ((*pfd)->fd, F_GETFL);
-                        ret = fcntl ((*pfd)->fd, F_SETFL, (flags & (~O_DIRECT)));
-                        (*pfd)->odirect = 0;
-                }
-
-                if (((offset & 0xfff) == 0) && (!(*pfd)->odirect)) {
-                        flags = fcntl ((*pfd)->fd, F_GETFL);
-                        ret = fcntl ((*pfd)->fd, F_SETFL, (flags | O_DIRECT));
-                        (*pfd)->odirect = 1;
-                }
-        }
-unlock:
-        UNLOCK (&fd->inode->lock);
-
-        return ret;
-}
