@@ -1,23 +1,13 @@
 /*
-  Copyright (c) 2011 Gluster, Inc. <http://www.gluster.com>
-  This file is part of GlusterFS.
+   Copyright (c) 2011-2012 Red Hat, Inc. <http://www.redhat.com>
+   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+   This file is licensed to you under your choice of the GNU Lesser
+   General Public License, version 3 or any later version (LGPLv3 or
+   later), or the GNU General Public License, version 2 (GPLv2), in all
+   cases as published by the Free Software Foundation.
 */
 
-
 #ifndef _POSIX_ACL_H
 #define _POSIX_ACL_H
 
@@ -27,6 +17,7 @@
 #include "common-utils.h"
 #include "byte-order.h"
 
+#define POSIX_ACL_MINIMAL_ACE_COUNT 3
 
 #define POSIX_ACL_READ                (0x04)
 #define POSIX_ACL_WRITE               (0x02)
@@ -53,7 +44,7 @@ struct posix_ace {
 struct posix_acl {
         int               refcnt;
         int               count;
-        struct posix_ace  entries[0];
+        struct posix_ace  entries[];
 };
 
 
